


;+
; NAME:
;       CUBE2RSS
;
; PURPOSE:
;
;       Generates a RSS file and a associated position table from a 3D cube FITS file.
;
;
; EXPLANATION:
;
;       The script transforms a 3D cube FITS file into a Row-Stacked-Spectra FITS
;       file and generates the (continuous) position table of the RSS according
;       to the original geometry of the cube. The size, resolution and reference
;       pixel of the Field-of-View are deduced from the 3D FITS header
;       (e.g. NAXIS1, CDELT1, CRPIX1, etc.). The geometry which is assumed
;       defines the reference pixel (1,1) at the bottom-right corner of the XY
;       projection, and the (N,M) pixel at the top-left of the image. 
;
;       Supported geometries include: 
;
;       (1) the pixel index increases horizontally, from right to left on
;       the same row (e.g. PPAK, CALIFA)
;
;       (2) the index increases vertically, from bottom to top on the same
;       column (e.g. VIMOS)
;
;       The default is set to 1, other geometries can be easily incorporated.
;
;       If the FITS header includes a reference pixel and WCS, this information
;       is considered in order to create the associated position table (in the
;       PINGSoft 'view_rss' format). 
;       If the /CALIFA keyword is set, the program needs only the name of the
;       input 'OBJECT.rscube.fits' file in order to generates the RSS FITS with
;       all the appropriate parameters and the position table in the standard
;       North-East configuration.
;
;
; CALLING SEQUENCE:
;       
;       cube2rss, '3D_cube.fits' [, out_str, PREFIX='prefix', GEOMETRY=geometry, TRIM_RED=trim_red, /PPAK, /CALIFA, /SILENT ]
;                       
;
; INPUTS:
;  
;        '3D_cube.fits':  3D cube FITS input file
;
;
; OPTIONAL KEYWORDS:
;       
;                PREFIX:  String with the prefix of the output files, if not set the prefix is
;                         taken from the input FITS file.
;                 
;              GEOMETRY:  Geometry of the input 3D cube, it is assumed that the
;                         reference pixel (1,1) at the bottom-right corner of
;                         the XY projection, and the (N,M) pixel at the top-left
;                         of the image. Supported geometries include: (1) the
;                         pixel index increases from horizontally from right to
;                         left on the same row (e.g. PPAK); (2) the index
;                         increases vertically, from bottom to top on the same
;                         column (e.g. VIMOS), the default is set to 1. 
;
;              TRIM_RED:  Number of pixels to be removed at the red end of each
;                         spectrum (useful for cosmetic purposes).
;
;                 /PPAK:  If set, creates the position table for the PPAK instrument in
;                         the right standard North-East orientation.
;
;               /CALIFA:  If set, the code assumes that the name of the input 3D
;                         file is 'OBJECT.rscube.fits' and generates the RSS
;                         FITS with all the appropriate parameters and the
;                         position table in the standard North-East
;                         configuration to use with PINGSoft. In this case, the
;                         outputs are: 'OBJECT.cube2rss.fits' and
;                         'OBJECT.cube2rss.pt.txt'
;
; OUTPUTS:
;
;       A RSS FITS and position table files generated from the 3D cube FITS
;
;              RSS file:   PREFIX_cube2rss.fits  
;        Position table:   PREFIX_cube2rss.txt
;
;              RSS file:   OBJECT.cube2rss.fits    (if /CALIFA is set)
;        Position table:   OBJECT.cube2rss.pt.txt
;
;
; EXAMPLES:
;
;       cube2rss, 'OBJECT_3Dcube.fits', prefix='OBJECT', geometry=2
;
;       cube2rss, 'OBJECT.rscube.fits', /CALIFA
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-

pro cube2rss, infile, out_str, $
              GEOMETRY=geometry, PREFIX=prefix, PPAK=ppak, CALIFA=califa, TRIM_RED=trim_red, SILENT=silent, BINNING=binning

compile_opt idl2, hidden
on_error, 2

if n_params() lt 1 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "    cube2rss, 'input_3Dcube.fits' [, GEOMETRY=geometry, PREFIX='prefix', TRIM_RED=trim_red, "
    print, '                                   /PPAK, /CALIFA, /SILENT ]'
    print, ''
    print, "   'input_3Dcube.fits':  3D cube FITS input file."
    print, ''
    print, 'OPTIONAL KEYWORDS:'
    print, ''      
    print, '            PREFIX:  String with the prefix of the output files.'
    print, ''
    print, '          GEOMETRY:  Geometry of the input 3D cube, it is assumed that the'
    print, '                     reference pixel (1,1) at the bottom-right corner of'
    print, '                     the XY projection, and the (N,M) pixel at the top-left'
    print, '                     of the image. Supported geometries include: '
    print, '                     (1) The pixel index increases from horizontally from right'
    print, '                         to left on the same row (e.g. PPAK)print'
    print, '                     (2) The index increases vertically, from bottom to top on the same'
    print, '                         column (e.g. VIMOS), the default is set to 1. '
    print, ''
    print, '          TRIM_RED:  Number of pixels to be removed at the red end of each'
    print, '                     spectrum (useful for cosmetic purposes).'
    print, ''
    print, '             /PPAK:  If set, creates the position table for the PPAK instrument in'
    print, '                     the right standard North-East orientation.'
    print, ''
    return
endif


if not keyword_set(geometry) then geometry = 1

if geometry lt 1 or geometry gt 2 then begin
    print, ''
    print, " ERROR:  GEOMETRY should be 1 or 2 (see the documentation)"
    print, ''
    return
endif


if not keyword_set(ppak) then ppak = 0


if keyword_set(prefix) then begin
   out_rss = prefix+'.cube2rss.fits'
   out_pt  = prefix+'.cube2rss.pt.txt'
   object = prefix
endif


if not keyword_set(prefix) then begin
   tmp0 = strmid(infile,strpos(infile,'/',/reverse_search)+1)
   tmp1 = strsplit(tmp0,'.rscube.fits.gz',/extract,/regex)
   tmp2 = strsplit(tmp1,'.rscube.fits',/extract,/regex)
   tmp3 = strsplit(tmp2,'.rss.fits.gz',/extract,/regex)
   tmp4 = strsplit(tmp3,'.rss.fits',/extract,/regex)
   tmp5 = strsplit(tmp4,'.fits.gz',/extract,/regex)
   tmp6 = strsplit(tmp5,'.fits',/extract,/regex)
   prefix = tmp6
   out_rss = prefix+'.cube2rss.fits'
   out_pt  = prefix+'.cube2rss.pt.txt'
endif



; CALIFA
if keyword_set(califa) then begin
       ppak = 1
   geometry = 1
   object = 'CALIFA: '+prefix
endif


; Reading data
  data = readfits(infile,hdr,/silent)
 crval = fxpar(hdr,'CRVAL3')
 cdelt = fxpar(hdr,'CDELT3')
 crpix = fxpar(hdr,'CRPIX3')
pixels = fxpar(hdr,'NAXIS3')


; Dimensions
size = size(data)
  xx = size[1]
  yy = size[2]
  zz = size[3]


; New data
rows = xx*yy

if not keyword_set(trim_ref) then rss = fltarr(zz,rows) 

if keyword_set(trim_red) then begin
   last = zz-trim_red
   rss = fltarr(last,rows)
endif


; New vector
id = long(0)

for j=0l, yy-1 do begin

    for i=0l, xx-1 do begin

       array = data[i,j,*]
       array = transpose(array)

       ; NaN and Inf step 1
       mask = finite(array)
       array = array*mask
       nan = where(array ne array)
       if n_elements(nan) gt 1 then array[nan] = 0.
       if n_elements(nan) eq 1 and nan[0] ge 0 then array[nan] = 0.
       ; NaN and Inf step 2
       inf = where(array lt -1e10 or array gt 1e10)
       if n_elements(inf) gt 1 then array[inf] = 0.
       if n_elements(inf) eq 1 and inf[0] ge 0 then array[inf] = 0.
       
       if not keyword_set(trim_red) then begin
          rss[*,id] = array
       endif else begin
          ok = indgen(last)
          rss[*,id] = array[ok]
       endelse

       id = id+1

    endfor

 endfor



; Obtaining extra FITS header info

cdelt1 = fxpar(hdr,'CDELT1')
cdelt2 = fxpar(hdr,'CDELT2')
crpix1 = fxpar(hdr,'CRPIX1')
crpix2 = fxpar(hdr,'CRPIX2')
naxis1 = fxpar(hdr,'NAXIS1')
naxis2 = fxpar(hdr,'NAXIS2')

    RA = fxpar(hdr,'CRVAL1')
   Dec = fxpar(hdr,'CRVAL2')

if cdelt1 eq 0 or cdelt2 eq 0 then resolution = 1 else resolution = cdelt1

if keyword_set(binning) then begin
   if binning gt 1 then resolution=binning else resolution=1
endif

if crpix1 ne 0 then  RA_off = double(crpix1*resolution) 
if crpix2 ne 0 then Dec_off = double(crpix2*resolution)

if  RA_off eq 0 then  RA_off = xx*resolution*0.5
if Dec_off eq 0 then Dec_off = yy*resolution*0.5


; Changing FITS header

rss_pixels = sctrim(fxpar(hdr,'NAXIS3'))
  rss_rows = sctrim(rows)

crval = fxpar(hdr,'CRVAL3')
cdelt = fxpar(hdr,'CDELT3')
crpix = fxpar(hdr,'CRPIX3')
fxaddpar, hdr, 'CRVAL1', crval, ' updated by PINGSoft'
fxaddpar, hdr, 'CDELT1', cdelt, ' updated by PINGSoft'
fxaddpar, hdr, 'CRPIX1', crpix, ' updated by PINGSoft'

fxaddpar, hdr, 'XYOFF', 'xyoff=['+sctrim(crpix1)+','+sctrim(crpix2)+']', ' [CRPIX1,CRPIX2] original cube', after='CRVAL1'
fxaddpar, hdr, 'CSIZE', 'csize=['+sctrim(naxis1)+','+sctrim(naxis2)+']', ' [NAXIS1,NAXIS2] original cube', after='CRVAL1'
fxaddpar, hdr, 'CDELT', 'cdelt=['+sctrim(cdelt1)+','+sctrim(cdelt2)+']', ' [CDELT1,CDELT2] original cube', after='CRVAL1'

sxdelpar, hdr, 'CRVAL2'
sxdelpar, hdr, 'CRVAL3'
sxdelpar, hdr, 'CRPIX2'
sxdelpar, hdr, 'CRPIX3'
sxdelpar, hdr, 'CDELT2'
sxdelpar, hdr, 'CDELT3'

sxdelpar, hdr, 'DISPAXIS'
sxdelpar, hdr, 'WCSDIM'


cd2 = fxpar(hdr,'CD2_2')
stype = size(cd2,/tname)
if stype eq 'STRING' then begin
   com = 'cd2 = double('+cd2+')'
   eva = execute(com)
endif
if cd2 ge 0 then $
   fxaddpar, hdr,  'CD2_2', cd2, ' updated by PINGSoft' else $
      fxaddpar, hdr,  'CD2_2', -cd2, ' updated by PINGSoft' 


; Writing RSS
out_rss = sctrim(out_rss)
writefits, out_rss, rss, hdr


; Updating WCS and FITS header
if RA ne 0 and Dec ne 0 then write_wcs, out_rss, RA, Dec


; Creating PTable
if not keyword_set(ppak) then rectangular_pt, xx, yy, resolution, geometry, xpt, ypt, off, $
                   X_OFF=RA_off, Y_OFF=Dec_off, OUT_PT=out_pt

if keyword_set(ppak) then rectangular_pt, xx, yy, resolution, geometry, xpt, ypt, off, $
                X_OFF=RA_off, Y_OFF=Dec_off, OUT_PT=out_pt, /PPAK



; Output

; Structure with strings

out_str = {$
          rss: out_rss ,$
          pt:  out_pt   $
          }

if not keyword_set(silent) then begin

   print, ''
   print, ' PINGSoft: cube2rss'   
   print, '----------------------------------------------'
   print, ''
   print, '   RSS file created:  ', out_rss
   print, ''
   print, '   Number of pixels:  ', rss_pixels
   print, '     Number of rows:  ', rss_rows
   print, ''
   print, ''
   print, ' New Position Table:  ', out_pt
   print, ''
   print, '          RA length:  '+sctrim(xx, format='(f6.2)')
   print, '         Dec length:  '+sctrim(yy, format='(f6.2)')
   print, ''
   print, '         Resolution:  '+sctrim(resolution, format='(f4.2)')
   print, ''
   print, '          RA offset:  '+sctrim(off[0], format='(f6.2)')
   print, '         Dec offset:  '+sctrim(off[1], format='(f6.2)')
   print, ''
   print, '----------------------------------------------'
   print, ''

endif

if file_test('rectangular.pt.txt') eq 1 then $
   spawn, 'rm -f rectangular.pt.txt'

end
