

function acosh, x
  xx = 1.0 > x
  y = alog( xx + sqrt( xx^2 - 1.0) )
  return, y
end



pro extract_cone, infile, x0, y0, theta, PA, out_str, EPSILON=epsilon, PT=pt, PREFIX=prefix, $
                  FILTER=filter, BAND=band, DRAW=draw, CT=ct, FONT=font, PLOT=plot, PS=ps,  $
                  CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
                  SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
                  _EXTRA=extra


;+
; NAME:
;       EXTRACT_CONE
;
; PURPOSE:
;
;       Extracts the spectra within a region defined by a hyperbolic cone.
;
; EXPLANATION:
;
;       The user can define a circular cone of any physical dimensions
;       (in arcsec) centered at a given position from the reference point.
;       The program then extracts the spaxels which centers fall within
;       the cone, generating a new RSS and position table files and the
;       integrated spectrum in ASCII and FITS format. By default,  the program
;       opens a FoV visualisation as VIEW_3D, showing the simulated cone. 
;       If the /PS keyword is set, an encapsulated Postscript file of the FoV is
;       generated. The /PLOT and /PS keywords are mutually exclusive.
;       Similar display options as in VIEW_3D.
;
;
; For CALLING SEQUENCE, INPUTS and OPTIONAL KEYWORDS enter:
;
;        IDL> extract_cone
;
;
; OUTPUTS:
;
;      If PREFIX is set:
;
;              Extracted RSS:   PREFIX_rss.fits
;             Position table:   PREFIX_rss.pt.txt
;           Integrated ASCII:   PREFIX_integ.txt
;                       FITS:   PREFIX_integ.fits
;                 Postscript:   PREFIX_integ.eps
;             Extraction FoV:   PREFIX_FoV.eps                    (only if /PS is set)
;
;
;      If PREFIX is NOT set:
;
;             Extracted RSS:   cone_x0-Y0_theta_PA_rss.fits
;            Position table:   cone_x0-Y0_theta_PA_rss.pt.txt
;          Integrated ASCII:   cone_X0-Y0_theta_PA_integ.txt
;                      FITS:   cone_X0-Y0_theta_PA_integ.fits
;                Postscript:   cone_X0-Y0_theta_PA_integ.eps
;            Extraction FoV:   cone_X0-Y0_theta_PA_FoV.eps        (only if /PS is set)
;
;
; COMMENTS
;
;       Only those spaxels which centers lie within the cone are extracted,
;       "fractional spaxels" are not considered. 
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-

compile_opt idl2, hidden
on_error, 2


if n_params() lt 5 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "       extract_cone, 'OBJECT.fits', x0, y0, theta, PA  [, EPSILON=epsilon,"
    print, "                     [, OUT.str, PREFIX='prefix', PT='PosTable.txt', EXTENSION=extension, "
    print, "                     /PLOT, /PS, FONT=font, _EXTRA={extra},  "
    print, '                     (+ all visual options of VIEW_3D) ]'
    print, ''
    print, ''
    print, "      'OBJECT.fits':  String of the wavelength calibrated 3D cube or RSS FITS file."
    print, ''
    print, '              x0/y0:  RA/Dec offsets (in arcsec) of the position of vertex of the cone to be simulated.'
    print, ''
    print, '              theta:  Angle between the major axis and either asymptote of the simulated cone.'
    print, ''
    print, '                 PA:  Position angle of the major axis of the cone, measured counterclockwise from the North.'
    print, ''
    print, ''
    print, '  OPTIONAL KEYWORDS:'
    print, ''
    print, '            EPSILON:  Floating parameter greater-equal than 1 controling the curvature of the vertex.'
    print, ''
    print, "            OUT.str:  Name of the optional output structure."
    print, ''
    print, "    PT='Ptable.txt':  Name of the position table in the case of an RSS file"
    print, '                      in ASCII format (North-East configuration).'
    print, '                      NOTE: compulsory if not included in the default instruments/setups'
    print, "                      or when the name is not in the 'OBJECT.pt.txt' format."
    print, ''
    print, "             PREFIX:  String with the prefix of the output files. Default: input positions"
    print, ''
    print, '          EXTENSION:  Non-negative scalar integer specifying the FITS extension to read.'
    print, '                      For example, specify EXTENSION = 1 to read the first FITS extension.'
    print, ''
    print, '              /PLOT:  Display the visualisation and the simulated aperture, this is the default.'
    print, ''
    print, '                /PS:  Writes an encapsulated Postscript file of the spaxels visualisation'
    print, '                      and the simulated aperture.'
    print, ''
    print, '               FONT:  Postscript IDL font to be used when /PS is set. Default: 12 (Helvetica)'
    print, ''
    print, "             _EXTRA:  Structure with the _EXTRA tags for user's defined graphics output,"
    print, "                      e.g. _EXTRA={title:'IFS cube'}"
    print, ''
    print, '   + all visual options of VIEW_3D  (valid only if /PLOT or /PS are set)'
    print, ''
    return
endif


;--------------------------  Initial tests

infile = sctrim(infile)

if file_test(infile) eq 0 then begin
    print, ''
    print, 'ERROR: The file:  "'+infile+'"  does not exist!'
    print, ''
    return
endif


;--------------------------  Routine properties


if keyword_set(plot) and keyword_set(ps) then begin
   print, ''
   print, ' WARNING: keywords PLOT and PS cannot be set at the same time'
   print, ''
endif

if not keyword_set(plot) then plot=1
if keyword_set(ps) then plot=0


if not keyword_set(prefix) then begin

   t1 = sctrim(x0, format='(f7.1)')
   t2 = sctrim(y0, format='(f7.1)')
   t3 = sctrim(theta)
   t4 = sctrim(PA)

   sprefix  = 'cone_x'+t1+'_y'+t2+'_theta'+t3+'_PA'+t4

   ps_file = sprefix+'_FoV.eps' 

endif

if keyword_set(prefix) then ps_file = prefix+'_FoV.eps'



;--------------------------- PA in radians and astro conventions

; correction for RA convenction (inverse sign, -90)
ang = -(PA-90)/!RADEG


;--------------------------  Hyperbola definition

theta = theta*!dtor

if not keyword_set(epsilon) then a = 1

if keyword_set(epsilon) then begin
   if epsilon le 0 then begin
      print, ''
      print, 'WARNING: the EPSILON parameter must be positive, assuming EPSILON=1'
      print, ''
      a = 1
      goto, epsilon_set
   endif
   a = epsilon
   epsilon_set:
endif

b = sqrt( a^2 * ( (1/cos(theta)^2) -1 ) )


; Shifting Ptable

x0 = x0 - a*cos(ang)
y0 = y0 - a*sin(ang)  


;--------------------------  PINGSoft core

pingsoft_core, infile, pings, core, ps_file, x0, y0, /SHIFT, $
               PT=pt, PREFIX=prefix,  $
               MIN_FLUX=min_flux, MAX_FLUX=max_flux, LMIN=lmin, LMAX=lmax, $
               FILTER=filter, BAND=band, CT=ct, DRAW=draw, PS=ps, FONT=font, PLOT=plot, $
               CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
               SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
               EFF_RAD=eff_rad, NOEXTRACT=noextract, _EXTRA=extra

; Variables from structure
tag = get_tags(core)
for k=0, n_elements(tag)-1 do begin
   dumb = strmid(tag[k],1)
   com = execute(dumb+' = core.'+dumb)
endfor

npoints = 200

xlength = abs(max(x)-min(x))
ylength = abs(max(y)-min(y))
if xlength ge ylength then length = xlength else length = ylength


mphi = acosh(length/a)
phi = -mphi + 2*mphi*findgen(npoints+1)/npoints

xm = a * cosh(phi)
ym = b * sinh(phi)

xn = xm * cos(ang) - ym * sin(ang)
yn = xm * sin(ang) + ym * cos(ang)

if PA le  90               then nn = where(xn le max(x) and yn le max(y))
if PA gt  90 and PA le 180 then nn = where(xn le max(x) and yn ge min(y))
if PA gt 180 and PA le 270 then nn = where(xn ge min(x) and yn ge min(y))
if PA gt 270 and PA le 359 then nn = where(xn ge min(x) and yn le max(y))

xh = xn[nn]
yh = yn[nn]


;--------------------------  Fibers subsample

fact = 0.1
xmin = min(xh) - (max(xh)-min(xh))*fact
xmax = max(xh) + (max(xh)-min(xh))*fact
ymin = min(yh) - (max(yh)-min(yh))*fact
ymax = max(yh) + (max(yh)-min(yh))*fact

index = where(x ge xmin and x le xmax and y ge ymin and y le ymax)
id_test = id[index]-1 ; IDL index 
 x_test = x[index]
 y_test = y[index]

loadct, 39,/silent
;plots, x_test, y_test, psym=4, color=150, symsize=0.5


hh = where(xn ge min(x) and xn le max(x) and yn ge min(y) and yn le max(y))
xp = xn[hh]
yp = yn[hh]



;--------------------------  Main loop

print, ''
print, 'Extracting spectra...'
id_in = 0l
points_in = 0l

for n=0l, n_elements(id_test)-1  do begin
        
    xi =  x_test[n]  ; RA definition
    yi =  y_test[n]

    ; rotating subsample to correct for PA
    xx =  (xi * cos(ang)) + yi * sin(ang)
    yy = -(xi * sin(ang)) + yi * cos(ang)

    if xx lt min(xn) then goto, next

    xphi = acosh(xx/a)

    y1 = b * sinh(xphi)
    y2 = b * sinh(-xphi)

    if yy gt y1 or yy lt y2 then goto, next

    ; Spaxels within cone 
    if keyword_set(plot) then plots, xi, yi, psym=1, color=190, symsize=0.4

    id_ok = id_test[n]
    id_in = [id_in, id_ok]
    points_in = points_in+1
    
    next:
    
endfor

if keyword_set(plot) then plots, xp, yp, thick=3, color=250
if keyword_set(ps) then plots, xp, yp, thick=7, color=250


id_in = id_in[1:*]

if keyword_set(ps) then pingsoft_psoff

extract = id_in


;-------------------------- Extracting spectra

all_flux = fltarr(pixels)
rss_data = fltarr(pixels,n_elements(id_in))


for i=0l, n_elements(id_in)-1 do begin
    id = id_in[i]
    flux  = data[*,id]
    all_flux = all_flux + flux
    rss_data[*,i]=flux
endfor

integ_flux = all_flux

nice_x = lambda
nice_y = integ_flux

xmin = min(nice_x) & xmax = max(nice_x)
ymin = min(nice_y) & ymax = max(nice_y)*1.2

loadct, 1, /silent
if keyword_set(plot) then begin
    window, 2
    plot, [0], /nodata, xr=[xmin-80,xmax+80], yr=[ymin,ymax], xs=1, ys=1, $
          xthick=pings.wthick, ythick=pings.wthick, position=pings.pos2, charsize=pings.csize, $
          title='Integrated flux', xtitle='Wavelength', ytitle='Flux'
    loadct, 39, /silent
    oplot, lambda, integ_flux, color=250
endif


;-------------------------- Outputs
 
; Integrated flux ASCII
if not keyword_set(prefix) then $
  out_integ = sprefix+'_integ.txt' else $
  out_integ = prefix+'_integ.txt'

openw,3,out_integ
for k=0, n_elements(lambda)-1 do printf,3, k+1, lambda[k], integ_flux[k]
close, 3

; Integrated flux FITS
if not keyword_set(prefix) then $
  out_fits = sprefix+'_integ.fits' else $
  out_fits = prefix+'_integ.fits'

writefits, out_fits, integ_flux, hdr

; RSS FITS file
if not keyword_set(prefix) then $
  out_rss = sprefix+'_rss.fits' else $
  out_rss = prefix+'_rss.fits'

writefits, out_rss, rss_data, hdr

; Position table
if not keyword_set(prefix) then $
  out_file = sprefix+'_rss.pt.txt' else $
  out_file = prefix+'_rss.pt.txt'

get_new_pt, id_in, pos_table, 'tmp_shift.pt.txt', /silent

; correcting by hyperbola semi-major axis
shift_ptable,  'tmp_shift.pt.txt', out_file, a*cos(ang), a*sin(ang),  /silent


; PS extracted spectrum
if not keyword_set(prefix) then $
   out_ps =  sprefix+'_integ.eps'  else $
   out_ps =  prefix+'_integ.eps'
pingsoft_pson, out_ps, 20, 16, font=psfont, axis_thick=pings.axis, encapsulate=1
loadct, 1, /silent
xvar = lambda
yvar = integ_flux
plot, [0], /nodata, xr=[min(xvar)-80,max(xvar)+80], yr=[min(yvar), max(yvar)*1.2], $
      xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, pos=pings.pos2, charsize=pings.csize*1.2, $
      xtitle='Wavelength', ytitle='Flux', title='Integrated spectrum'
oplot, xvar, yvar, color=100
pingsoft_psoff



; Saving structure
out_str = { $
          newrss:  out_rss ,$
          lambda:  lambda  ,$
          finteg:  integ_flux ,$
          indices: extract $
          }

print, ''
print, ' PINGSoft'
print, ' ===================='
print, ''
print, ' Files written '
print, ''
print, '       Extracted RSS:   ', out_rss
print, '      Position table:   ', out_file
print, '    Integrated ASCII:   ', out_integ
print, '                FITS:   ', out_fits
print, '          Postscript:   ', out_ps

if keyword_set(ps) then $
print, '      Extraction FoV:   ', ps_file
print, ''

if n_params() eq 6 then begin
print, ''
print, ' IDL structure saved:   '
print, ''
help, out_str, /struc
print, ''
endif

if file_test('tmp.pt.txt') eq 1 then spawn, 'rm -f tmp.pt.txt'
if file_test('tmp_3D.pt.txt') eq 1 then spawn, 'rm -f tmp_3D.pt.txt'


end
