

pro extract_filter, infile, filter, out_str, USER=user, SUBTRACT=subtract,  $
                    PT=pt, PREFIX=prefix, EXTENSION=extension, $
                    BAND=band, CT=ct, DRAW=draw,  PS=ps, FONT=font, $
                    CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
                    SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
                    SILENT=silent,  WIDGET=widget, _EXTRA=extra


;+
; NAME:
;       EXTRACT_FILTER
;
; PURPOSE:
;
;       Generates a 2D FITS image after convolving the 3D data with a narrow or
;       broad-band filter.
;
; EXPLANATION:
;
;       The command creates a 2D FITS image slice of the IFS data using either
;       one of the PINGSoft default transmission curves or a user's
;       defined filter. It works only with continuous-regular grids. By default,
;       the program opens a FoV visualisation as VIEW_3D, showing the output
;       image scaling and in a second window, it displays the integrated spectrum
;       of the IFS data, showing the convolved band. The central wavelength of
;       the filter can be shifted to any within the spectral range using the
;       BAND keyword. For narrow band filters, the continuum at +/- 100 AA can
;       be subtracted using the /SUBTRACT option. The user can supply its own
;       filter by setting the USER keyword, with the string of the ASCII file
;       containing the transmission filter in the same format as the PINGSoft
;       filters. Similar display options as in VIEW_3D.
;
;       NOTE: expect weird orientations/results with RSS data.
;
;
; For CALLING SEQUENCE, INPUTS and OPTIONAL KEYWORDS enter:
;
;        IDL> extract_filter
;
;
; OUTPUTS:
;
;       If PREFIX is set:
;
;          Filter image (FITS):   PREFIX_slice.fits
;              PS file written:   PREFIX_slice.eps        (only if /PS set)
;
;      If not PREFIX set:
;
;          Filter image (FITS):   OBJECT_FILTER.fits
;                           or
;          Filter image (FITS):   OBJECT_FILTER_sub.fits  (if /SUBTRACT set and filter=1,2)
;              PS file written:   OBJECT_FILTER.eps       (only if /PS set)
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-


compile_opt idl2, hidden
on_error, 2

if n_params() lt 1 then begin
    print, ''
    print, " extract_filter,  'OBJECT.fits', filter [, OUT.str, EXTENSION=extension, PREFIX='prefix', "
    print, "                                  BAND=band, USER=user, /SUBTRACT, /PS, PT='Postable.txt', _EXTRA=extra,"
    print, "                                  (+ all graphic options as VIEW_3D) ]" 
    print, ''
    print, ''
    print, "    'OBJECT.fits':  String of the wavelength calibrated 3D cube or RSS FITS file."
    print, ''
    print, '           filter:  Integer value of the default PINGSoft filter to be used (see below)'
    print, ''
    print, ' OPTIONAL KEYWORDS'
    print, ''
    print, "          OUT.str:  Name of the optional output structure."
    print, ''
    print, '             BAND:  Floating value to which the central wavelength of the broad/narrow'
    print, '                    band filter will be shifted.'
    print, ''
    print, '             USER:  String of the ASCII file containing the transmission filter, '
    print, '                    must be in the same format as the PINGSoft filters.'
    print, ''
    print, '        /SUBTRACT:  Subtracts the adjacent continuum at +/- 100 Ang, works only with '
    print, '                    Halpha filters (1,2)'
    print, '        EXTENSION:  Non-negative scalar integer specifying the FITS extension to read.'
    print, '                    For example, specify EXTENSION = 1 to read the first FITS extension.'
    print, ''
    print, "           PREFIX:  String with the prefix of the output files."
    print, ''
    print, '              /PS:  Writes an encapsulated Postscript file of the 2D convolved image.'
    print, ''
    print, '      + all visual options of VIEW_3D'
    print, ''
    print, '  PINGSoft FILTERS: '
    print, ''
    print, '    1: Halpha KPNO-NOAO   | 6547 AA  80-FWHM'
    print, '    2: Halpha CTIO-SINGS  | 6586 AA  20-FWHM'
    print, ''
    print, '    3: B-Johnson (1965)   | 4400 AA'
    print, '    4: V-Johnson          | 5500 AA'
    print, '    5: R-Johnson          | 7000 AA'
    print, ''
    print, '    6: u-SDSS-III         | 3551 AA'
    print, '    7: g-SDSS-III         | 4686 AA'
    print, '    8: r-SDSS-III         | 6166 AA'
    print, '    9: i-SDSS-III         | 7480 AA'
    print, '   10: z-SDSS-III         | 8932 AA'
    print, ''
    print, '   11: B-Bessell (1990)   | 4520 AA'
    print, '   12: V-Bessell          | 5524 AA'
    print, '   13: R-Bessell          | 6535 AA'
    print, ''
    print, '   14: B-CTIO-SINGS       | 4350 AA'
    print, '   15: V-CTIO-SINGS       | 5550 AA'
    print, '   16: R-CTIO-SINGS       | 6500 AA'
    print, ''
    print, '   17: B-KPNO-Harris      | 4357 AA'
    print, '   18: V-KPNO-Harris      | 5375 AA'
    print, '   19: R-KPNO-Harris      | 6425 AA'
    print, ''
    ;---------- Plot
    plot_filters, PS=0
    return

endif


;--------------------------  Initial tests

infile = sctrim(infile)

if file_test(infile) eq 0 then begin
   print, ''
   print, 'ERROR: The file:  "'+infile+'"  does not exist!'
   print, ''
   return
endif

if n_params() eq 1 and keyword_set(user) then filter = 1

if keyword_set(subtract) and filter gt 2 then begin
   print, ''
   print, 'ERROR: /SUBTRACT option only available with Halpha filters [1,2]'
   print, ''
   return
endif



;--------------------------  Strings

tmp0 = strmid(infile,strpos(infile,'/',/reverse_search)+1)
tmp1 = strsplit(tmp0,'.rscube.fits.gz',/extract,/regex)
tmp2 = strsplit(tmp1,'.rscube.fits',/extract,/regex)
tmp3 = strsplit(tmp2,'.rss.fits.gz',/extract,/regex)
tmp4 = strsplit(tmp3,'.rss.fits',/extract,/regex)
tmp5 = strsplit(tmp4,'.fits.gz',/extract,/regex)
tmp6 = strsplit(tmp5,'.fits',/extract,/regex)
object = tmp6


;--------------------------  Reading data

fits = readfits(infile,hdr,/silent)

author = fxpar(hdr,'AUTHOR')
author = string(author)

if keyword_set(extension) then $
   fits = readfits(infile,hdr,ext=extension,/silent) else $
   fits = readfits(infile,hdr,/silent)

size = size(fits)
type = size[0]


;-------------------------- Testing for CALIFA data

califa = 0
ext_bpix = 0
if keyword_set(extension) then goto, ext_set
extend = fxpar(hdr,'EXTEND')

if author eq 'CALIFA Collaboration' then begin

   califa = 1

   ; CALIFA intensity map
   if not keyword_set(ct) then ct = 42

   ; Reading extensions
   if extend eq 1 then begin
      if type eq 2 then begin
         ext_bpix = readfits(infile,ext=2,/silent)
         ext_pt = readfits(infile,hdr_pt,ext=3,/silent)
      endif
      if type eq 3 then ext_bpix = readfits(infile,ext=3,/silent)
   endif

   if extend eq 0 then ext_bpix = -1

   ; Object type
   object = sctrim(fxpar(hdr,'OBJECT'))
   califaid = sctrim(fxpar(hdr,'CALIFAID'),format='(i4)')
   
endif

; CALIFA header without extensions
if n_elements(ext_bpix) eq 1 then begin
   if ext_bpix eq -1 then califa = 0
endif


ext_set:


;--------------------------  Graphic device settings

title_str = object
if califa eq 1 then title_str = 'CALIFA: '+object else title_str = object

pingsoft_param, pings, title_str
pingsoft_display, pings, /silent


;--------------------------  Data type


; Assuming data in arcsec
mas = 0


; RSS type
if type eq 2 then begin

   pixels = size[1]
   rows   = size[2]

   crval = fxpar(hdr,'CRVAL1')
   cdelt = fxpar(hdr,'CDELT1')
   crpix = fxpar(hdr,'CRPIX1')

   ref_ra  = fxpar(hdr,'RA')
   ref_dec = fxpar(hdr,'DEC')

    if crpix le 0 then begin
      print, ''
      print, 'WARNING: value of CRPIX1 in FITS header is 0 or negative, is this OK?'
      print, ''
   endif

   data = fits

   if keyword_set(califa) then begin
      ; Badpix map
      bpix = ext_bpix
      ; Position table
      xpt = tbget(hdr_pt, ext_pt, 1)
      ypt = tbget(hdr_pt, ext_pt, 2)
      shape = fxpar(hdr_pt,'FIBSHAPE')
      pt_sizex = fxpar(hdr_pt,'FIBSIZEX')
      pt_sizey = fxpar(hdr_pt,'FIBSIZEy')
      header = shape+string(pt_sizex)+string(pt_sizey)+'  0'
      openw,1,'tmp_3D.pt.txt'
      printf,1,header
      for i=0,n_elements(xpt)-1 do printf,1, string(i+1), xpt[i], ypt[i], 0
      close,1
      pt = 'tmp_3D.pt.txt'
   endif

   hdr1 = hdr
   hdr2 = hdr
   hdr3 = hdr

   sxdelpar, hdr1, 'CD1_1'
   sxdelpar, hdr1, 'CD2_2'
   sxdelpar, hdr1, 'CTYPE1'

   sxdelpar, hdr1, 'EXTEND'
   sxdelpar, hdr2, 'EXTEND'
   sxdelpar, hdr3, 'EXTEND'

endif



; CUBE type
if type eq 3 then begin

   xx = size[1]
   yy = size[2]

   rows = xx*yy

   crval  = fxpar(hdr,'CRVAL3')
   cdelt  = fxpar(hdr,'CDELT3')
   crpix  = fxpar(hdr,'CRPIX3')
   pixels = fxpar(hdr,'NAXIS3')

   if crpix le 0 then begin
      print, ''
      print, 'WARNING: value of CRPIX3 in FITS header is 0 or negative, is this OK?'
      print, ''
   endif

   data = fltarr(pixels,rows)
   if keyword_set(califa) then bpix = fltarr(pixels,rows)

   ; New vector
   id = long(0)
   for j=0l, yy-1 do begin

      for i=0l, xx-1 do begin

         array = fits[i,j,*]
         array = transpose(array)

         ; NaN and Inf step 1
         mask = finite(array)
         array = array*mask
         nan = where(array ne array)
         if n_elements(nan) gt 1 then array[nan] = 0.
         if n_elements(nan) eq 1 and nan[0] ge 0 then array[nan] = 0.
         ; NaN and Inf step 2
         inf = where(array lt -1e10 or array gt 1e10)
         if n_elements(inf) gt 1 then array[inf] = 0.
         if n_elements(inf) eq 1 and inf[0] ge 0 then array[inf] = 0.
       
         data[*,id] = array
         
         if keyword_set(califa) then begin
            array_bp = ext_bpix[i,j,*]
            array_bp = transpose(array_bp)
            bpix[*,id] = array_bp
         endif

         id = id+1

      endfor
      
   endfor

   cdelt1 = fxpar(hdr,'CDELT1')
   cdelt2 = fxpar(hdr,'CDELT2')
   crpix1 = fxpar(hdr,'CRPIX1')
   crpix2 = fxpar(hdr,'CRPIX2')
   naxis1 = fxpar(hdr,'NAXIS1')
   naxis2 = fxpar(hdr,'NAXIS2')

    RA = fxpar(hdr,'CRVAL1')
   Dec = fxpar(hdr,'CRVAL2')

   ref_ra  = RA
   ref_Dec = Dec

   ; Milli-arcseconds?
   if cdelt1 lt 1 or cdelt2 lt 1 then begin
      num = sctrim(cdelt1, format='(e)')
      exp = strsplit(num[0],'e',/extract,/regex)
      if exp[1] eq '-06' then begin
         cdelt1 = cdelt1 * 1e6
         cdelt2 = cdelt2 * 1e6
         mas = 1
      endif
   endif

   if cdelt1 eq 0 or cdelt2 eq 0 then resolution = 1 else resolution = cdelt1
   
   if crpix1 ne 0 then  RA_off = double(crpix1*resolution) 
   if crpix2 ne 0 then Dec_off = double(crpix2*resolution)
   
   if  RA_off eq 0 then  RA_off = xx*resolution*0.5
   if Dec_off eq 0 then Dec_off = yy*resolution*0.5

   rectangular_pt, xx, yy, resolution, 1, xpt, ypt, off, $
                   X_OFF=RA_off, Y_OFF=Dec_off, OUT_PT='tmp_3D.pt.txt' ;, /PPAK

   pt = 'tmp_3D.pt.txt'

   ; FITS header for RSS data and 1D FITS
   
   hdr1 = hdr
   hdr3 = hdr

  ; hdr1
   fxaddpar, hdr1, 'CRVAL1', crval, ' updated by PINGSoft'
   fxaddpar, hdr1, 'CDELT1', cdelt, ' updated by PINGSoft'
   fxaddpar, hdr1, 'CRPIX1', crpix, ' updated by PINGSoft'
   sxdelpar, hdr1, 'CRVAL2'
   sxdelpar, hdr1, 'CRVAL3'
   sxdelpar, hdr1, 'CRPIX2'
   sxdelpar, hdr1, 'CRPIX3'
   sxdelpar, hdr1, 'CDELT2'
   sxdelpar, hdr1, 'CDELT3'
   sxdelpar, hdr1, 'DISPAXIS'
   sxdelpar, hdr1, 'WCSDIM'
   sxdelpar, hdr1, 'CTYPE1'
   sxdelpar, hdr1, 'CD1_1'
   sxdelpar, hdr1, 'CD2_2'
   sxdelpar, hdr1, 'CD1_2'
   sxdelpar, hdr1, 'CD2_1'

   sxdelpar, hdr1, 'EXTEND'
   sxdelpar, hdr3, 'EXTEND'

   hdr2 = hdr1

endif

if cdelt eq 0 then begin
   print, ''
   print, ' WARNING: CDELT entry not found, using CDELT=1'
   print, ''
   cdelt = 1
endif

lambda = dblarr(pixels)
for i=0, pixels-1 do lambda[i] = (crval + i*cdelt)-(crpix-1)*cdelt

; Data in microns?
if mean(lambda) lt 100 then lambda = lambda*1e4

; Data in mas?
if mas eq 1 then wcs = 0

lam = lambda

if not keyword_set(lmin) then lmin = min(lambda)

if not keyword_set(lmax) then lmax = max(lambda)

indice = where(lambda ge lmin and lambda le lmax)

lambda = lambda[indice]

xmin = min(lambda) & xmax = max(lambda)

if n_elements(lam) ne pixels then begin
   print, ''
   print, ' WARNING: number of elements in wavelength vector NE to pixels in RSS!'
   print, ''
endif



;--------------------------  Broad/Narrow band 

if crval ne 0. then begin

    ; Pick filter
    if not keyword_set(filter) then filter = 1

    if not keyword_set(user) then pingsoft_filters, filter, sband, lband, fband, cwl

    if keyword_set(user) then begin
       if file_test(user) eq 0 then begin
          print, ''
          print, 'ERROR: The file:  "'+user+'"  does not exist!'
          print, ''
          return
       endif
       pingsoft_filters, filter, sband, lband, fband, cwl, USER=user
    endif

    bad_cwl:

    ; Shift
    if not keyword_set(band) then band = cwl
    shift = cwl/band
    shift = shift[0]
    newl = lband/shift
    
    if filter le 2 then begin
       band1 = cwl-100
       band2 = cwl+100
       shift1 = cwl/band1  &  shift1 = shift1[0]
       shift2 = cwl/band2  &  shift2 = shift2[0]
       newl1 = lband/shift1
       newl2 = lband/shift2
    endif

    ; Interpolation with data
    res = interpol(fband,newl,lam)
    nn = where(lam lt min(newl) or lam gt max(newl))
    if n_elements(nn) gt 1 then res[nn] = 0.

    mm = where(lam ge min(newl) and lam le max(newl))
    if n_elements(mm) eq 1 and mm[0] lt 0 then begin
       print, ''
       print, 'WARNING: Using a V-band filter within the data wavelength limits: '+sctrim(lmin)+' - '+sctrim(lmax)
       print, ''
       filter = 4
       pingsoft_filters, filter, sband, lband, fband, cwl
       band = mean(lam)
       goto, bad_cwl
    endif

    kernel = res/total(res)

    ; optional continuum subtraction
    if filter le 2 then begin
       res1 = interpol(fband,newl1,lam)
       nn = where(lam lt min(newl1) or lam gt max(newl1))
       if n_elements(nn) gt 1 then res1[nn] = 0.
       kernel1 = res1/total(res1)

       ;mm1 = where(lam ge min(newl1) and lam le max(newl1))

       res2 = interpol(fband,newl2,lam)
       nn = where(lam lt min(newl2) or lam gt max(newl2))
       if n_elements(nn) gt 1 then res2[nn] = 0.
       kernel2 = res2/total(res2)

       ;mm2 = where(lam ge min(newl2) and lam le max(newl2))
    endif

    ; convolution
    convolve = res[mm]
    dband = fltarr(rows)

    if filter le 2 then begin
       dband1= fltarr(rows)
       dband2= fltarr(rows)
       for k=0l, rows-1 do dband1[k] = total(kernel1*data[*,k])
       for k=0l, rows-1 do dband2[k] = total(kernel2*data[*,k])
    endif
 
    if keyword_set(subtract) then $
       for k=0l, rows-1 do dband[k] = total(kernel*data[*,k]) - mean([dband1[k],dband2[k]]) else $
       for k=0l, rows-1 do dband[k] = total(kernel*data[*,k])
    
    ; Negative values
    nband = where(dband lt 0)
    if nband[0] gt 0 then dband[nband]=0
    mband = min(dband)

    ;------ Default scaling (linear, clipping=1%)
    scale = pingsoft_scl( dband, /scale, stretch=1, clip=1 )

    ;------ Other scalings

    if keyword_set(clip) then $
       scale = pingsoft_scl( dband, /scale, stretch=2, minvalue=min(dband), maxvalue=max(dband) )
    if keyword_set(gamma) then $
       scale = pingsoft_scl( dband, /scale, stretch=3, minvalue=min(dband), maxvalue=max(dband) )
    if keyword_set(log) then $
       scale = pingsoft_scl( dband, /scale, stretch=4, minvalue=min(dband), maxvalue=max(dband) )
    if keyword_set(asinh) then $
       scale = pingsoft_scl( dband, /scale, stretch=5, minvalue=min(dband), maxvalue=max(dband), beta=30 )
     if keyword_set(sqrt) then $
        scale = pingsoft_scl( dband, /scale, stretch=6, minvalue=min(dband), maxvalue=max(dband) )
    if keyword_set(histogram) then $
       scale = pingsoft_scl( dband, /scale, stretch=7, minvalue=min(dband), maxvalue=max(dband) )
    if keyword_set(gaussian) then $
       scale = pingsoft_scl( dband, /scale, stretch=8, minvalue=min(dband), maxvalue=max(dband) )
    if keyword_set(stddev) then $
       scale = pingsoft_scl( dband, /scale, stretch=10, minvalue=min(dband), maxvalue=max(dband) )

    ; Borders
    nband = where(dband gt mband and scale le 5)
    if nband[0] gt 0 then scale[nband] = 5

endif else begin
    print, ''
    print, 'ERROR: CRVAL1, CDELT1 and/or CRPIX1 not found in RSS header!'
    print, ''
    return
endelse


; Defining output names

if not keyword_set(prefix) then $
   ps_file = object+'_'+sband+'.eps' else $
      ps_file = prefix+'_slice.eps'



;--------------------------  Position table

if keyword_set(pt) then begin

   pos_table = sctrim(pt)

   if file_test(pos_table) eq 0 then begin
      print, ''
      print, "ERROR: The file:  '"+pos_table+"'  does not exist!"
      print, ''
      return
   endif

endif


if not keyword_set(pt) then begin

   test_pt = strsplit(infile,'.fits',/extract,/regex)+'.pt.txt'

   if file_test(test_pt) eq 1 then pos_table = test_pt

   if file_test(test_pt) eq 0 then begin

      pos_table = identify_rss(rows,hdr) 

      if pos_table eq 'none' then begin
         print, ''
         print, "ERROR: PINGSoft cannot find/determine the associated position table"
         print, ''
         print, "       (is the instrument's resolution present in the FITS header?)"
         print, ''
         print, "       try setting the PT='pos_table.pt.txt' keyword"
         print, ''
         return
      endif
      if file_test(pos_table) eq 0 then begin
         print, ''
         print, "ERROR: The file:  '"+pos_table+"'  does not exist!"
         print, ''
         return
      endif
      
   endif

endif

pos_table = pos_table[0]

;-------------------------- Position table tests

; Mosaic type (reading first line of the position table)

readcol, pos_table, f='a,f,f,i', mtype, xsize, ysize, numline=1, /silent

xsize = xsize[0]
ysize = ysize[0]

if mtype eq 'C' then begin
   print, ''
   print, 'ERROR: Position table does not correspond to a rectangular-contiguous grid'
   print, ''
   return
endif

if mtype eq 'S' then begin
    radius = xsize*0.5
    xarr = [-1,1,1,-1,-1]*radius
    yarr = [-1,-1,1,1,-1]*radius
endif


; Read position table

readcol, pos_table, f='l,f,f,f', id, x, y, flag, /silent

if n_elements(id) ne rows then begin
   print, ''
   print, 'ERROR: Number of rows in RSS file not equal to number of entries in the position table'
   print, ''
   print, '       Are you using the right position table?'
   print, ''
   print, '       PT file: '+pos_table
   print, ''
   return
endif



;--------------------------  Writing output FITS

if type eq 2 then begin

   ; Resolution
   ll = n_elements(x)
   testx = fltarr(ll)
   for i=0, ll-2 do testx[i] = abs(x[i+1]-x[i])
   testx = testx[where(testx gt 0.)]
   resol = min(testx)

   xx = ( max(x) - min(x) + resol ) / resol
   yy = ( max(y) - min(y) + resol ) / resol

   ; Geometry
   xgeom = xx-1
   if xgeom gt n_elements(x) then xgeom = n_elements(x)-1
   testx = x[0:xgeom]
   if min(testx) eq max(testx) then geometry = 2 else geometry = 1

   RA_off = xx*resol*0.5
   Dec_off = yy*resol*0.5
   resolution = resol

endif


out_band = fltarr(xx,yy)
   
k=0l
m=fix(xx-1,type=3)
n=fix(yy-1,type=3)

for j=0l, n do begin
   for i=0l, m do begin
      out_band[i,j] = dband[k]
      k = k+1
   endfor
endfor


; Writing FITS file

if not keyword_set(prefix) then begin
   if keyword_set(subtract) then $
      out_fits = object+'_'+sband+'_sub.fits' else $
         out_fits = object+'_'+sband+'.fits'
endif else begin
   out_fits = prefix+'_slice.fits'
endelse

if type eq 2 then writefits, out_fits, out_band

if type eq 3 then writefits, out_fits, out_band, hdr3

if keyword_set(subtract) then sband = sband+' - CONTINUUM'


if keyword_set(widget) then goto, skip_plot

;--------------------------  Integrate RSS
integrate_3D, infile, integ, /no_output, /silent


;--------------------------  Showing band convolution

!p.multi=[0,1,1]

loadct, 1, /silent

; Spaxels window limits
box_limits = pingsoft_boxlim(id,x,y,radius)
xmax_mosaic = box_limits[0]
xmin_mosaic = box_limits[1]
ymax_mosaic = box_limits[2]
ymin_mosaic = box_limits[3]

if keyword_set(ps) then begin
   
   if not keyword_set(font) then psfont=pings.font else psfont=font
   
   pingsoft_pson, ps_file, pings.x_ps, pings.y_ps, font=psfont, axis_thick=pings.axis, encapsulate=1
   
   if mas eq 0 then units = 'arcsec' else units = 'mas'

   plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
         xs=1, ys=1, charsize=pings.csize_ps, position=pings.pos0, $
         xtitle='!9D!X!e !nRA ('+units+')', ytitle='!9D!X!e !nDec ('+units+')', $
         title=title_str, _extra=extra
   goto, ps_on

endif


; Windows position and dimensions
   win_dim = pingsoft_win()
xwin0_size = win_dim[0]
ywin0_size = win_dim[1]
xwin2_size = win_dim[2]
ywin2_size = win_dim[3]

;set_font
window, 0, xs=xwin0_size, ys=ywin0_size
plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], xs=1, ys=1, $
      xthick=pings.wthick, ythick=pings.wthick, position=pings.pos0, charsize=pings.csize,$
      xtitle='!9D!5!e !nRA (arcsec)!X', ytitle='!9D!5!e !nDec (arcsec)!X', title=title_str ,$
      /noerase, _extra=extra

ps_on:


;--------- Color tables 

if not keyword_set(ct) then ct = 1

spawn, 'echo $PINGSOFT_PATH', path
ctfile = path+'/pingsoft_ct.tbl'


;--------- Plot spaxels

for i=0l, n_elements(x)-1 do begin

    xcoor =  x[i]
    ycoor =  y[i]

    x_fib = xarr + xcoor
    y_fib = yarr + ycoor

    loadct, ct, file=ctfile, /silent
    polyfill, x_fib, y_fib, color=scale[i]

    loadct, 1, /silent

    if keyword_set(draw) then oplot, x_fib, y_fib, color=pings.dcolor1

    if keyword_set(ps) then begin
      loadct, ct, file=ctfile, /silent
       polyfill, x_fib, y_fib, color=scale[i]
       loadct, 1, /silent
       if keyword_set(draw) then oplot, x_fib, y_fib, color=pings.dcolor2
    endif

endfor

if keyword_set(ps) then begin
    pingsoft_psoff
    goto,fin
endif



;-------------------------- Integrated spectrum + pass-band

window, 2, xs=xwin2_size, ys=ywin2_size

y_max = max(integ.finteg)*1.2
y_min = min(integ.finteg)

plot, [0], /nodata, xr=[xmin-50,xmax+50], yr=[y_min, y_max], xs=1, ys=1, $
      xthick=pings.wthick, ythick=pings.wthick, position=pings.pos2, charsize=pings.csize, $
      xtitle=pings.xtitle2, ytitle='Relative Flux', title='Integrated spectrum filter'




loadct, 39, /silent
fact=1
if filter ge 6 and filter le 10 then fact=1.5
oplot, lam, res*y_max*fact, linestyle=2, color=220
oplot, integ.lambda, integ.finteg, color=30

;caro
fin:

; Structure output

out_str = {$
          filter_image: out_band   ,$
          filter_lam: lam          ,$
          filter_flux: res         ,$
          integ_lam: integ.lambda  ,$
          integ_flux: integ.finteg  $
          }


;-------------------------- Outputs

if not keyword_set(silent) then begin

   print, ''
   print, ''
   print, ' PINGSoft'
   print, ' ====================='
   print, ''
   print, '         Using filter:   '+sband
   print, ''
   print, '  Filter image (FITS):   '+out_fits
   print, ''
   if keyword_set(ps) then begin
   print, '      PS file written:   '+ps_file
   print, ''
   if n_params() eq 3 then help, out_str, /struc
   print, ''
   endif

endif

if file_test('tmp.pt.txt') eq 1 then spawn, 'rm -f tmp.pt.txt'
if file_test('tmp_3D.pt.txt') eq 1 then spawn, 'rm -f tmp_3D.pt.txt'

skip_plot:
end
