
pro extract_index, infile, index, out_str, PT=pt, PREFIX=prefix, EXTENSION=extension, $
                   FILTER=filter, BAND=band, CT=ct, DRAW=draw, PS=ps, FONT=font, PLOT=plot, $
                   CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
                   SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
                   SIMPLE=simple, SILENT=silent, _EXTRA=extra

;+
; NAME:
;       EXTRACT_INDEX
;
; PURPOSE:
;
;       Extracts new a RSS file and correspoding position table based on an
;       input index vector.
;
; EXPLANATION:
;
;       The program extracts the spectra corresponding to the ID of the spaxels
;       defined in an integer vector. It stores the spectra in a new RSS file
;       and creates a new position table of the selected spaxels. If the PREFIX
;       keyword is not set, the program uses the default: 'index'. If the /PLOT
;       keyword is set, the program opens the RSS spatial visualisation showing
;       the selected spaxels. If the /PS keyword is set, a Postscript file is
;       generated. The /PLOT and /PS keywords are mutually exclusive. Some
;       display options are similar to 'view_rss.pro'.
;
;
; CALLING SEQUENCE:
;       
;       extract_index, 'OBJECT.fits', index [, 'pos_table.txt', PREFIX=prefix, EXTENSION=extension, /NOEXTRACT,$
;                       BAND=band, WIDTH=width, CT=ct, FONT=font, /PLOT, /PS, /DRAW, /LINEAR, /GAMMA, /LOG, /ASINH ]
;                       
;
; INPUTS:
;  
;            'RSS.fits':  Wavelength calibrated RSS FITS file.  
;
;                 index:  Integer vector index of the spaxels to be extracted.
;
; OPTIONAL KEYWORDS:
;       
;       'pos_table.txt':  Position table of the RSS file in ASCII format (in NE configuration).
;                         (compulsory if not included in the default instruments/setups)
;                         
;            /NOEXTRACT:  If set, the program only displays the spaxels on the
;                         visualisation window, without performing the extraction.
;
;                PREFIX:  String with the prefix of the output files, if not chosen the default is 'index'.
;
;                 /PLOT:  Display the RSS visualisation and the extracted spaxels.
;
;                   /PS:  Writes a Postscript file of the spaxels visualisation.'
;
;        Valid only if /PLOT or /PS are set:
;
;                  BAND:  Central wavelength of the narrow band used to display the data,
;                         defaults: BAND=6563 (i.e. Halpha) if within the spectral range, 
;                         else BAND=mean(lambda).
; 
;                 WIDTH:  Width of the band to display the data, if not set the
;                         default is 100 Angstroms.
;
;                    CT:  IDL Color Table used to display the data, (default ct=1, BLUE/WHITE).
;                     
;                  FONT:  Vector-drawn IDL font to be used, default: 3 (Simplex Roman).
;    
;                 /DRAW:  Draws the contour of the spaxels.
; 
;               /LINEAR:  Displays the range of intensities using a linear min/max scaling.'
;
;                /GAMMA:  Displays the range of intensities using a power-law (gamma) scaling.'
;
;                  /LOG:  Displays the range of intensities using a logarithmic scaling.'
;
;                /ASINH:  Displays the range of intensities using an inverse hyperbolic sine function scaling.'
;
;
; OUTPUTS:
;
;       A new RSS and position table files of the selected spaxels and
;       integrated spectrum:
;
;         Extracted RSS:   PREFIX_rss.fits
;        Position table:   PREFIX_rss.pt.txt
;      Integrated ASCII:   PREFIX_integ.txt
;       Integrated FITS:   PREFIX_integ.fits
;       Postscript file:   PREFIX_index.ps             (only if /PS is set)
;
;
; EXAMPLES:
;
;       index=[82,86,91,107,108,115,124,132,133,134,138,$               ; Spaxels' IDs
;              166,167,168,184,189,192,197,201,204,205]
;
;       extract_index, 'object_rss.fits', index                         ; Direct extraction, no visualisation
;
;       extract_index, 'object_rss.fits', index, PREFIX='test', /plot   ; Extraction with prefix name & visualisation
;
;
; REVISION HISTORY:
;
;       Written by:     F. Fabian Rosales-Ortega
;
;       First version:  January, 2010
;
; NOTE:
;
;       This routine belongs to the PINGS IDL software available at:
;
;       http://www.ast.cam.ac.uk/research/pings/
;
;       where additional documentation can be found. If you find this code
;       useful on your research, please acknowledge the use of PINGSoft by
;       including the original project reference in your publications.
;
; COPYRIGHT:
;
;       This file is part of PINGSoft.
;
;       Copyright (C) 2010  F. Fabian Rosales-Ortega
;
;       PINGSoft is free software: you can redistribute it and/or modify it
;       under the terms of the GNU General Public License as published by the
;       Free Software Foundation, version 3. See the GNU General Public License
;       for more details.
;
;       PINGSoft is distributed in the hope that it will be useful, but WITHOUT
;       ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;       FITNESS FOR A PARTICULAR PURPOSE. 
;
;       The GNU General Public License is found in: http://www.gnu.org/licenses
;
;-


compile_opt idl2, hidden
on_error, 2


if n_params() lt 2 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, ''
    print, "   extract_index, 'OBJECT.fits', index  [, out_str, PT=pt, PREFIX=prefix, EXTENSION=extension, $"
    print, '                   FILTER=filter, BAND=band, CT=ct, DRAW=draw, /PLOT, /PS, FONT=font, $'
    print, '                   /LINEAR, /GAMMA, /LOG, /ASINH, /SIMPLE, _EXTRA={extra} ]'
    print,''
    print, ''
    print, "       'RSS.fits':  String of the wavelength calibrated RSS FITS file."
    print, ''
    print, '            index:  Integer vector index of the spaxels to be extracted.'
    print, ''
    print, "  'pos_table.txt':  String of the position table of the RSS file in ASCII format (in NE configuration)."
    print, '                    (compulsory if not included in the default instruments/setups)'
    print, ''
    print, '       /NOEXTRACT:  If set, the program only displays the spaxels on the'
    print, '                     visualisation window, without performing the extraction.'
    print, ''
    print, "           PREFIX:  String with the prefix of the output files, if not chosen the default is 'index'"
    print, ''
    print, '            /PLOT:  Display the RSS visualisation and the extracted spaxels.'
    print, ''
    print, '              /PS:  Writes a Postscript file of the spaxels visualisation.'
    print, ''
    print, '   Valid only if /PLOT or /PS are set:'
    print, ''
    print, '             BAND:  Central wavelength of the narrow band used to display the data, '
    print, '                    defaults: BAND=6563 (i.e. Halpha) or BAND=mean(lambda).'
    print, ''
    print, '            WIDTH:  Width of the band to display the data, if not set the default is 100 Angstroms.'
    print, ''
    print, '               CT:  IDL Color Table used to display the data (default ct=1, BLUE/WHITE).'
    print, ''
    print, '             FONT:  Vector-drawn IDL font to be used, default: 3 (Simplex Roman)'
    print, ''
    print, '            /DRAW:  Draws the contour of the spaxels.'
    print, ''
    print, '          /LINEAR:  Displays the range of intensities using a linear min/max scaling.'
    print ,''
    print, '           /GAMMA:  Displays the range of intensities using a power-law (gamma) scaling.'
    print, ''
    print, '             /LOG:  Displays the range of intensities using a logarithmic scaling.'
    print, ''
    print, '           /ASINH:  Displays the range of intensities using an inverse hyperbolic sine function scaling.'
    print, ''
    print, ''
    return
endif

;--------------------------  Initial tests

infile = sctrim(infile)

if file_test(infile) eq 0 then begin
    print, ''
    print, 'ERROR: The file:  "'+infile+'"  does not exist!'
    print, ''
    return
endif



;--------------------------  Routine properties

if keyword_set(plot) and keyword_set(ps) then begin
   print, ''
   print, ' WARNING: keywords PLOT and PS cannot be set at the same time'
   print, ''
endif

if not keyword_set(plot) then plot=1
if keyword_set(ps) then plot=0
if keyword_set(silent) then plot=0 

if not keyword_set(prefix) then prefix='index'

ps_file = prefix+'_FoV.eps'


;--------------------------  PINGSoft core

this_plot = plot

 if not keyword_set(asinh) and not keyword_set(clip) $
and not keyword_set(gamma) and not keyword_set(log) then asinh=1

pingsoft_core, infile, pings, core, ps_file, $
               PT=pt, PREFIX=prefix, EXTENSION=extension, $
               MIN_FLUX=min_flux, MAX_FLUX=max_flux, LMIN=lmin, LMAX=lmax, $
               FILTER=filter, BAND=band, CT=ct, DRAW=draw, PS=ps, FONT=font, PLOT=plot, $
               CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
               SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
               LARGE=large, NOEXTRACT=noextract,  _EXTRA=extra

; Variables from structure
tag = get_tags(core)
for k=0, n_elements(tag)-1 do begin
   dumb = strmid(tag[k],1)
   com = execute(dumb+' = core.'+dumb)
endfor
plot = this_plot


;--------------------------  Extraction

new_data = data[*,index]

if keyword_set(plot) or keyword_set(ps) then begin

    loadct, 39, /silent
    for i=0, n_elements(index)-1 do begin
        id = index[i]
        xcoor =  x[id]
        ycoor =  y[id]
        x_fib = xarr+xcoor
        y_fib = yarr+ycoor
        if keyword_set(plot) then oplot, x_fib, y_fib, thick=2, color=250
        if keyword_set(ps) then oplot, x_fib, y_fib, thick=5, color=250
    endfor
    loadct, 1, /silent
endif

if keyword_set(ps) then begin
    pingsoft_psoff
endif

;--------------------------  Integrated spectra

all_flux = fltarr(pixels)

for i=0, n_elements(index)-1 do begin
    id = index[i]
    flux  = data[*,id]
    all_flux = all_flux + flux
endfor

integ_flux = all_flux

xmin = min(lambda)     & xmax = max(lambda)
ymin = min(integ_flux) & ymax = max(integ_flux)*1.2

loadct, 1, /silent
if keyword_set(plot) then begin
    window, 2
    plot, [0], /nodata, xr=[xmin-80,xmax+80], yr=[ymin,ymax], xs=1, ys=1, $
          xthick=pings.wthick, ythick=pings.wthick, position=pings.pos2, charsize=pings.csize, $
          title='Integrated spectrum', xtitle='Wavelength', ytitle='Flux', _extra=extra
    loadct, 39, /silent
    oplot, lambda, integ_flux, color=250
endif


;-----------------------------------  Outputs

  out_rss = prefix+'_rss.fits'
   out_pt = prefix+'_rss.pt.txt'
 out_fits = prefix+'_integ.fits'
out_integ = prefix+'_integ.txt'

; Integrated flux ASCII
openw,3,out_integ
for k=0, n_elements(lambda)-1 do printf,3, k+1, lambda[k], integ_flux[k]
close, 3

if not keyword_set(simple) then begin

   ; Integrated flux FITS
   writefits, out_fits, integ_flux, hdr2

   ; RSS FITS file
   writefits, out_rss, new_data, hdr2

   ; PT file
   get_new_pt, index, pos_table, out_pt, /silent

   ; PS extracted spectrum
   out_ps =  prefix+'_integ.eps'
   pingsoft_pson, out_ps, 20, 16, font=psfont, axis_thick=pings.axis, encapsulate=1
   loadct, 1, /silent
   xvar = lambda
   yvar = integ_flux
   plot, [0], /nodata, xr=[min(xvar)-80,max(xvar)+80], yr=[min(yvar), max(yvar)*1.2], $
         xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, pos=pings.pos2, charsize=pings.csize*1.2, $
         xtitle='Wavelength', ytitle='Flux', title='Integrated spectrum'
   oplot, xvar, yvar, color=100
   pingsoft_psoff

endif

; Saving structure
if not keyword_set(simple) then $
out_str = { $
          object: infile ,$
          header: hdr    ,$
          index:  index  ,$
          lambda: lambda ,$
          finteg: integ_flux ,$
          newrss: out_rss ,$
          ptable: out_pt  $
          } else $
out_str = { $
          object: infile ,$
          header: hdr    ,$
          index:  index  ,$
          lambda: lambda ,$
          finteg: integ_flux $
          }

if not keyword_set(silent) then begin

print, ''
print, ' PINGSoft'
print, ' ===================='
print, ''
print, ' Files written '
print, ''
if not keyword_set(simple) then begin
print, '       Extracted RSS:   ', out_rss
print, '      Position table:   ', out_pt
print, '     Integrated FITS:   ', out_fits
endif
print, '               ASCII:   ', out_integ
if not keyword_set(simple) then $
print, '          Postscript:   ', out_ps
if keyword_set(ps) then $
print, '      Extraction FoV:   ', ps_file
if n_params() eq 3 then begin
print, ''
print, ' IDL structure saved:   '
print, ''
help, out_str, /struc
print, ''
endif
print, ''

endif

if file_test('tmp.pt.txt') eq 1 then spawn, 'rm -f tmp.pt.txt'
if file_test('tmp_3D.pt.txt') eq 1 then spawn, 'rm -f tmp_3D.pt.txt'


end
