


;+
; NAME:
;       EXTRACT_MASK
;
; PURPOSE:
;
;        Extracts the spectra based on a user's given mask or segmentation map.
;
; EXPLANATION:
;
;        
;         The program works in two different ways depending on whether
;         the input is a "mask" (i.e. a FITS image with values 0 and 1)
;         or a segmentation map (i.e. a  FITS image with integer values ranging
;         from 0,1,2,...,N). In the first case, the routine simply applies a
;         mask extraction to the IFS data, i.e. those regions in the mask image
;         equal to 0 are discarded, while regions with values equal to 1 are
;         preserved, and a new cube is created. In the second case, for each
;         region in the segmentation map with values different than 0, the
;         command integrates in the IFS cube the spectra for all spaxels with
;         the same integer value on the segmentation map, and appends the
;         integrated spectrum to a new RSS file. The final output is a RSS file
;         with the same number of entries as the non-zero integer values of the
;         segmentation map, plus a Pseudo-position table for visualisation.
;         Additionally, the program writes a new 3D cube with the regions
;         segmented as the input segmentation map, in which each spaxel
;         corresponds to the integrated spectrum at that particular region. In
;         the segmentation map case, the program displays the FoV and the
;         segmented regions on an IDL window.
;
;
; For CALLING SEQUENCE, INPUTS and OPTIONAL KEYWORDS enter:
;
;        IDL> extract_mask
;
;
; OUTPUTS:
;
;        1st case:  Simple mask
;
;
;                   Masked 3D cube:  PREFIX_mask.cube.fits
;         Postscript segmented FoV:  PREFIX_mask.FoV.eps     (if /PS is set)
;
;
;        2nd case:  Segmentation map
;
;         Extracted spectra in RSS:  PREFIX_mask.rss.fits
;            Pseudo-position table:  PREFIX_mask.rss.pt.txt
;           Full segmented 3D cube:  PREFIX_mask.cube.fits
;         Postscript segmented FoV:  PREFIX_mask.FoV.eps     (if /PS is set)
;
;
;         If PREFIX is not set, then PREFIX='mask'
;
;
; COMMENTS
;
;         Note the script works only with regular-grid data, i.e. continuous
;         3D cubes. The 2D mask/segmentation map and 3D cube must have the  
;         same X-Y dimensions.
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-


pro extract_mask, infile, smask, NORMALIZE=normalize, SEGMENT=segment,  $
                  EXTENSION=extension, PREFIX=prefix, SILENT=silent, $
                  FILTER=filter, BAND=band, CT=ct, FONT=font,  PS=ps,  $
                  CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
                  SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
                  DRAW=draw, NUMBER=number,  _EXTRA=extra


compile_opt idl2, hidden
on_error, 2

if n_params() lt 2 then begin
   print, ''
   print, " extract_mask,  'OBJECT.fits', 'mask.fits' [, PREFIX='prefix', "
   print, '                 /NORMALIZE, /SEGMENT, /PS, /SILENT, EXTENSION=extension'
   print, '                 (+ all visual options of VIEW_3D) ]  '
   print, ''
   print, ''
   print, "     'OBJECT.fits':  String of the wavelength calibrated 3D cube or RSS FITS file."
   print, ''
   print, "       'mask.fits':  String of the 2D mask or segmentation map FITS file."
   print, ''
   print, ' OPTIONAL KEYWORDS'
   print, ''
   print, "            PREFIX:  String with the prefix of the output files. Default: 'mask'"
   print, ''
   print, '          EXTENSION:  Non-negative scalar integer specifying the FITS extension to read.'
   print, '                      For example, specify EXTENSION = 1 to read the first FITS extension.'
   print, ''
   print, '        /NORMALIZE:  If set, the integrated spectrum is normalized by the number'
   print, '                     of spaxels used to integrate the segmented region.'
   print, ''
   print, '          /SEGMENT:  If set, the on-screen or Postscript visualisation of the'
   print, '                     segmented regions uses a special, nicer color-coding'
   print, ''
   print, '           /NUMBER:  If set, the segmentation integer numbe is plot on top of'
   print, '                     the region'
   print, ''
   print, '           /SILENT:  Omits printing output information on the terminal'
   print, ''
   print, '               /PS:  Writes an encapsulated Postscript file of the segmented regions'
   print, ''
   print, "             _EXTRA:  Structure with the _EXTRA tags for user's defined graphics output,"
   print, "                      e.g. _EXTRA={title:'IFS cube'}"
   print, ''
   print, '   + all visual options of VIEW_3D'
   print, ''
   return
endif


;--------------------------  Initial tests

  infile = sctrim(infile)
smask = sctrim(smask)

if file_test(infile) eq 0 then begin
   print, ''
   print, 'ERROR: The file:  "'+infile+'"  does not exist!'
   print, ''
   return
endif
if file_test(smask) eq 0 then begin
   print, ''
   print, 'ERROR: The file:  "'+smask+'"  does not exist!'
   print, ''
   return
endif



;--------------------------  Graphic device settings
pingsoft_param, pings, ''
pingsoft_display, pings, /silent


;--------------------------  PINGSoft core

if keyword_set(ps) then tmp_ps = 1 else tmp_ps = 0
this_ps = 0

pingsoft_core, infile, pings, core, ps_file, $
               PT=pt, EXTENSION=extension, PREFIX=prefix, $
               MIN_FLUX=min_flux, MAX_FLUX=max_flux, LMIN=lmin, LMAX=lmax, $
               FILTER=filter, BAND=band, CT=ct, DRAW=draw, PS=this_ps, FONT=font, PLOT=0, $
               CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
               SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
               LARGE=large, NOEXTRACT=noextract,  _EXTRA=extra

; Variables from structure
tag = get_tags(core)
for k=0, n_elements(tag)-1 do begin
   dumb = strmid(tag[k],1)
   com = execute(dumb+' = core.'+dumb)
endfor


if not keyword_set(prefix) then prefix='mask'
ps = tmp_ps

;--------------------------  Mask/Segmentation map

mask = readfits(smask, /silent)
size = size(mask)
xm = size[1]
ym = size[2]

if xx ne xm or yy ne ym then begin
   print, ''
   print, 'ERROR: dimensions of the mask and input FITS do not match!'
   print, ''
   return
endif

; Number of segmented regions
nreg = fix(max(mask))
rmap = mask


;--------------------------  Extraction if nreg = 1

if nreg eq 1 then begin
   print, ''
   print, ' Number of segmented regions:  '+sctrim(nreg)
   print, ''
   print, ' Assuming a simple mask extraction...'
   print, ''
   
   data = readfits(infile,hdr,/silent)
   nsize = size(data)
   pixels = nsize[3]
   new = fltarr(nsize[1],nsize[2],pixels)
   for i=0l, pixels-1 do new[*,*,i] = mask * data[*,*,i]  


   ; Output

   if califa eq 1 then begin
      ext1 = readfits(infile,ext=1,h1,/silent)
      ext2 = readfits(infile,ext=2,h2,/silent)
      ext3 = readfits(infile,ext=3,h3,/silent)

      new_1 = fltarr(nsize[1],nsize[2],pixels)
      new_2 = fltarr(nsize[1],nsize[2],pixels)
      new_3 = fltarr(nsize[1],nsize[2],pixels)

      for k=0l, pixels-1 do new_1[*,*,k] = mask * ext1[*,*,k]
      for k=0l, pixels-1 do new_2[*,*,k] = mask * ext2[*,*,k]  
      for k=0l, pixels-1 do new_3[*,*,k] = mask * ext3[*,*,k]  
   endif

   out_cube = prefix+'_mask.cube.fits'
   writefits, out_cube, new, hdr

   if califa eq 1 then begin
      writefits, out_cube, new_1, h1, /append
      writefits, out_cube, new_2, h2, /append
      writefits, out_cube, new_3, h3, /append
   endif

   if not keyword_set(silent) then begin
      print, ''
      print, ' PINGSoft:  extract_mask'
      print, ' ==========================='
      print, ''
      print, '    Masked 3D cube:  '+out_cube
      print, ''
   endif

   goto, fin

endif



;--------------------------  Extraction if nreg > 1

print, ''
print, ' Number of segmented regions:  '+sctrim(nreg)
print, ''
print, ' Extracting segmented spectra....'
print, ''


seg_rss = fltarr(pixels,nreg)
seg_xy = fltarr(2,nreg)
seg_pix = fltarr(nreg)
seg_cube = fltarr(xx,yy,pixels)

nreg_string = sctrim(nreg)

for i=0l, nreg-1 do begin

   nn = i+1
   index = where(rmap eq nn)

   if index[0] lt 0 then goto, bad_data

   ; Integrated spectra
   all_flux = fltarr(pixels)
   for j=0, n_elements(index)-1 do begin
      id = index[j]
      flux  = data[*,id]
      all_flux = all_flux + flux
   endfor

   if keyword_set(normalize) then nfact = n_elements(index) else nfact = 1.

   integ_flux = all_flux / nfact

   ; RSS output
   seg_rss[*,i] = integ_flux
   
   ; Cube output
   seg_cube = set_value3d(seg_cube,index,integ_flux)

   ; Center-of-Mass coordinates in pixels
   tindex = where(rmap ne nn)
   mtest = set_value2d(rmap,tindex,0.)
   seg_xy[*,i] = centroid(mtest)

   ; N-pixels
   seg_pix[i] = n_elements(index)

   bad_data:
endfor


;Center-of-Mass coordinates in arcsec

xypix = seg_xy*resolution

x_arcsec = transpose( -(xypix[0,*]-RA_off) )
y_arcsec = transpose(   xypix[1,*]-Dec_off )


; Pseudo-position table
out_pt = prefix+'_mask.rss.pt.txt'

seg_pix = fix(seg_pix)

openw, 1, out_pt
printf, 1, 'S  1.0  1.0  0'
printf, 1, ''
printf, 1, '# PINGSoft: Pseudo-position table for mask extraction'
printf, 1, '# ----------------------------------------------------'
printf, 1, '#'
printf, 1, '# N-bin, X-bin, Y-bin, N-pixels per bin'
printf, 1, '#'
printf, 1, ''
for i=0l, n_elements(x_arcsec)-1 do printf, 1,  i+1, x_arcsec[i], y_arcsec[i], seg_pix[i]
close, 1


;----------------------------  Outputs

; RSS
out_rss = prefix+'_mask.rss.fits'
writefits, out_rss,  seg_rss, hdr2

; 3D cube
out_cube = prefix+'_mask.cube.fits'
writefits, out_cube, seg_cube, hdr3


;--------------------------  Plot

if keyword_set(ps) then begin

   plot_segments, infile, rmap, /ps, SEGMENT=segment, PREFIX=prefix, $
                  FILTER=filter, BAND=band, CT=ct, FONT=font,  $
                  CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
                  SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
                  DRAW=draw, x_arcsec=x_arcsec, y_arcsec=y_arcsec, $
                  NUMBER=number, _EXTRA=extra

endif else begin

   plot_segments, infile, rmap, /plot, SEGMENT=segment, PREFIX=prefix, $
                  FILTER=filter, BAND=band, CT=ct, FONT=font,  $
                  CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
                  SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
                  DRAW=draw, x_arcsec=x_arcsec, y_arcsec=y_arcsec, $
                  NUMBER=number, _EXTRA=extra

endelse


;--------------------------  Screen info

if not keyword_set(silent) then begin
   print, ''
   print, ' PINGSoft:  extract_mask'
   print, ' ==========================='
   print, ''
   print, ' Number of segmented regions: '+nreg_string
   print, ''
   print, ' Files written: '
   print, ''
   print, '   Extracted spectra in RSS:  '+out_rss
   print, '      Pseudo-position table:  '+out_pt
   print, '     Full segmented 3D cube:  '+out_cube
   print, ''
   if keyword_set(ps) then begin
      print, '   Postscript segmented FoV:  '+prefix+'_mask.FoV.eps'
   endif
   print, ''
endif


fin:
if file_test('tmp.pt.txt') eq 1 then spawn, 'rm -f tmp.pt.txt'
if file_test('tmp_3D.pt.txt') eq 1 then spawn, 'rm -f tmp_3D.pt.txt'

end
