
;+
; NAME:
;       S2N_RAT
;
; PURPOSE:
;
;       Function to calculate the S/N ratio at each bin.
;

function s2n_rat, flux, PROXY=proxy

  ; S/N standard definition: Mean/StdDev
  s2n_val = round( mean(flux)/ stddev(flux) )

  ; Alternative Proxy: Mean/sqrt(StdDev)
  if keyword_set(proxy) then s2n_val = round( mean(flux)/sqrt(stddev(flux)) )

return, s2n_val
end


;+
; NAME:
;       EXTRACT_RADIAL
;
; PURPOSE:
;
;       Extracts radial average spectra within consecutive elliptical rings from
;       a reference point, based on either fixed bins or S/N floor.
;
;       
; EXPLANATION:
;
;       Taking as a reference the input central coordinates the program extracts
;       and co-adds the spectra within successive rings of either 1) a given
;       width in arcsec, or 2) an adaptive size depending on a input S/N target,
;       creating in both cases an integrated spectrum in each of the bins, which
;       are then stored in a RSS file. The initial and final radius for the
;       extraction can be defined. 
;       If the /PLOT keyword is set, the program opens the FoV visualisation
;       showing the extend of the extracted rings. If the /PS keyword is set, a
;       Postscript file is generated. The /PLOT and /PS keywords are mutually
;       exclusive. The program also generates a 'Radial reference table' with
;       the information of the bin size and integration radius, as displayed in
;       the terminal. This position table can be used to visualise the radial
;       extracted spectra using the VIEW_3D command.
;       Similar display options as in VIEW_3D.
;
;
; For CALLING SEQUENCE, INPUTS and OPTIONAL KEYWORDS enter:
;
;        IDL> extract_aperture
;
;
; OUTPUTS:
;
;      If PREFIX is set:
;
;           Extracted RSS:   PREFIX_rss.fits
;          Position table:   PREFIX_rss.pt.txt
;        Integrated ASCII:   PREFIX_integ.txt
;                    FITS:   PREFIX_integ.fits
;              Postscript:   PREFIX_integ.eps
;          Extraction FoV:   PREFIX_FoV.eps         (only if /PS is set)
;
;
; Otherwise, PREFIX='radial' 

;
; COMMENTS
;
;       Only those spaxels which centers lie within the aperture are extracted,
;       "fractional spaxels" are not considered. 
;
; S/N CALCULATION
;
;       In the case of adaptive bin size based on S/N floor, the S/N is
;       calculated using a "standard" definition: S/N = Mean(flux)/StdDev(flux)
;       If the user sets the /PROXY keyword, the S/N is calculated using
;       an "optional" S/N definition: S/N = Mean(flux)/sqrt(StdDev(flux))
;       which is equivalent to a "density S/N", i.e. a PROXY for the surface
;       brightness of the object.
;       Note that the user can change the calculation of the S/N by editing the
;       's2n_rat' function at the top of the program.
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-


pro extract_radial, infile, x0, y0, epsilon, PA, step, out_str, PT=pt, PREFIX=prefix, EXTENSION=extension, $
                    RMIN=rmin, RMAX=rmax, S2N=s2n, LAM_S2N=lam_s2n, WIDTH=width, PROXY=proxy, $
                    FILTER=filter, BAND=band, FONT=font, DRAW=draw, SILENT=silent, $
                    CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh, SQRT=sqrt, $
                    HISTOGRAM=histogram, GAUSSIAN=gaussian, STDDEV=stddev, $
                    PLOT=plot, PS=ps, _EXTRA=extra


compile_opt idl2, hidden
on_error, 2


if n_params() lt 6 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "     extract_radial, 'OBJECT.fits', x0, y0, epsilon, PA, step "
    print, "                      [, RMIN=rmin, RMAX=rmax, S2N=s2n, LAM_S2N=lam_s2n, WIDTH=width, /PROXY, "
    print, "                      OUT.str, PT='pos_table.txt', PREFIX='prefix', EXTENSION=extension,  "
    print, '                      (+ all visual options of VIEW_3D) ]'
    print, ''
    print, "      'OBJECT.fits':  String of the wavelength calibrated 3D cube or RSS FITS file."
    print, ''
    print, '              x0/y0:  RA/Dec offsets (in arcsec) of the central position of the aperture to be simulated.'
    print, ''
    print, '            epsilon:  Eccentricity of the elliptical apertures (circle=0,  ellipse= < 1)'
    print, ''
    print, '                 PA:  Position angle of the major-axis, measured counterclockwise from the North.'
    print, ''
    print, '               step:  Radial bin size in arcsec (minimum bin size in case of S/N floor).'
    print, ''
    print, ''
    print, ' OPTIONAL KEYWORDS'
    print, ''
    print, '               RMIN:  Initial extraction radius (in arcsec), default: 0'
    print, ''
    print, '               RMAX:  Final extraction radius (in arcsec), default: maximum FoV radius'
    print, ''
    print, '                S2N:  Floating value of the target S/N for each radial bin.'
    print, '                      If set, the S/N is calculated for each bin at the minimum radial size'
    print, "                      set by the 'step' argument, if the S/N is below this value, the radius"
    print, '                      will grow by 1 arcsec step until the S/N target is achieved.'
    print, '                      The S/N is calculated using the "standard" definition Mean(flux)/StdDev(flux)'
    print, ''
    print, '            LAM_S2N:  Central wavelength of the band at which the S/N will be calculated.'
    print, '                       If not set, default: mean(lambda)'
    print, ''
    print, '              WIDTH:  Width of the band at which the S/N will be calculated, default: 100'
    print, ''
    print, '             /PROXY:  The S/N is calculated using an "optional" S/N definition: '
    print, '                      Mean(flux)/sqrt(StdDev(flux)), which is equivalent to a "density S/N"'
    print, '                      and it is a PROXY for the surface brightness of the object.'
    print, ''
    print, "            OUT.str:  Name of the optional output structure."
    print, ''
    print, "    PT='Ptable.txt':  Name of the position table in the case of an RSS file"
    print, '                      in ASCII format (North-East configuration).'
    print, '                      NOTE: compulsory if not included in the default instruments/setups'
    print, "                      or when the name is not in the 'OBJECT.pt.txt' format."
    print, ''
    print, "             PREFIX:  String with the prefix of the output files. Default: 'radial'"
    print, ''
    print, '          EXTENSION:  Non-negative scalar integer specifying the FITS extension to read.'
    print, '                      For example, specify EXTENSION = 1 to read the first FITS extension.'
    print, ''
    print, '              /PLOT:  Display the FoV visualisation and the extracted rings, this is the default.'
    print, ''
    print, '                /PS:  Writes an encapsulated PS image of the FoV and extracted rings.'
    print, ''
    print, '               FONT:  Postscript IDL font to be used when /PS is set. Default: 12 (Helvetica)'
    print, ''
    print, '   + all visual options of VIEW_3D  (valid only if /PLOT or /PS are set)'
    print, ''
    return
endif


;--------------------------  Initial tests

infile = sctrim(infile)

if file_test(infile) eq 0 then begin
    print, ''
    print, 'ERROR: The file:  "'+infile+'"  does not exist!'
    print, ''
    return
endif


;--------------------------  Routine properties

if not keyword_set(plot) then plot=1
if keyword_set(ps) then plot=0
if keyword_set(silent) then plot=0 


t1 = strcompress(string(step, format='(f7.1)'),/remove_all)
if keyword_set(s2n) then t1 = 's2n'

if not keyword_set(prefix) then out_rss = 'radial_'+t1+'.fits' else $
  out_rss = prefix+'_radial.fits'

if not keyword_set(prefix) then  out_pt = 'radial_'+t1+'.pt.txt' else $
  out_pt = prefix+'_radial.pt.txt'

if not keyword_set(prefix) then  ps_FoV = 'radial_'+t1+'.FoV.eps' else $
  ps_FoV = prefix+'_radial.FoV.eps'

if not keyword_set(prefix) then ps_plot = 'radial_'+t1+'.eps' else $
  ps_plot = prefix+'_radial.eps'


if epsilon lt 0 or epsilon ge 1 then begin
   print,''
   print,'ERROR: the value EPSILON (eccentricity) should be between 0 and 1'
   print,''
   return
endif


;--------------------------  PINGSoft core

pingsoft_core, infile, pings, core, ps_FoV, x0, y0, /SHIFT, $
               PT=pt, PREFIX=prefix,  $
               MIN_FLUX=min_flux, MAX_FLUX=max_flux, LMIN=lmin, LMAX=lmax, $
               FILTER=filter, BAND=band, CT=ct, DRAW=draw, PS=ps, FONT=font, PLOT=plot, $
               LINEAR=linear, GAMMA=gamma, LOG=log, ASINH=asinh, $
               LARGE=large, NOEXTRACT=noextract,  _EXTRA=extra

; Variables from structure
tag = get_tags(core)
for k=0, n_elements(tag)-1 do begin
   dumb = strmid(tag[k],1)
   com = execute(dumb+' = core.'+dumb)
endfor


spawn, 'echo $PINGSOFT_PATH', path
ctfile = path+'/pingsoft_ct.tbl'


;  X/Y offsets after shift
x0 = 0
y0 = 0


;--------------------------- PA in radians and astro conventions

if PA ge 180 then PA = PA - 180

; correction for RA convenction (inverse sign, -90)
ang = -(PA-90)/!RADEG

npoints = 200
phi = 2*!pi*findgen(npoints+1)/npoints


;--------------------------  Radii

rad = sqrt( x^2 + y^2 )

if not keyword_set(rmin) then rmin = 0

if not keyword_set(rmax) then rad_max = max(rad) else rad_max = rmax
 
a_ini = rmin
  bin = 1
color = 255
theta = 0

; calling dumb functions
dumb = [1,2,3]
abc = mean(dumb)
abc = stddev(dumb)
abc = exclude_index(dumb,[1,2])


;--------------------------  Extracting spectra

sstep = sctrim(step)

rss_data = fltarr(pixels)

loadct, 39,/silent

openw, 1, out_pt

if not keyword_set(silent) then begin
print, ''
print, ''
print, ' PINGSoft: Radial extraction'
print, ' ========================================================'
print, ''
print, ''
print, '     Bin     R(1)     R(2)     R(size)   S/N    Spaxels'
print, ' --------------------------------------------------------'
print, ''    
endif

printf,1, 'S  '+sstep+'  '+sstep+'  9'
printf,1, ''
printf,1, '# Source: '+object
printf,1, '#'
printf,1, '# PINGSoft: Radial extraction'
printf,1, '# ========================================================'
printf,1, '#'
printf,1, '#     Bin     R(1)     R(2)     R(size)   S/N    Spaxels'
printf,1, '# --------------------------------------------------------'


if not keyword_set(lam_s2n) then lam_s2n = mean(lambda)

if keyword_set(s2n) then s2n_target = s2n
if keyword_set(s2n) then s2n_str = sctrim(s2n)

if not keyword_set(width) then width=100

lam_str = sctrim(lam_s2n, format='(f7.1)')
wdt_str = sctrim(width)

ok=1
bad_bin=0

while rmin le rad_max do begin

   if rmin eq 0 then begin
      next=step
      goto, skip
   endif

   if ok eq 1 then begin
      theta = theta + 180
      color = color + cos(theta/!RADEG)
   endif

   ; ellipse definition
   a = rmin
   b = sqrt( a^2 * ( 1 - epsilon^2) ) 

   com = 'rad_'+sctrim(bin)+' = '+sctrim(a)
   res = execute(com)

   xm = a * cos(phi)
   ym = b * sin(phi)

   xn = xm * cos(ang) - ym * sin(ang)
   yn = xm * sin(ang) + ym * cos(ang)

   ; Fibers subsample
   fact = 1.2
   xmin = -a*fact
   xmax =  a*fact
   ymin = -a*fact
   ymax =  a*fact

   index = where(x ge xmin and x le xmax and y ge ymin and y le ymax)
   id_test = id[index]-1   ; IDL index
   x_test = x[index]
   y_test = y[index]

   ; Extracting loop
   id_in = 0l
   for n=0l, n_elements(id_test)-1  do begin
      xi = -x_test[n]   ; RA definition
      yi =  y_test[n]
      ; rotating subsample to PA
      xx = xi * cos(ang) - yi * sin(ang)
      yy = xi * sin(ang) + yi * cos(ang)
      ; ellipse definition
      d = (xx/a)^2 + (yy/b)^2 
      if d gt 1 then goto, next
      id_ok = id_test[n]
      id_in = [id_in, id_ok]
      next:
   endfor
   nn = id_in[1:*]

   if n_elements(nn) eq 1 then begin
      if  nn lt 0 and rmin lt rad_max then goto, skip
   endif

   ; First bin: case when RMIN = 0
   if bin eq 1 and a_ini eq 0 then begin
      diff = nn
      bin_aper_1 = nn
      rad_0 = a_ini
      goto, extract
   endif

   ; Skipping bin, case when RMIN gt 0
   if bin eq 1 and rmin eq a_ini then begin
      bin_aper_1 = nn
      next=step
      if keyword_set(plot) then plots, xn, yn, thick=3, color=color
      goto, skip
   endif

   ; First bin: case when RMIN gt 0
   if bin eq 1 and rmin gt a_ini then begin
      bin_aper = bin_aper_1 
      diff = exclude_index(nn,bin_aper)
      diff = diff[sort(diff)]
      rad_0 = a_ini
   endif

   ; All other bins
   if bin ge 2 then begin
      com = 'bin_aper = bin_aper_'+sctrim(bin-1)
      res = execute(com)
      diff = exclude_index(nn,bin_aper)
      diff = diff[sort(diff)]

      t1 = n_elements(nn)
      t2 = n_elements(bin_aper) 
   endif

   com = 'bin_aper_'+sctrim(bin)+'=nn'
   res = execute(com)

   extract:

   fibres = n_elements(diff)
   all_flux = fltarr(pixels)

   for i=0l, fibres-1 do begin
      k = diff[i]
      flux = data[*,k]
      all_flux = all_flux + flux
   endfor


   ;----------  S/N calculation

   mm = where(lambda ge lam_s2n-width*0.5 and lambda le lam_s2n+width*0.5)
   ftest = all_flux[mm]

   s2n_region = s2n_rat( ftest, PROXY=proxy )

   if keyword_set(s2n) then begin

      if s2n_region ge s2n_target then begin
         rmin=a
         next=step
      endif

      if s2n_region lt s2n_target and a ge rad_max then bad_bin=1
 
      if s2n_region lt s2n_target and a lt rad_max then begin
         ok=0
         next=1
         goto, skip
      endif


   endif

   ok=1

   ; Spaxels within ellipse
   if keyword_set(plot) then plots, x[diff], y[diff], psym=1, color=color, symsize=0.4
   if keyword_set(plot) then plots, xn, yn, thick=3, color=color

   if keyword_set(ps) then plots, xn, yn, thick=6, color=color

   ; Saving IDs of bin for final structure
   com = 'id_bin'+sctrim(bin)+'=diff'
   res = execute(com)

   rss_data = [[rss_data],[all_flux]]

   com1 = 'rstart = rad_'+sctrim(bin-1)
   com2 = '  rend = rad_'+sctrim(bin)
   res1 = execute(com1)
   res2 = execute(com2)

   rbin = rend-rstart

   if not keyword_set(silent) then $
   print, format='( 4x,i4, 3(4x,f5.1), 2(5x,i4) )', $
      bin, rstart, rend, rbin, s2n_region, fibres

   printf,1, format='( 5x,i4, 3(4x,f5.1), 2(5x,i4) )', $
      bin, rstart, rend, rbin, s2n_region, fibres

   bin = bin + 1
   
   skip:
   
   if not keyword_set(s2n) then next=step

   rmin = rmin+next
   
endwhile

printf,1, ''
close, 1


rss_data = rss_data[*,1:bin-1]

 fbin = sctrim(bin-1)
rzero = sctrim(a_ini)


if not keyword_set(silent) then begin

   print, ''
   print, ' --------------------------------------------------------'

   if bad_bin eq 1 then begin
      print, ''
      print, '  WARNING: Growing aperture exceeds maximum radius, '
      print, '           S/N of last bin below the required target!'
      print, ''
   endif
   print, ''
   print, '       No. of bins:   ', fbin
   print, ''
   if keyword_set(s2n) then $
   print, '        S/N target:   ', s2n_str
   print, '     S/N at lambda:   ', lam_str
   print, '             width:   ', wdt_str
   print, ''

endif


if keyword_set(ps) then pingsoft_psoff

;--------------------------  Output RSS file
writefits, out_rss, rss_data, hdr



;--------------------------  Saving structure
out_str = { $
          rstart:  rzero    ,$
          rbin:    fbin     ,$
          newrss:  out_rss  ,$
          rflux:   rss_data ,$
          lambda:  lambda   $
          }

if not keyword_set(silent) then begin

   print, ''
   print, ' Files written'
   print, ''
   print, '          Extracted RSS:   ', out_rss
   print, '  Radial bins pseudo PT:   ', out_pt
   if keyword_set(ps) then begin
   print, '      Radial spectra PS:   ', ps_plot
   print, '      Extraction FoV PS:   ', ps_FoV
   endif

   if n_params() eq 7 then begin
   print, ''
   print, '    IDL structure saved:   '
   print, ''
   help, out_str, /struc
   print, ''
   endif
   print, '' 

endif

;--------------------------  Radial Plot

if fbin eq 1 then begin
   nrow=1
   xvar = lambda
   yvar1 = rss_data
   yvar2 = yvar1
   goto, uno
endif

vsize = size(rss_data,/dim)
nrow = vsize[1]

if nrow lt 9 then ncol = fix(255./nrow) else ncol = nrow

pmean = fltarr(1)

for i=0, vsize[1]-1 do pmean = [pmean,mean(rss_data[*,i])]
pmean = pmean[1:*]
n1 = where(pmean eq max(pmean))
n2 = where(pmean eq min(pmean))

xvar = lambda
yvar1 = rss_data[*,n1]
yvar2 = rss_data[*,n2]

uno:
if keyword_set(ps) then begin
   pingsoft_pson, ps_plot, 20, 16, font=psfont, axis_thick=pings.axis, encapsulate=1 
   csize = 1.2
   goto,plot_spec
endif

if keyword_set(silent) then goto, fin
window, 2, xs=xwin2_size, ys=ywin2_size
csize = pings.csize


plot_spec:
plot, [0], /nodata, xs=1, ys=1,  $
      xthick=pings.wthick, ythick=pings.wthick,  charsize=csize, $
      xr=[min(xvar)-80,max(xvar)+80], yr=[min(yvar2), max(yvar1)*1.1],  $
      xtitle='Wavelength', ytitle='Flux', title='Radial extraction'

if fbin eq 1 then begin
   loadct, 39, /silent
   oplot, lambda, rss_data, color=250
   goto, unof
endif

for i=0, nrow-1 do begin
   col = 10+ncol*i
   loadct, 42, file=ctfile, /silent
   oplot, lambda, rss_data[*,i], color=col
   loadct, 0, /silent
   xyouts, lam_s2n, mean(rss_data[*,i]), i+1, charsize=pings.csize, align=0.15*i
endfor

unof:
if keyword_set(ps) then pingsoft_psoff


fin:
if file_test('tmp.pt.txt') eq 1 then spawn, 'rm -f tmp.pt.txt'
if file_test('tmp_3D.pt.txt') eq 1 then spawn, 'rm -f tmp_3D.pt.txt'

end

