

pro extract_region, infile, out_str, PT=pt, PREFIX=prefix, EXTENSION=extension, $
                    MIN_FLUX=min_flux, MAX_FLUX=max_flux, LMIN=lmin, LMAX=lmax, $
                    FILTER=filter, BAND=band, CT=ct, DRAW=draw, $
                    CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
                    SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
                    VLINE=vline, LARGE=large, NOBAND=noband

;+
; NAME:
;       EXTRACT_REGION
;
; PURPOSE:
;
;       Extracts the spectra of spaxels or regions selected by hand.
;
; EXPLANATION:
;
;        Extracts the spectra of regions selected by hand. This is a direct
;        implementation of the "Extract region" button of the VIEW_IFS widget
;        or the MIDDLE-click option of VIEW_3D. The script opens a similar
;        visualisation as VIEW_3D, where the user can select spaxels by
;        successive LEFT-clicks on the right window. The same output files
;        are created as in the previous examples.
;        Similar display options as in VIEW_3D.
;
;
; For CALLING SEQUENCE, INPUTS and OPTIONAL KEYWORDS enter:
;
;        IDL> extract_region
;
;
; OUTPUTS:
;
;        Extracted RSS:   PREFIX_rss.fits
;       Position table:   PREFIX_rss.pt.txt
;     Integrated ASCII:   PREFIX_integ.txt
;                 FITS:   PREFIX_integ.fits
;           Postscript:   PREFIX_integ.eps
;          IDL indices:   PREFIX_index.txt
;
;
; REVISION HISTORY:
;
;       Written by:   F. Fabian Rosales-Ortega
;
;       Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-


compile_opt idl2, hidden
on_error, 2


if n_params() lt 1 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "   extract_region, 'OBJECT.fits' [, OUT.str, PT='PosTable.txt', PREFIX='prefix', $"
    print, "                                 EXTENSION=extension, (+ all visual options of VIEW_3D) ]"
    print, ''                
    print, ''
    print, "    'OBJECT.fits':  String of the wavelength calibrated 3D cube or RSS FITS file."
    print, ''
    print, ''
    print, ' OPTIONAL KEYWORDS'
    print, ''
    print, "          OUT.str:  Name of the optional output structure."
    print, ''
    print, "  PT='Ptable.txt':  Name of the position table in the case of an RSS file"
    print, '                    in ASCII format (North-East configuration).'
    print, '                    NOTE: compulsory if not included in the default instruments/setups'
    print, "                    or when the name is not in the 'OBJECT.pt.txt' format."
    print, ''
    print, "           PREFIX:  String with the prefix of the output files, default: 'region'"
    print, ''
    print, '        EXTENSION:  Non-negative scalar integer specifying the FITS extension to read.'
    print, '                    For example, specify EXTENSION = 1 to read the first FITS extension.'
     print, ''
    return
endif


;--------------------------  Initial tests

infile = sctrim(infile)

if file_test(infile) eq 0 then begin
    print, ''
    print, 'ERROR: The file:  "'+infile+'"  does not exist!'
    print, ''
    return
endif

if not keyword_set(prefix) then prefix='region'
prefix = sctrim(prefix)

ps_file = prefix+'_FoV.eps'



;--------------------------  PINGSoft core

pingsoft_core, infile, pings, core, ps_file, $
               PT=pt, PREFIX=prefix, EXTENSION=extension, $
               MIN_FLUX=min_flux, MAX_FLUX=max_flux, LMIN=lmin, LMAX=lmax, $
               FILTER=filter, BAND=band, CT=ct, DRAW=draw, PS=ps, FONT=font, PLOT=plot, $
               LINEAR=linear, GAMMA=gamma, LOG=log, ASINH=asinh, $
               LARGE=large, NOEXTRACT=noextract, CALIFA=califa, _EXTRA=extra

; Variables from structure
tag = get_tags(core)
for k=0, n_elements(tag)-1 do begin
   dumb = strmid(tag[k],1)
   com = execute(dumb+' = core.'+dumb)
endfor


;--------------------------  Graphics

lam = lambda
xmin = min(lambda) & xmax = max(lambda)

if ref_ra le 0 then wcs = 0 else wcs = 1
if (wcs) then read_wcs = offset2radec(ref_ra,ref_dec,0,0,/SEXAG)

set_font
window, 2, xs=xwin2_size, ys=ywin2_size
plot, [0], /nodata, xr=[xmin-80,xmax+80], yr=[0, 10], xs=1, ys=1, $
      xthick=pings.wthick, ythick=pings.wthick, position=pings.pos2, charsize=pings.csize, $
      xtitle=pings.xtitle2, ytitle=pings.ytitle2

window, 0, xs=xwin0_size, ys=ywin0_size
plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
      xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, position=pings.pos0, charsize=pings.csize, $
      xtitle=pings.xtitle0, ytitle=pings.ytitle0, $
      title=pings.title0, /noerase, _extra=extra


;--------- Color tables 

if not keyword_set(ct) then ct = 1

spawn, 'echo $PINGSOFT_PATH', path
ctfile = path+'/pingsoft_ct.tbl'

ctables = [4,5,25,27,33,39,41,42]
ntest = where(ctables eq ct)
if ntest[0] ge 0 then full_ct = 1 else full_ct = 0


; Plot spaxels

for i=0l, n_elements(x)-1 do begin

    xcoor =  x[i]
    ycoor =  y[i]

    x_fib = xarr + xcoor
    y_fib = yarr + ycoor

    loadct, ct, file=ctfile, /silent
    polyfill, x_fib, y_fib, color=scale[i]

    loadct, 1, /silent

    if keyword_set(draw) then oplot, x_fib, y_fib, color=pings.dcolor1

endfor


;--------------------------  Extraction

print, ''
print, ''
print, ' PINGSoft: RSS spectra viewer'
print, ' =============================================='
print, ''
print, ' Filter:  '+sband
print, ''
print, ' Move mouse over the mosaic to plot the spectra'
print, ''
print, ' Options:'
print, ''
print, '    LEFT-click: select spaxels'
print, ''
print, " MIDDLE-button: output PREFIX if not previously set (default: 'region')"
print, ''
print, '   RIGHT-click: QUIT'
print, ''

if type eq 2 then begin
   if wcs eq 0 then $
      print, '  Spaxel ID     [RA,Dec] offset' else $
      print, '  Spaxel ID     [RA,Dec] offset        RA             Dec            RA deg     Dec deg'
   print, ''
endif
if type eq 3 then begin
   if wcs eq 0 then $
      print, '    Spaxel       ID      [RA,Dec] offset' else $
      print, '    Spaxel       ID      [RA,Dec] offset         RA             Dec            RA deg     Dec deg'
   print, ''
endif


!MOUSE.button=0

extract = fltarr(1)

while (!MOUSE.button ne 4) do begin

    cursor, xm, ym, /change

    x_min = xm-radius
    x_max = xm+radius
    y_min = ym-radius
    y_max = ym+radius

    index = where(x ge x_min and x le x_max and y ge y_min and y le y_max)

    if n_elements(index) eq 1 then begin
        test = index[0]
        if test lt 0 then goto,next
    endif

    id_ok  = id[index]
    x_test = x[index]
    y_test = y[index]
    d = sqrt((x_test-xm)^2. + (y_test-ym)^2.)
    ind = where(d eq min(d))
    fibre = fix(id_ok[ind]-1,type=3)
    fibre = fibre[0]

    xcoor =  x[fibre]
    ycoor =  y[fibre]

    flux = data[*,fibre]

    if not keyword_set(min_flux) then a_flux = min(flux[indice]) else a_flux = min_flux

    if not keyword_set(max_flux) then y_flux = max(flux[indice])*1.2 else y_flux = max_flux

    wset, 2

    sfibre = sctrim(fibre)
    if type eq 2 then vtitle='!5Spaxel ID: '+sfibre+'!X'

    if type eq 3 then begin
       ncol = xx
       col = (fibre MOD ncol)+1
       row = (fibre / ncol)+1
       scol = sctrim(col)
       srow = sctrim(row)
       vtitle='!5Spaxel: ['+scol+','+srow+']    ID: '+sfibre+'!X'
    endif

    plot, [0], /nodata, xr=[xmin-80,xmax+80], yr=[a_flux,y_flux], xs=1, ys=1, $
          xthick=pings.wthick, ythick=pings.wthick, pos=pings.pos2, charsize=pings.csize, $
          title=vtitle, $
          xtitle=pings.xtitle2, ytitle=pings.ytitle2
    
    if keyword_set(noband) or pings.showband eq 0 then goto, noband
    oplot, lam, res*y_flux, linestyle=pings.bline, color=pings.bcolor
    noband:

    if keyword_set(vline) then begin
       if size(vline,/tname) eq 'STRING' then com=execute(vline)
       loadct, pings.ctline, /silent
       if n_elements(vline) gt 1 then for k=0, n_elements(vline)-1 do $
          plots, [vline[k],vline[k]],[a_flux,y_flux], color=pings.cline, thick=pings.tline $
       else $
          plots, [vline,vline],[a_flux,y_flux], color=pings.cline, thick=pings.tline
       loadct, 1, /silent
    endif

    oplot, lambda, flux[indice], color=pings.fcolor

    next:

    wset, 0
    plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
          xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, pos=pings.pos0, charsize=pings.csize, $
          xtitle=pings.xtitle0, ytitle=pings.ytitle0, $
          title=pings.title0, /noerase, _extra=extra


    if !MOUSE.button eq 1 then begin


       if n_elements(index) eq 1 then begin
          test = index[0]
          if test lt 0 then goto, out_range
       endif
       
       if wcs eq 0 then wcs_coord1 = ''
       if wcs eq 0 then wcs_coord2 = ''
       
       if wcs eq 1 then begin
          
          coord1 = offset2radec( ref_ra, ref_dec, xcoor, ycoor, /SEXAG)
          str_0 = string(coord1[0],format='(i2)')
          str_1 = string(coord1[1],format='(i2)')
          str_2 = string(coord1[2],format='(f4.1)')
          str_3 = string(coord1[3],format='(i2)')
          str_4 = string(coord1[4],format='(i2)')
          str_5 = string(coord1[5],format='(f4.1)')
          wcs_coord1 = str_0+'h '+str_1+'m '+str_2+'s   '+str_3+'d '+str_4+'m '+str_5+'s'
          
          coord2 = offset2radec( ref_ra, ref_dec, xcoor, ycoor )
          ra_deg = coord2[0]
          dec_deg = coord2[1]
          wcs_coord2 = sctrim(ra_deg)+'   '+sctrim(dec_deg)
          
       endif
       
        s_fibre = string(fibre,format='(i6)')
        s_xcoor = string(xcoor,format='(f7.1)')
        s_ycoor = string(ycoor,format='(f7.1)')

        if type eq 2 then $
           print, '    '+s_fibre+'    '+s_xcoor+'  '+s_ycoor+'    '+wcs_coord1+'    '+wcs_coord2
        if type eq 3 then begin
           s_col = string(col,format='(i3)')
           s_row = string(row,format='(i3)')
           print, '  ['+s_col+','+s_row+']   '+s_fibre+'    '+s_xcoor+'  '+s_ycoor+'    '+wcs_coord1+'    '+wcs_coord2
        endif

        ; Extraction
        if full_ct eq 1 then thisCol = 255 else thisCol = 250
        loadct, 39, /silent
        extract = [extract,fibre]
        oplot, xarr+xcoor, yarr+ycoor, color=thisCol, thick=2
        polyfill, xarr+xcoor, yarr+ycoor, /line_fill, color=thisCol, orient=45, spacing=0.1
        loadct, 1, /silent

        out_range:

    endif


    if !MOUSE.button eq 2 then begin
        prefix = ''
        read, prefix, prompt='Enter output PREFIX: '
    endif

    loadct, 1, /silent

endwhile


if n_elements(extract) eq 1 then goto, fin


;--------------------------  New FITS file

extract = extract[1:*]

; ignoring repeated fibers
new = (0)
for k=0l, n_elements(extract)-1 do begin
    test = extract[k]
    for n=k+1,  n_elements(extract)-1 do begin
        if test eq extract[n] then goto, incl
    endfor
    new = [new,test]
    incl:
endfor

new = fix(new[1:*],type=3)

extract = new[sort(new)]

integrated = fltarr(pixels)
rss_data = fltarr(pixels,n_elements(extract))

for i=0l, n_elements(extract)-1 do begin

    index = fix(extract[i],type=3)
    integrated = integrated + data[*,index]

    rss_data[*,i] = data[*,index]

endfor


; Indices in ASCII file
out_index = prefix+'_index.txt'
openw, lun, out_index, /get_lun    
for i=0l, n_elements(extract)-1 do begin
    nn = extract[i]
    printf, lun, extract[i]
endfor
close, lun


; Integrated flux ASCII
out_integ = prefix+'_integ.txt'
openw,3,out_integ
for k=0l, n_elements(lam)-1 do printf,3, k+1, lam[k], integrated[k]
close, 3

; Integrated flux FITS
out_fits = prefix+'_integ.fits'
writefits, out_fits, integrated, hdr

; RSS FITS file
out_rss = prefix+'_rss.fits'
writefits, out_rss, rss_data, hdr

; PT file
out_file = prefix+'_rss.pt.txt'
get_new_pt, extract, pos_table, out_file, /silent


wset, 2
plot, [0], /nodata, xs=1, ys=1,  $
      xthick=pings.wthick, ythick=pings.wthick, position=pings.pos2, charsize=pings.csize, $
      xr=[min(lam)-80,max(lam)+80], yr=[min(integrated), max(integrated)*1.2],  $
      xtitle='Wavelength', ytitle='Flux', title='Integrated spectrum'
loadct, 39, /silent
oplot, lam, integrated, color=250

; PS extracted spectrum
out_ps =  prefix+'_integ.eps'
pingsoft_pson, out_ps, 20, 16, font=psfont, axis_thick=pings.axis, encapsulate=1
loadct, 1, /silent
xvar = lam
yvar = integrated
plot, [0], /nodata, xr=[min(xvar)-80,max(xvar)+80], yr=[min(yvar), max(yvar)*1.2], $
      xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, pos=pings.pos2, charsize=pings.csize*1.2, $
      xtitle='Wavelength', ytitle='Flux', title='Integrated spectrum'
oplot, xvar, yvar, color=100
pingsoft_psoff


; Saving structure
out_str = { $
          newrss:  out_rss ,$
          lambda:  lam  ,$
          finteg:  integrated ,$
          indices: extract $
          }

print, ''
print, ' PINGSoft'
print, ' ===================='
print, ''
print, ' Files written '
print, ''
print, '       Extracted RSS:   ', out_rss
print, '      Position table:   ', out_file
print, '    Integrated ASCII:   ', out_integ
print, '                FITS:   ', out_fits
print, '          Postscript:   ', out_ps
print, '         IDL indices:   ', out_index
print, ''

if n_params() eq 2 then begin
print, ''
print, ' IDL structure saved:   '
print, ''
help, out_str, /struct
print, ''
endif

fin:
if file_test('tmp.pt.txt') eq 1 then spawn, 'rm -f tmp.pt.txt'
if file_test('tmp_3D.pt.txt') eq 1 then spawn, 'rm -f tmp_3D.pt.txt'

end

