
pro extract_slit, infile, x0, y0, xs, ys, PA, out_str, EXTENSION=extension, $
                  PT=pt, PREFIX=prefix, PLOT=plot, PS=ps, $
                  FILTER=filter, BAND=band, CT=ct,  DRAW=draw, FONT=font, $
                  LINEAR=linear, GAMMA=gamma, LOG=log, ASINH=asinh,  _EXTRA=extra


;+
; NAME:
;       EXTRACT_SLIT
;
; PURPOSE:
;
;       Extracts the spectra within a rectangular aperture, resembling a
;       long-slit observation.
;
; EXPLANATION:
;
;       The user can define a rectangular aperture of any physical dimensions
;       (in arcsec) centered at a given position from the reference point (in
;       arcsec) and with a given orientation (defined by the PA). The program
;       then extracts the spaxels which fall within the aperture, generating a
;       new RSS and position table files and the integrated spectrum in ASCII
;       and FITS format. By default, the program opens a FoV visualisation as
;       VIEW_3D, showing the simulated slit and extracted spaxels.
;       If the /PS keyword is set, a Postscript file of the FoV is generated. 
;       The /PLOT and /PS keywords are mutually exclusive. 
;       Similar display options as in VIEW_3D.
;
;
; For CALLING SEQUENCE, INPUTS and OPTIONAL KEYWORDS enter:
;
;        IDL> extract_slit
;
;
; OUTPUTS:
;
;      If PREFIX is set:
;
;              Extracted RSS:   PREFIX_rss.fits
;             Position table:   PREFIX_rss.pt.txt
;           Integrated ASCII:   PREFIX_integ.txt
;                       FITS:   PREFIX_integ.fits
;                 Postscript:   PREFIX_integ.eps
;             Extraction FoV:   PREFIX_FoV.eps                   (only if /PS is set)
;
;
;      If PREFIX is NOT set:
;
;             Extracted RSS:   slit_X0-Y0_PA_XSxYS_rss.fits
;            Position table:   slit_X0-Y0_PA_XSxYS_pt.txt
;          Integrated ASCII:   slit_X0-Y0_PA_XSxYS_integ.txt
;                      FITS:   slit_X0-Y0_PA_XSxYS_integ.fits
;                Postscript:   slit_X0-Y0_PA_XSxYS_integ.eps
;            Extraction FoV:   slit_X0-Y0_PA_XSxYS_FoV.eps       (only if /PS is set)
;
; NOTE:
;
;       Only those spaxels which centers lie within the aperture are
;       extracted, "fractional spaxels" are not considered. 
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-


compile_opt idl2, hidden
on_error, 2

if n_params() lt 6 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "  extract_slit, 'OBJECT.fits', x0, y0, length, width, PA  "
    print, "                     [, OUT.str, PREFIX='prefix', PT='PosTable.txt', EXTENSION=extension, "
    print, "                     /PLOT, /PS, FONT=font, _EXTRA={extra},  "
    print, '                     (+ all visual options of VIEW_3D) ]'
    print, ''
    print, ''
    print, "    'OBJECT.fits':  String of the wavelength calibrated 3D cube or RSS FITS file."
    print, ''
    print, '            x0/y0:  RA/Dec offsets (in arcsec) of the central position of the slit to be simulated.'
    print, ''
    print, '     length/width:  Physical dimensions of the simulated slit (in arcsec). For the PA convention, length > width.'
    print, ''
    print, '               PA:  Position angle of the slit, measured counterclockwise from the North.'
    print, ''
    print, ''
    print, ' OPTIONAL KEYWORDS'
    print, ''
    print, "            OUT.str:  Name of the optional output structure."
    print, ''
    print, "    PT='Ptable.txt':  Name of the position table in the case of an RSS file"
    print, '                      in ASCII format (North-East configuration).'
    print, '                      NOTE: compulsory if not included in the default instruments/setups'
    print, "                      or when the name is not in the 'OBJECT.pt.txt' format."
    print, ''
    print, "             PREFIX:  String with the prefix of the output files. Default: input positions"
    print, ''
    print, '          EXTENSION:  Non-negative scalar integer specifying the FITS extension to read.'
    print, '                      For example, specify EXTENSION = 1 to read the first FITS extension.'
    print, ''
    print, '              /PLOT:  Display the visualisation and the simulated aperture, this is the default.'
    print, ''
    print, '                /PS:  Writes an encapsulated Postscript file of the spaxels visualisation'
    print, '                      and the simulated aperture.'
    print, ''
    print, '               FONT:  Postscript IDL font to be used when /PS is set. Default: 12 (Helvetica)'
    print, ''
    print, "             _EXTRA:  Structure with the _EXTRA tags for user's defined graphics output,"
    print, "                      e.g. _EXTRA={title:'IFS cube'}"
    print, ''
    print, '   + all visual options of VIEW_3D  (valid only if /PLOT or /PS are set)'
    print, ''
    return
endif

;--------------------------  Initial tests

infile = sctrim(infile)

if file_test(infile) eq 0 then begin
    print, ''
    print, 'ERROR: The file:  "'+infile+'"  does not exist!'
    print, ''
    return
 endif


;--------------------------  Routine properties

if keyword_set(plot) and keyword_set(ps) then begin
   print, ''
   print, ' WARNING: keywords PLOT and PS cannot be set at the same time'
   print, ''
endif

if not keyword_set(plot) then plot=1
if keyword_set(ps) then plot=0
if keyword_set(silent) then plot=0 


if not keyword_set(prefix) then begin

   t1 = sctrim(x0, format='(f8.1)')
   t2 = sctrim(y0, format='(f8.1)')
   t3 = sctrim(xs)
   t4 = sctrim(ys)
   t5 = sctrim(PA)

   sprefix = 'slit_x'+t1+'_y'+t2+'_l'+t3+'_w'+t4+'_PA'+t5
   ps_file = sprefix+'_FoV.eps' 

endif

if keyword_set(prefix) then ps_file = prefix+'_FoV.eps'


;--------------------------  PINGSoft core SHIFT

pingsoft_core, infile, pings, core, ps_file, x0, y0, /SHIFT, $
               PT=pt, PREFIX=prefix, $
               MIN_FLUX=min_flux, MAX_FLUX=max_flux, LMIN=lmin, LMAX=lmax, $
               FILTER=filter, BAND=band, CT=ct, DRAW=draw, PS=ps, FONT=font, PLOT=plot, $
               LINEAR=linear, GAMMA=gamma, LOG=log, ASINH=asinh, $
               EFF_RAD=eff_rad, NOEXTRACT=noextract, CALIFA=califa, _EXTRA=extra

; Variables from structure
tag = get_tags(core)
for k=0, n_elements(tag)-1 do begin
   dumb = strmid(tag[k],1)
   com = execute(dumb+' = core.'+dumb)
endfor


; X/Y offsets
x_0 = 0
y_0 = 0


;--------------------------- Slit

xbox = [-1,-1, 1, 1,-1] * xs * 0.5
ybox = [-1, 1, 1,-1,-1] * ys * 0.5



;--------------------------- PA in radians and astro conventions


if PA ge 180 then PA = PA - 180

; correction for RA convenction (inverse sign, -90)
ang = -(PA-90)/!RADEG

xn = xbox*cos(ang) - ybox*sin(ang)
yn = xbox*sin(ang) + ybox*cos(ang)
xn = xn+x_0
yn = yn+y_0


loadct, 39,/silent


;--------------------------- Fibers subsample

xp = xn[sort(xn[0:3])]
yp = yn[sort(yn[0:3])]

fac = 1.5
xmin = x_0+min(xp-x_0)*fac
xmax = x_0+max(xp-x_0)*fac
ymin = y_0+min(yp-y_0)*fac
ymax = y_0+max(yp-y_0)*fac

index = where(x ge xmin and x le xmax and y ge ymin and y le ymax)
id_test = id[index]-1 ; IDL index
x_test = x[index]
y_test = y[index]


; Subsample spaxels
; plots, x_test, y_test, psym=1, color=120, symsize=0.8, thick=2


;--------------------------- Main loop

print, ''
print, 'Extracting spectra...'
id_in = 0l
points_in = 0l

for n=0l, n_elements(id_test)-1  do begin

    xfiber = x_test[n]
    yfiber = y_test[n]

    r = sqrt((x_0-xfiber)^2+(y_0-yfiber)^2)


;--------------------------- Calculating angle
    
      k = 0 & diff = 50
      while diff gt 1 do begin
          angle = -k/!RADEG
          xtest = r*cos(angle) + x_0
          ytest = r*sin(angle) + y_0
          diff = sqrt((xfiber-xtest)^2+(yfiber-ytest)^2)
          angulo = k
          k = k+1
      endwhile
    
      ang = (angulo-pa)/!RADEG
      xg = r*cos(ang) + x_0
      yg = r*sin(ang) + y_0


;--------------------------- Points within the aperture

     xrec = xbox + x_0
     yrec = ybox + y_0

     xd1 = abs(yg-min(xrec))
     xd2 = abs(yg-max(xrec))
    
     yd1 = abs(xg-min(yrec))
     yd2 = abs(xg-max(yrec))
          
    xl = xd1 + xd2
    yl = yd1 + yd2
    
    if xl gt xs then goto, next
    if yl gt ys then goto, next

    id_ok = id_test[n]
    id_in = [id_in, id_ok]
    points_in = points_in+1

    ; Plots selected spaxels
    ;if keyword_set(plot) then plots, xfiber, yfiber, psym=1, color=190, symsize=0.4

    next:

endfor

if keyword_set(plot) then plots, xn, yn, thick=3, color=250
if keyword_set(ps) then plots, xn, yn, thick=7, color=250


id_in = id_in[1:*]

extract = id_in

if keyword_set(ps) then pingsoft_psoff


;--------------------------- Extracting spectra

all_flux = fltarr(pixels)
rss_data = fltarr(pixels,n_elements(id_in))

for i=0l, n_elements(id_in)-1 do begin
    id = id_in[i]
    flux  = data[*,id]
    all_flux = all_flux + flux
    rss_data[*,i]=flux
endfor

integ_flux = all_flux

nice_x = lambda
nice_y = integ_flux

xmin = min(nice_x) & xmax = max(nice_x)
ymin = min(nice_y) & ymax = max(nice_y)*1.2

loadct, 1, /silent
if keyword_set(plot) then begin
    window, 2
    plot, [0], /nodata, xr=[xmin-80,xmax+80], yr=[ymin,ymax], xs=1, ys=1, $
          xthick=pings.wthick, ythick=pings.wthick, position=pings.pos2, charsize=pings.csize, $
          title='Integrated flux', xtitle='Wavelength', ytitle='Flux'
    loadct, 39, /silent
    oplot, lambda, integ_flux, color=250
endif


;-------------------------- Outputs

; Integrated flux ASCII
if not keyword_set(prefix) then $
  out_integ = sprefix+'_integ.txt' else $
  out_integ = prefix+'_integ.txt'

openw,3,out_integ
for k=0, n_elements(integ_flux)-1 do printf,3, k+1, nice_x[k], nice_y[k]
close, 3

; Integrated flux FITS
if not keyword_set(prefix) then $
  out_fits = sprefix+'_integ.fits' else $
  out_fits = prefix+'_integ.fits'

writefits, out_fits, integ_flux, hdr

; RSS FITS file
if not keyword_set(prefix) then $
  out_rss = sprefix+'_rss.fits' else $
  out_rss = prefix+'_rss.fits'

writefits, out_rss, rss_data, hdr2

; Position table
if not keyword_set(prefix) then $
  out_file = sprefix+'_rss.pt.txt' else $
  out_file = prefix+'_rss.pt.txt'

get_new_pt, id_in, pos_table, out_file, /silent


; PS extracted spectrum
if not keyword_set(prefix) then $
   out_ps =  sprefix+'_integ.eps'  else $
   out_ps =  prefix+'_integ.eps'
pingsoft_pson, out_ps, 20, 16, font=psfont, axis_thick=pings.axis, encapsulate=1
loadct, 1, /silent
xvar = lambda
yvar = integ_flux
plot, [0], /nodata, xr=[min(xvar)-80,max(xvar)+80], yr=[min(yvar), max(yvar)*1.2], $
      xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, pos=pings.pos2, charsize=pings.csize*1.2, $
      xtitle='Wavelength', ytitle='Flux', title='Integrated spectrum'
oplot, xvar, yvar, color=100
pingsoft_psoff


; Saving structure
out_str = { $
          newrss:  out_rss ,$
          lambda:  lambda  ,$
          finteg:  integ_flux ,$
          indices: extract $
          }

print, ''
print, ' PINGSoft'
print, ' ===================='
print, ''
print, ' Files written '
print, ''
print, '       Extracted RSS:   ', out_rss
print, '      Position table:   ', out_file
print, '    Integrated ASCII:   ', out_integ
print, '                FITS:   ', out_fits
print, '          Postscript:   ', out_ps

if keyword_set(ps) then $
print, '      Extraction FoV:   ', ps_file
print, ''

if n_params() eq 7 then begin
print, ''
print, ' IDL structure saved:   '
print, ''
help, out_str, /struc
print, ''
endif

if file_test('tmp.pt.txt') eq 1 then spawn, 'rm -f tmp.pt.txt'
if file_test('tmp_3D.pt.txt') eq 1 then spawn, 'rm -f tmp_3D.pt.txt'

fin:
end
