
pro get_new_pt, index, in_file, out_file, SILENT=silent


;+
; NAME:
;       GET_NEW_PT
;
; PURPOSE:
;
;       Generates a new position table based on an index of selected spaxels.
; 
;
; EXPLANATION:
;
;       Given a position table and the index of selected spaxels, this program
;       generates a new position table including only the position of those
;       spaxels. This is a subroutine of extract_index.pro.
;
; CALLING SEQUENCE:
;       
;       get_new_pt, index, 'in_PosTable.txt', 'out_PosTable.txt' [, /SILENT ]
;
; INPUTS:
;
;              index:  Integer vector index of the spaxels to be included in the new position table.
;       
;  'in_PosTable.txt':  String of the original position table.
;
; 'out_PosTable.txt':  Name of the new position table.
;
;            /SILENT:  Omits additional information on screen
;
; OUTPUTS:
;
;       A new position table with the position of the selected spaxels.
;
; EXAMPLES:
;       
;       index = [188,191,218,227,239,244,246,252,255,259,$             ; Indices of selected spaxels
;                269,271,278,290,292,295,302,307,310,312]               
;
;       get_new_pt, index, 'original_pt.txt', 'final_pt.txt', /SILENT
;
;
; REVISION HISTORY:
;
;       Written by:     F. Fabian Rosales-Ortega
;
;       First version:  January, 2010
;
; NOTE:
;
;       This routine belongs to the PINGS IDL software available at:
;
;       http://www.ast.cam.ac.uk/research/pings
;
;       where additional documentation can be found. If you find this code
;       useful on your research, please acknowledge the use of PINGSoft by
;       including the original project reference in your publications.
;
; COPYRIGHT:
;
;       This file is part of PINGSoft.
;
;       Copyright (C) 2010  F. Fabian Rosales-Ortega
;
;       PINGSoft is free software: you can redistribute it and/or modify it
;       under the terms of the GNU General Public License as published by the
;       Free Software Foundation, version 3. See the GNU General Public License
;       for more details.
;
;       PINGSoft is distributed in the hope that it will be useful, but WITHOUT
;       ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;       FITNESS FOR A PARTICULAR PURPOSE. 
;
;       The GNU General Public License is found in: http://www.gnu.org/licenses
;
;-


compile_opt idl2, hidden
on_error, 2


if n_params() lt 3 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "    get_new_pt, index, 'in_PosTable.txt', 'out_PosTable.txt' [, /SILENT ]"
    print, ''
    print, ''
    print, '                 index:  Integer vector index of the selected spaxels to'
    print, '                         be included in the new position table.'
    print, ''
    print, "     'in_PosTable.txt':  String of the original position table."
    print, ''
    print, "    'out_PosTable.txt':  Name of the new position table."
    print, ''
    print, '               /SILENT:  Omits additional information on screen.'
    print, ''
    return
endif


;--------------------------  Initial tests


 in_file = sctrim(in_file)
out_file = sctrim(out_file)

if file_test(in_file) eq 0 then begin
    print, ''
    print, 'ERROR: The file:  "'+in_file+'"  does not exist!'
    print, ''
    return
endif



; Spaxel type (reading first line of the position table)

readcol, in_file, f='a,f,f,i', mtype, xsize, ysize, numline=1, /silent


; Reading the offsets

readcol, in_file, f='l,f,f,i', id, x, y, flag, /silent

xx = x[index]
yy = y[index]
ff = flag[index]

openw, lun, out_file, /get_lun

if mtype eq 'C' then printf, lun, 'C  '+sctrim(xsize)+'  '+sctrim(ysize)+'  0'
if mtype eq 'S' then printf, lun, 'S  '+sctrim(xsize)+'  '+sctrim(ysize)+'  0'

for i=0, n_elements(index)-1 do begin

    str = string(i+1, xx[i], yy[i], ff[i])
    printf, lun, strtrim(str, 1)

endfor

close, lun
free_lun, lun

if not keyword_set(silent) then begin
    print, ''
    print, 'File created: ', out_file
    print, ''
endif

end
