

pro integrate_3d, infile, out_str, PREFIX=prefix, BAD_ROWS=bad_rows, MASK=mask, $
                  NORMALIZE=normalize, SILENT=silent, NO_OUTPUT=no_output


;+
; NAME:
;       INTEGRATE_3D
;
; PURPOSE:
;
;       Integrates the spectra within a 3D cube or RSS file into a single spectrum.
;
; EXPLANATION:
;
;       The program generates an ASCII and a FITS file of the integrated
;       spectrum. By default, all the spectra in the 3D cube or RSS file are
;       considered for the integration. This can be changed by setting the MASK
;       (3D cube) or BAD_ROWS (RSS) keywords. In the first case, the mask should
;       be a 2D FITS image with values 0 and 1, those regions in the mask
;       image equal to 0 are discarded, while regions with values equal to 1 are
;       considered for the spectra integration. The 2D mask image must have the
;       same X-Y dimensions as the input cube. In the second case, the string
;       passed to BAD_ROWS should be an ASCII file containing the indices of the
;       spaxels to be neglected. As an output, the integrated spectrum is
;       displayed on screen, and a encapsulated Postscript file is created,
;       unless the /SILENT keyword is set.
;
;
; For CALLING SEQUENCE, INPUTS and OPTIONAL KEYWORDS enter:
;
;        IDL> integrate_3D
;
;
; OUTPUTS:
;
;     If PREFIX is set:
;
;       Integrated ASCII:   PREFIX_integ.txt
;                   FITS:   PREFIX_integ.fits
;             Postscript:   PREFIX_integ.eps
;
;     If PREFIX is NOT set:
;
;       Integrated ASCII:   integ_3D.txt
;                   FITS:   integ_3D.fits
;             Postscript:   integ_3D.eps
;
;
; COMMENTS
;
;        NOTE: ASCII and FITS output are in input units (e.g. 10^-16 erg s^-1 cm^-2 AA^-1 for CALIFA)
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-



compile_opt idl2, hidden
on_error, 2



if n_params() lt 1 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "  integrate_3D, 'OBJECT.fits' [, OUT.str, PREFIX='prefix', BAD_ROWS='bad_rows.txt', "
    print, "                              MASK='mask.fits', NORMALIZE=normalize, /SILENT ]"
    print, ''
    print, ''
    print, "      'OBJECT.fits':  String of the wavelength calibrated 3D cube or RSS FITS file."
    print, ''
    print, ''
    print, '  OPTIONAL KEYWORDS:'
    print, ''
    print, "            OUT.str:  Name of the optional output structure."
    print, ''
    print, "             PREFIX:  String with the prefix of the output files."
    print, ''
    print, '               MASK:  String of a 2D FITS image with values 0 and 1, those regions in'
    print, '                      the mask image equal to 0 are discarded, while regions with values'
    print, '                      equal to 1 are considered for the spectra integration. The 2D mask'
    print, '                       image must have the same X-Y dimensions as the input cube.'
    print, ''
    print, '           BAD_ROWS:  String of an ASCII file with the indices of rows to be discarded '
    print, '                      if the input is a RSS file (indices in IDL format).'
    print, ''
    print, '          NORMALIZE:  Value of the wavelength where the flux is to be normalized.'
    print, ''
    print, '            /SILENT:  Omits printing output information on the terminal'
    print, ''
    print, '      NOTE: ASCII and FITS output in input units (10^-16 erg s^-1 cm^-2 A^-1 for CALIFA)'
    print, ''
    return
endif

;--------------------------  Initial tests


infile = sctrim(infile)

if file_test(infile) eq 0 then begin
    print, ''
    print, 'ERROR: The file:  "'+infile+'"  does not exist!'
    print, ''
    return
endif

if not keyword_set(font) then font=3
sfont = sctrim(font)


;--------------------------  Graphic device settings
pingsoft_param, pings, ''
pingsoft_display, pings, /silent


;--------------------------  Reading data

fits = readfits(infile,hdr,/silent)
size = size(fits)
type = size[0]

if type lt 1 or type gt 3 then begin
   print, ''
   print, ' ERROR: input FITS is not a 3D cube or RSS valid file'
   print, ''
   return
endif


; Assuming data in arcsec
mas = 0

; Single FITS (for redshift_3D, etc.)

if type eq 1 then begin

   pixels = size[1]
   rows = 1

   crval = fxpar(hdr,'CRVAL1')
   cdelt = fxpar(hdr,'CDELT1')
   crpix = fxpar(hdr,'CRPIX1')

   ref_ra  = fxpar(hdr,'RA')
   ref_dec = fxpar(hdr,'DEC')

   if crpix le 0 then begin
      print, ''
      print, 'WARNING: value of CRPIX1 in FITS header is 0 or negative, is this OK?'
      print, ''
   endif

   data = fits

endif


; RSS type
if type eq 2 then begin

   pixels = size[1]
   rows   = size[2]

   crval = fxpar(hdr,'CRVAL1')
   cdelt = fxpar(hdr,'CDELT1')
   crpix = fxpar(hdr,'CRPIX1')

   ref_ra  = fxpar(hdr,'RA')
   ref_dec = fxpar(hdr,'DEC')

   if crpix le 0 then begin
      print, ''
      print, 'WARNING: value of CRPIX1 in FITS header is 0 or negative, is this OK?'
      print, ''
   endif

   data = fits

   if keyword_set(califa) then begin
      ; Badpix map
      bpix = ext_bpix
      ; Position table
      xpt = tbget(hdr_pt, ext_pt, 1)
      ypt = tbget(hdr_pt, ext_pt, 2)
      shape = fxpar(hdr_pt,'FIBSHAPE')
      pt_sizex = fxpar(hdr_pt,'FIBSIZEX')
      pt_sizey = fxpar(hdr_pt,'FIBSIZEy')
      header = shape+string(pt_sizex)+string(pt_sizey)+'  0'
      xpt = -xpt
      openw,1,'tmp_3D.pt.txt'
      printf,1,header
      for i=0,n_elements(xpt)-1 do printf,1, string(i+1), xpt[i], ypt[i], 0
      close,1
      pt = 'tmp_3D.pt.txt'
   endif


   ; FITS header for RSS data and 1D FITS

   hdr1 = hdr
   hdr3 = hdr
   
   sxdelpar, hdr1, 'CD1_1'
   sxdelpar, hdr1, 'CD2_2'
   sxdelpar, hdr1, 'CTYPE1'

   sxdelpar, hdr1, 'EXTEND'
   sxdelpar, hdr3, 'EXTEND'

   hdr2 = hdr

endif


; CUBE type
if type eq 3 then begin

   ; Masking
   ;
   if keyword_set(mask) then begin
      if file_test(mask) eq 0 then begin
         print, ''
         print, 'ERROR: The file:  "'+mask+'"  does not exist!'
         print, ''
         return
      endif
      test_mask = readfits(mask,/silent)
      mask_size = size(test_mask)
      if mask_size[1] ne size[1] or mask_size[2] ne size[2] then begin
         print, ''
         print, 'ERROR: input cube and mask dimensions do not match'
         print, ''
         return
      endif
      if min(test_mask) ne 0 or max(test_mask) ne 1 then begin
         print, ''
         print, 'ERROR: Invalid format, mask values should be 0: null, 1: valid'
         print, ''
         return
      endif
      extract_mask, infile, mask, prefix='tmp', /silent
      fits = readfits('tmp_mask.cube.fits', /silent)
      smask = mask
   endif

   xx = size[1]
   yy = size[2]

   rows = xx*yy

   crval  = fxpar(hdr,'CRVAL3')
   cdelt  = fxpar(hdr,'CDELT3')
   crpix  = fxpar(hdr,'CRPIX3')
   pixels = fxpar(hdr,'NAXIS3')

   if crpix le 0 then begin
      print, ''
      print, 'WARNING: value of CRPIX3 in FITS header is 0 or negative, is this OK?'
     print, ''
   endif

   data = fltarr(pixels,rows)
   if keyword_set(califa) then bpix = fltarr(pixels,rows)

   ; New vector
   id = long(0)
   for j=0l, yy-1 do begin

      for i=0l, xx-1 do begin

         array = fits[i,j,*]
         array = transpose(array)

         ; NaN and Inf step 1
         mask = finite(array)
         array = array*mask
         nan = where(array ne array)
         if n_elements(nan) gt 1 then array[nan] = 0.
         if n_elements(nan) eq 1 and nan[0] ge 0 then array[nan] = 0.
         ; NaN and Inf step 2
         inf = where(array lt -1e10 or array gt 1e10)
         if n_elements(inf) gt 1 then array[inf] = 0.
         if n_elements(inf) eq 1 and inf[0] ge 0 then array[inf] = 0.
       
         data[*,id] = array
       
         if keyword_set(califa) then begin
            array_bp = ext_bpix[i,j,*]
            array_bp = transpose(array_bp)
            bpix[*,id] = array_bp
         endif

         id = id+1

      endfor

      
   endfor

   cdelt1 = abs(fxpar(hdr,'CDELT1'))
   cdelt2 = abs(fxpar(hdr,'CDELT2'))
   crpix1 = fxpar(hdr,'CRPIX1')
   crpix2 = fxpar(hdr,'CRPIX2')
   naxis1 = fxpar(hdr,'NAXIS1')
   naxis2 = fxpar(hdr,'NAXIS2')

    RA = fxpar(hdr,'CRVAL1')
   Dec = fxpar(hdr,'CRVAL2')

   ref_ra  = RA
   ref_Dec = Dec

   ; Milli-arcseconds?
   if cdelt1 lt 1 or cdelt2 lt 1 then begin
      num = sctrim(cdelt1, format='(e)')
      exp = strsplit(num[0],'e',/extract,/regex)
      if exp[1] eq '-06' then begin
         cdelt1 = cdelt1 * 1e6
         cdelt2 = cdelt2 * 1e6
         mas = 1
      endif
   endif

   if cdelt1 eq 0 or cdelt2 eq 0 then resolution = 1 else resolution = cdelt1
   
   if crpix1 ne 0 then  RA_off = double(crpix1*resolution) else  RA_off = 0
   if crpix2 ne 0 then Dec_off = double(crpix2*resolution) else Dec_off = 0
   
   if  RA_off eq 0 then  RA_off = xx*resolution*0.5
   if Dec_off eq 0 then Dec_off = yy*resolution*0.5

   rectangular_pt, xx, yy, resolution, 1, xpt, ypt, off, $
                   X_OFF=RA_off, Y_OFF=Dec_off, OUT_PT='tmp_3D.pt.txt' ;, /PPAK

   pt = 'tmp_3D.pt.txt'

   ; FITS header for RSS data and 1D FITS

   hdr1 = hdr
   hdr3 = hdr

   ; hdr1
   fxaddpar, hdr1, 'CRVAL1', crval, ' updated by PINGSoft'
   fxaddpar, hdr1, 'CDELT1', cdelt, ' updated by PINGSoft'
   fxaddpar, hdr1, 'CRPIX1', crpix, ' updated by PINGSoft'
   fxaddpar, hdr1,     'RA',    RA, ' updated by PINGSoft'
   fxaddpar, hdr1,    'DEC',   Dec, ' updated by PINGSoft'
   sxdelpar, hdr1, 'CRVAL2'
   sxdelpar, hdr1, 'CRVAL3'
   sxdelpar, hdr1, 'CRPIX2'
   sxdelpar, hdr1, 'CRPIX3'
   sxdelpar, hdr1, 'CDELT2'
   sxdelpar, hdr1, 'CDELT3'
   sxdelpar, hdr1, 'DISPAXIS'
   sxdelpar, hdr1, 'WCSDIM'
   sxdelpar, hdr1, 'CTYPE1'
   sxdelpar, hdr1, 'CD1_1'
   sxdelpar, hdr1, 'CD2_2'
   sxdelpar, hdr1, 'CD1_2'
   sxdelpar, hdr1, 'CD2_1'

   sxdelpar, hdr1, 'EXTEND'
   sxdelpar, hdr3, 'EXTEND'

   hdr2 = hdr1

endif


if ref_ra le 0 then wcs = 0 else wcs = 1

read_wcs = offset2radec(ref_ra,ref_dec,0,0,/SEXAG) ; useless, just compiling functions

if cdelt eq 0 then begin
   print, ''
   print, ' WARNING: CDELT entry not found, using CDELT=1'
   print, ''
   cdelt = 1
endif

lambda = dblarr(pixels)
for i=0, pixels-1 do lambda[i] = (crval + i*cdelt)-(crpix-1)*cdelt

; Data in microns?
if mean(lambda) lt 100 then lambda = lambda*1e4

; Data in mas?
if mas eq 1 then wcs = 0

lam = lambda

if not keyword_set(lmin) then lmin = min(lambda)

if not keyword_set(lmax) then lmax = max(lambda)

indice = where(lambda ge lmin and lambda le lmax)

lambda = lambda[indice]

xmin = min(lambda) & xmax = max(lambda)


!p.multi=[0,1,1]


;--------------------------  Bad rows: RSS type

good = intarr(1)

if keyword_set(bad_rows) and type eq 2 then begin

    readcol, bad_rows, f='l', bad, /silent
    
    for i=0, rows-1 do begin
        for k=0, n_elements(bad)-1 do begin
            if bad[k] eq i then goto, mas
        endfor
        good = [good,i]
        mas:
    endfor
    goto, bad_rows
endif

for i=0, rows-1 do good = [good,i]

bad_rows:
good = good[1:*]
final = data[*,good]



;-------------------------------------  Integrated Flux

all_flux = fltarr(pixels)
for i=0, n_elements(good)-1 do begin
    id = good[i]
    flux  = data[*,id]
    all_flux = all_flux + flux
endfor


if keyword_set(normalize) then begin
    anchor = where(lambda ge normalize)
    norm = min(anchor)
    f_norm = all_flux[norm]
endif else begin
    f_norm = 1.
endelse

integ_flux = all_flux/f_norm



;-------------------------------------  Output

if not keyword_set(no_output) then begin

   ; Integrated flux FITS
   if not keyword_set(prefix) then $
      out_fits = 'integ_3D.fits' else $
         out_fits = prefix+'_integ.fits'
   
   writefits, out_fits, integ_flux, hdr1

   ; Integrated flux ASCII
   if not keyword_set(prefix) then $
      out_ascii = 'integ_3D.txt' else $
         out_ascii = prefix+'_integ.txt'

   openw,1, out_ascii
   for i=0, n_elements(lambda)-1 do begin
      printf, 1, i+1,lambda[i], integ_flux[i]
   endfor
   close,1

endif


;-------------------------------------  Output structure

out_str = { $
          lambda: lambda     ,$
          finteg: integ_flux  $
          }

;-------------------------------------  Plot

; PS integrated spectrum

if not keyword_set(prefix) then $
   out_ps = 'integ_3D.eps' else $
      out_ps =  prefix+'_integ.eps'

xvar = lambda
yvar = integ_flux
pingsoft_pson, out_ps, 20, 16, font=psfont, axis_thick=pings.axis, encapsulate=1
loadct, 1, /silent
plot, [0], /nodata, xr=[min(xvar)-80,max(xvar)+80], yr=[min(yvar), max(yvar)*1.2], $
      xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, pos=pings.pos2, charsize=pings.csize*1.2, $
      xtitle='Wavelength', ytitle='Flux', title='Integrated spectrum'
oplot, xvar, yvar, color=100
pingsoft_psoff


if not keyword_set(silent) then begin

   window, 0, xs=xwin2_size, ys=ywin2_size
   plot, [0], /nodata, xs=1, ys=1,  $
         xthick=pings.wthick, ythick=pings.wthick, position=pings.pos2, charsize=pings.csize, $
         xr=[min(xvar)-80,max(xvar)+80], yr=[min(yvar), max(yvar)*1.2],  $
         xtitle='Wavelength', ytitle='Flux', title='Integrated spectrum'
   loadct, 39, /silent
   oplot, xvar, yvar, color=250
   
   if not keyword_set(no_output) then begin
         
      print, ''
      print, ' PINGSoft'
      print, ' ==================='
      print, ''
      print, ' Files written '
      print, ''
      print, '   Integrated ASCII:   ', out_ascii
      print, '               FITS:   ', out_fits
      print, '         Postscript:   ', out_ps
      print, ''
      if keyword_set(smask) then $
      print, '       Mask applied:   ', smask
      print, ''

      if n_params() eq 2 then begin
         print, ''
         print, ' IDL structure saved:   '
         print, ''
         help, out_str, /struct
       print, ''
    endif
   endif

endif


end
