
function offset2radec, RA_ref, Dec_ref, X_off, Y_off, SEXAG=sexag

;+
; NAME:
;       OFFSET2RADEC
;
; PURPOSE:
;
;       Transforms small angle offsets in arcsec from a reference point to
;       equatorial coordinates.
;
; CALLING SEQUENCE:
;       
;       result = offset2radec(RA_ref, Dec_ref, X_off, Y_off, /SEXAG)
;
; INPUTS:
;       
;       RA_ref/Dec_ref:  Equatorial coordinates of the reference point in DEGREES.
;
;          X_off/Y_off:  Offset from the reference point in ARCSEC.
;
;
; OPTIONAL KEYWORD:
;   
;               /SEXAG:  Output in sexagesimal units (default: degrees).
;
; OUTPUTS:
;
;       A vector with 2 numbers containing the RA,Dec coordinates in DEGREES:
;
;       result = [ RA, Dec ]
;
;       if the /SEXAG parameter is set, then a vector with 6 numbers:
;
;       result = [ RA_hr,RA_min,RA_sec, Dec_deg,Dec_min,Dec_sec ]
;
; EXAMPLES:
;       
;       Equatorial coordinates of the reference point (0,0): RA=23.501865, Dec=15.528838
;
;       Offset from the reference point: (15,23)
;
;       IDL> coord = offset2radec( 23.501865, 15.528838, 15, 23)
;
; RESTRICTIONS:
;
;       The value of RA is an approximation, valid within 1 degree.
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-



compile_opt idl2, hidden
on_error, 2



if n_params() lt 4 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, '    result = offset2radec(RA_ref, Dec_ref, X_off, Y_off, /SEXAG)'
    print, ''
    print, ''
    print, '    RA_ref/Dec_ref:  coordinates of the reference point in DEGREES'
    print, ''
    print, '       X_off/Y_off:  offset from the reference point in ARCSEC'
    print, ''
    print, '            /SEXAG:  Output in sexagesimal units (default: degrees).'
    print, ''
    print, ''
    print, ' OUTPUT: a vector with 2 numbers containing the RA,Dec coordinates in DEGREES:'
    print, ''
    print, '         result = [ RA, Dec ]'
    print, ''
    print, '         if the /SEXAG parameter is set, then a vector with 6 numbers:'
    print, ''
    print, '         result = [ RA_hr,RA_min,RA_sec, Dec_deg,Dec_min,Dec_sec ]'
    print, ''
    print, ''
    print, ' NOTE: The value of RA is an approximation, valid within 1 degree.'
    print, ''
    goto, fin
endif


;------------  Dec

dec0 = Dec_ref

dDec = y_off/3600d

dec = dDec + dec0

mean_Dec = mean([dec,dec0])*!pi/180.


;------------  RA approximation
    
ra0 = RA_ref 

dRA = x_off/(3600d*cos(mean_Dec)) ; valid within ~ 1 degree

ra = dRA + ra0


if keyword_set(SEXAG) then begin

    radec, ra, dec, RA_hr, RA_min, RA_sec, Dec_deg, Dec_min, Dec_sec

    ra = [ RA_hr, RA_min, RA_sec ]
    dec = [ Dec_deg, Dec_min, Dec_sec ]

endif

res = [ra,dec]


return, res
fin:
end
