

pro pingsoft_pson, output, xx, yy, FONT=font, AXIS_THICK=axis_thick, $
                   PFONT=pfont, LANDSCAPE=landscape, ENCAPSULATE=encapsulated


;+
;  PURPOSE:
;             Switch over to Postsctipt device, it sets by default the hardware
;             font (!p.font=0), and it gives the user the opportunity to define 
;             the name of the output PS file and the x-y size of the plot, and
;             the font to use. For vector fonts set PFONT=-1
;             
;
;  INPUTS:
;
;             output = string with the name of the output PS file
;
;  OPTIONAL KEYWORDS:
;
;             xx = size of the plot in the x-axis in cm
;
;             yy = size of the plot in the y-axis in cm
;
;                  The default parameters for xx, yy produce a square
;                  plot, suitable for publication in papers
;
;
;           NOTE: In order to close the device, produce the file and come back
;           to 'X' device, you have to either type:
;
;              IDL> device, /close
;              IDL> set_plot,"x"
;
;           or use a sort of PSOFF command (see my example).
;
;
;  This routine belongs to the PINGS IDL software available at:
;
;  http://www.ast.cam.ac.uk/research/pings
;
;  where additional documentation can be found. If you find this code
;  useful on your research, please acknowledge the use of PINGSoft by
;  including the original project reference in your publications.
;
;
;  Copyright (C) 2010  F. Fabian Rosales-Ortega
;
;  PINGSoft is free software: you can redistribute it and/or modify it
;  under the terms of the GNU General Public License as published by the
;  Free Software Foundation, version 3. See the GNU General Public License
;  for more details.
;
;  PINGSoft is distributed in the hope that it will be useful, but WITHOUT
;  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;  FITNESS FOR A PARTICULAR PURPOSE. 
;
;  The GNU General Public License is found in: http://www.gnu.org/licenses
;
;-


compile_opt idl2, hidden
on_error, 2
if n_params() lt 1 then begin
   print, ''
   print, " pingsoft_pson, 'output_name' [, x_size, y_size, FONT=font, AXIS_THICK=axis_thick, PFONT=pfont, /ENCAPSULATE, /LANDSCAPE ]  "
   print, ''
   print, ''
   print, ' Defaults: FONT=12, AXIS_THICK=3'
   print, ''
   print, ' Hardware fonts '
   print, ''
   print, '   0: AvantGarde-Book'
   print, '   1: AvantGarde-BookOblique'
   print, '   2: AvantGarde-Demi'
   print, '   3: AvantGarde-DemiOblique'
   print, '   4: Bookman-Demi'
   print, '   5: Bookman-DemiItalic'
   print, '   6: Bookman-Light'
   print, '   7: Bookman-LightItalic'
   print, '   8: Courier'
   print, '   9: Courier-Bold'
   print, '  10: Courier-BoldOblique'
   print, '  11: Courier-Oblique'
   print, '  12: Helvetica'
   print, '  13: Helvetica-Bold'
   print, '  14: Helvetica-BoldOblique'
   print, '  15: Helvetica-Narrow'
   print, '  16: Helvetica-Narrow-Bold'
   print, '  17: Helvetica-Narrow-BoldOblique'
   print, '  18: Helvetica-Narrow-Oblique'
   print, '  19: Helvetica-Oblique'
   print, '  20: NewCenturySchlbk-Bold'
   print, '  21: NewCenturySchlbk-BoldItalic'
   print, '  22: NewCenturySchlbk-Italic'
   print, '  23: NewCenturySchlbk-Roman'
   print, '  24: Palatino-Bold'
   print, '  25: Palatino-BoldItalic'
   print, '  26: Palatino-Italic'
   print, '  27: Palatino-Roman'
   print, '  28: Symbol'
   print, '  29: Times-Bold'
   print, '  30: Times-BoldItalic'
   print, '  31: Times-Italic'
   print, '  32: Times-Roman'
   print, '  33: ZapfChancery-MediumItalic'
   print, ''
   return
endif



; PS Size
if not keyword_set(xx) then xx = 25.
if not keyword_set(yy) then yy = 20.


; Set PS device
set_plot,"ps"


; PS size
if n_params() gt 1 then device, xsize=xx, ysize=yy


; PS pfont
if not keyword_set(pfont) then begin
   !p.font=0
endif else begin
   if pfont eq -1 or pfont eq 1 then begin
      !p.font=pfont
   endif else begin
      print, 'No valid PFONT value'
      print, ''
      print, '-1: Hershey vector-drawn fonts (default)'
      print, ' 0: Device fonts (supplied by the graphic device)'
      print, ' 1: TrueType fonts'
      return
   endelse
endelse


; PS name
device, /color, /isolatin1, Bits_per_Pixel=8, language_level=2, filename=output


; Define fonts

if not keyword_set(font) then font = 12

case font of
   0: tfont='AvantGarde-Book'
   1: tfont='AvantGarde-BookOblique'
   2: tfont='AvantGarde-Demi'
   3: tfont='AvantGarde-DemiOblique'
   4: tfont='Bookman-Demi'
   5: tfont='Bookman-DemiItalic'
   6: tfont='Bookman-Light'
   7: tfont='Bookman-LightItalic'
   8: tfont='Courier'
   9: tfont='Courier-Bold'
  10: tfont='Courier-BoldOblique'
  11: tfont='Courier-Oblique'
  12: tfont='Helvetica'
  13: tfont='Helvetica-Bold'
  14: tfont='Helvetica-BoldOblique'
  15: tfont='Helvetica-Narrow'
  16: tfont='Helvetica-Narrow-Bold'
  17: tfont='Helvetica-Narrow-BoldOblique'
  18: tfont='Helvetica-Narrow-Oblique'
  19: tfont='Helvetica-Oblique'
  20: tfont='NewCenturySchlbk-Bold'
  21: tfont='NewCenturySchlbk-BoldItalic'
  22: tfont='NewCenturySchlbk-Italic'
  23: tfont='NewCenturySchlbk-Roman'
  24: tfont='Palatino-Bold'
  25: tfont='Palatino-BoldItalic'
  26: tfont='Palatino-Italic'
  27: tfont='Palatino-Roman'
  28: tfont='Symbol'
  29: tfont='Times-Bold'
  30: tfont='Times-BoldItalic'
  31: tfont='Times-Italic'
  32: tfont='Times-Roman'
  33: tfont='ZapfChancery-MediumItalic'
endcase

device, set_font=tfont

; Axis thickness
if not keyword_set(axis_thick) then axist=3 else axist=axis_thick

!x.thick=axist
!y.thick=axist
!y.thick=axist

; EPS
if keyword_set(encapsulated) then device,/encapsulated

; Landscape?
if keyword_set(landscape) then device, /landscape

end
