

;+
; NAME:
;       RADEC2OFFSET
;
; PURPOSE:
;
;       Transforms equatorial coordinates to small angle offsets from a given reference point.
;
; CALLING SEQUENCE:
;       
;       result = radec2offset(RA_ref, Dec_ref, RA_off, Dec_off, /PLOT)
;
; INPUTS:
;       
;       RA_ref/Dec_ref:  Equatorial coordinates of the reference point.
;
;       RA_off/Dec_off:  Equatorial coordinates of the offset.
;
;
;                 NOTE:  All coordinates must be either in DEGREES or in SEXAGESIMAL notation,
;                        in the latter case, each entry must be a vector with 3 entries, i.e.
;
;                          RA = [hr,min,sec]  
;                         Dec = [deg,min,sec] 
;
;
; OPTIONAL KEYWORD:
;   
;                /PLOT:  Plots a diagram of the offset in arcsec from the reference point.'
;
; OUTPUTS:
;
;        A vector with 4 numbers, containing the X,Y-offsets in ARCSEC, the
;        first 2 numbers using the RA approximation for small angles with the
;        right symbol for a NE configuration.
;        The second pair of offsets are the exact values using the general formula,
;        but probably with a wrong symbol.
;
;        result = [ X-approx,Y-approx, X-exact,Y-exact ]'
;
; EXAMPLES:
;       
;       Equatorial coordinates of the reference point (0,0):
;       RA0=23.501865, Dec0=15.528838
;
;       Equatorial coordinates for which the offset in arcsec is sought:
;       RA_off=23.506190, Dec_off=15.535227
;
;       IDL> offset = radec2offset( 23.501865, 15.528838, 23.506190, 15.535227, /plot)
;
;
; RESTRICTIONS:
;
;       The value of RA is an approximation, valid within 1 degree.
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-


function radec2offset, RA_ref, Dec_ref, RA_off, Dec_off, PLOT=plot

compile_opt idl2, hidden
on_error, 2


if n_params() lt 4 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, '  result = radec2offset(RA_ref, Dec_ref, RA_off, Dec_off, /PLOT)'
    print, ''
    print, ''
    print, '      RA_ref/Dec_ref:  Coordinates of the reference point'
    print, ''
    print, '      RA_off/Dec_off:  Coordinates of the offset from the reference point'
    print, ''
    print, '               /PLOT:  Plots a diagram of the offset from the reference point'
    print, ''
    print, '      NOTE: All coordinates must be either in DEGREES or in SEXAGESIMAL notation,'
    print, '            in the latter case, each entry must be a vector with 3 entries, i.e.'
    print, ''
    print, '              RA = [hr,min,sec]'
    print, '             Dec = [deg,min,sec]'
    print, ''
    print, ''
    print, '    OUTPUT: a vector with 4 numbers, containing the X,Y-offsets in ARCSEC,'
    print, '            the first 2 numbers using the RA approximation for small angles'
    print, '            with the right symbol for a NE configuration.'
    print, '            The second pair of offsets are the exact values using the general'
    print, '            formula, but probably with a wrong symbol.'
    print, ''
    print, '            result = [ X-approx,Y-approx, X-exact,Y-exact ]'
    print, ''
    goto, fin
endif


if n_elements(RA_ref) eq 3 then begin


    ;------------  Y-OFFSET

    dec0_hr = Dec_ref[0]
    dec0_min = Dec_ref[1]
    dec0_sec = Dec_ref[2]
    
    dec_hr = Dec_off[0]
    dec_min = Dec_off[1]
    dec_sec = Dec_off[2]
    
    dec = dec_hr + (dec_min+dec_sec/60.)/60.
    dec0 = dec0_hr + (dec0_min+dec0_sec/60.)/60.
    
    dDec = dec - dec0
    
    mean_Dec = mean([dec,dec0])*!pi/180.
    
    y_offset = dDec*3600
    
    
    ;------------  X-OFFSET approximation
    
    ra0_hr = RA_ref[0]
    ra0_min = RA_ref[1]
    ra0_sec = RA_ref[2]
    
    ra_hr = RA_off[0]
    ra_min = RA_off[1]
    ra_sec = RA_off[2]
    
    ra = ( ra_hr + (ra_min+ra_sec/60)/60)*15
    ra0 = (ra0_hr + (ra0_min+ra0_sec/60)/60)*15
    
    dRA = ra - ra0
    
    x_offset = dRA*3600*cos(mean_Dec) ; valid within ~ 1 degree
    
    
    ;------------  Exact arclength
    
    dec = dec*!pi/180d
    dec0 = dec0*!pi/180d
    
    ra = ra*!pi/180d
    ra0 = ra0*!pi/180d
    
    gamma = cos(!pi/2 - dec)*cos(!pi/2 - dec0) + sin(!pi/2 - dec)*sin(!pi/2 - dec0)*cos(ra-ra0)
    
    arc_length = acos(gamma)
    
    theta = asin((dec0-dec)/arc_length)
    
    x_off = arc_length*cos(theta)*180/!pi*3600
    
    y_off = (dec-dec0)*180/!pi*3600
    
    goto, results

endif



if n_elements(RA_ref) eq 1 then begin
    

    ;------------  Y-OFFSET

    dec = Dec_off
    dec0 = Dec_ref

    dDec = dec - dec0

    mean_Dec = mean([dec,dec0])*!pi/180.

    y_offset = dDec*3600

    
    ;------------  X-OFFSET approximation
    
    ra = RA_off
    ra0 = RA_ref
    
    dRA = ra - ra0
    
    x_offset = dRA*3600*cos(mean_Dec) ; valid within ~ 1 degree
 

    ;------------  Exact arclength
       
     dec = dec*!pi/180d
     dec0 = dec0*!pi/180d
    
     ra = ra*!pi/180d
     ra0 = ra0*!pi/180d
    
     gamma = cos(!pi/2 - dec)*cos(!pi/2 - dec0) + sin(!pi/2 - dec)*sin(!pi/2 - dec0)*cos(ra-ra0)
    
     arc_length = acos(gamma)
    
     theta = asin((dec0-dec)/arc_length)
    
     x_off = arc_length*cos(theta)*180/!pi*3600
    
     y_off = (dec-dec0)*180/!pi*3600
    
endif

results:


if keyword_set(plot) then begin

   pingsoft_param, pings, ' '
    pingsoft_display, pings, /silent

    max = max([abs(x_offset),abs(y_offset)])*1.2
    window, 0, xsize=600, ysize=500
    plot, [0], /nodata, xr=[max,-max], yr=[-max,max], xs=1, ys=1, $
      xtitle='!7D!3RA', $
      ytitle='!7D!3Dec'
    
    plots, [0,0], [-max,max]
    plots, [max,-max], [0,0]
    
    loadct,39,/silent
    plots, x_offset, y_offset, psym=1, thick=2, color=250

endif


res = [x_offset,y_offset,x_off,y_off]

return, res
fin:
end

