

pro rectangular_pt, xside, yside, resolution, geometry, xpt, ypt, off, $
                    OUT_PT=out_pt, X_OFF=X_off, Y_OFF=y_off, PPAK=ppak


;+
; NAME:
;
;      RECTANGULAR_PT
;
; PURPOSE:
;
;      Creates a position table for a rectangular (or squared) continuous IFU.
;      
; EXPLANATION:
;
;      The script construct a position table ID, x, y, of a given size,
;      resolution (in arcsec) and geometry. Offsets can be provided to shift the
;      reference pixel (0,0), which is assumed to be at the bottom-right corner
;      of the XY plane. Supported geometries include: (1) the pixel index
;      increases from horizontally from right to left on the same row
;      (e.g. PPAK, CALIFA); (2) the index increases vertically, from bottom to
;      top on the same column (e.g. VIMOS), the default is set to 1. 
;
;
; CALLING SEQUENCE:
;       
;      rectangular_pt, xlength, ylength, resolution, geometry $
;                     [, xpt_val, ypt_val, offsets, OUT_PT=out_pt, X_OFF=x_off, Y_OFF=y_off,
;                      /PPAK ]"
;
;
; INPUTS:
;
;       X/YLENGTH:  X/Y length of the new position table
;
;      RESOLUTION:  size or resolution (in arcsec) of each spaxel
;
;        GEOMETRY:  Geometry of the input 3D cube, it is assumed that the
;                   reference pixel (1,1) at the bottom-right corner of
;                   the XY projection, and the (N,M) pixel at the top-left
;                   of the image. Supported geometries include: (1) the
;                   pixel index increases from horizontally from right to
;                   left on the same row (e.g. PPAK); (2) the index
;                   increases vertically, from bottom to top on the same
;                   column (e.g. VIMOS), the default is set to 1. 
;
; OPTIONAL KEYWORDS:
;
;       X/YPT_VAL:  Output variables with the values of x(RA) and y(Dec) of the new
;                   position table (in arcsec).
;
;         X/Y_OFF:  RA/Dec offset values of the reference point (0,0) within the
;                   Field-of-View of the new position table.
;
;         OFFSETS:  Output vector with the values of the offsets [RA_off,Dec_off]
;                   in arcsec, applied to the reference pixel/spaxel when no
;                   X/Y_OFF values are given (x/ylenght*resolution*0.5).
;
;          OUT_PT:  Name of the new position table, if not set default is: 'rectangular_pt.txt'
;
;
; EXAMPLES:
;
;       rectangular_pt, 80, 80, 0.65, 1, xpt, ypt, X_OFF=10, Y_OFF=20, OUT_PT='new_squared.pt.txt'
;
;
; REVISION HISTORY:
;
;         Written by:   F. Fabian Rosales-Ortega
;
;          Version 1:  January, 2010
;                  2:  April, 2011
;      
; NOTE:
;
;       This routine belongs to the PINGS IDL software available at:
;
;       http://www.ast.cam.ac.uk/ioa/research/pings/
;
;       where additional documentation can be found. If you find this code
;       useful on your research, please acknowledge the use of PINGSoft by
;       including the appropriate reference in your publications:
;       F. F. Rosales-Ortega 2011, NewAstron 16, 220
;
; COPYRIGHT:
;
;       This file is part of PINGSoft.
;
;       Copyright (C) 2010  F. Fabian Rosales-Ortega
;
;       PINGSoft is free software: you can redistribute it and/or modify it
;       under the terms of the GNU General Public License as published by the
;       Free Software Foundation, version 3. See the GNU General Public License
;       for more details.
;
;       PINGSoft is distributed in the hope that it will be useful, but WITHOUT
;       ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;       FITNESS FOR A PARTICULAR PURPOSE. 
;
;       The GNU General Public License is found in: http://www.gnu.org/licenses
;
;-


if n_params() lt 4 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "    rectangular_pt, xlength, ylength, resolution, geometry  $"
    print, "                    [, xpt_val, ypt_val, offsets, OUT_PT='out_pt', X_OFF=x_off, Y_OFF=y_off, /SILENT]"
    print, ''
    return
endif

res = resolution

; Offsets
if not keyword_set(x_off) then x_off = xside*res*0.5
if not keyword_set(y_off) then y_off = yside*res*0.5
xoff = x_off
yoff = y_off


; Writting test position table

res_str = string(res, format='(f5.2)')

out_junk = 'junk.pt.txt'

openw, 1, out_junk
printf,1, 'S   '+res_str+'   '+res_str+'   0'

id = 1l

xpt = fltarr(1)
ypt = fltarr(1)

if geometry eq 1 then begin

   for j=0l, yside-1 do begin
      y = j*res - yoff + res*0.5
      for i=0l, xside-1 do begin
         x = i*res - xoff + res*0.5
         xval = -x
         yval =  y
         if keyword_set(ppak) then yval = -y
         printf,1, id, xval, yval, 1
         xpt = [xpt,xval]
         ypt = [ypt,yval]
         id = id+1l
      endfor    
   endfor

endif


if geometry eq 2 then begin

   for i=0l, xside-1 do begin
      x = i*res - xoff + res*0.5
      for j=0l, yside-1 do begin
         y = j*res - yoff + res*0.5
         xval = -x
         yval =  y
         if keyword_set(ppak) then yval = -y
         printf,1, id, xval, yval, 1
         xpt = [xpt,xval]
         ypt = [ypt,yval]
         id = id+1
      endfor    
   endfor

endif

close, 1

off = [xoff,yoff]

if keyword_set(out_pt) then $
   spawn, 'mv junk.pt.txt '+out_pt else $
      spawn, 'mv junk.pt.txt rectangular.pt.txt'

xpt = xpt[1:*]
ypt = ypt[1:*]

end
