
;+
; NAME:
;       REDSHIFT_3D
;
; PURPOSE:
;
;       Returns the redshift of the input IFS data measured from the integrated
;       spectra.
;
;
; For CALLING SEQUENCE, INPUTS and OPTIONAL KEYWORDS enter:
;
;        IDL> result = redshift_3d()
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-



function redshift_3D, infile, lam_rest,  WIDTH=width, LAM_Z=lam_z, SILENT=silent


if n_params() lt 1 then begin
    print, ''
    print, "  z = redshift_3D( 'OBJECT', [ lam_rest,  WIDTH=width, LAM_Z=lam_z, /SILENT ]) "
    print, ''
    print, "        'OBJECT':  String of the IFS 3D cube, RSS FITS or 1-D ASCII file"
    print, ''
    print, '           WIDTH:  Width of the spectral band centered at lam_rest where '
    print, '                   the reference emission-line will be sought for the'
    print, '                   redshift determination.'
    print, ''
    print, '           LAM_Z:  output, lam_rest at derived redshift'
    print, ''
    print, '        Defaults:  lam_rest = 6563 (Halpha), width=100 AA'
    print, '                   Note potential problems  if the intensity of [NII] is '
    print, '                   approximate or higher than Halpha (e.g. LINERS/AGN)'
    print, ''
    goto, fin
endif


;--------------------------  Initial tests

infile = sctrim(infile)

if file_test(infile) eq 0 then begin
   print, ''
   print, 'ERROR: The file:  "'+infile+'"  does not exist!'
   print, ''
   goto, fin
endif

com="file "+infile+" | awk '{print $2}'"
spawn, com, type

if type eq 'ASCII' then begin
   readcol, infile, id, lam, flux, /silent
   goto, ascii
endif



;--------------------------  Lambda/Flux values

integrate_3d, infile, integ, /no_output, /silent

 lam = integ.lambda
flux = integ.finteg

ascii:


;--------------------------  Defaults

if n_params() lt 2 then lam_rest = 6562.8   ; Halpha

if not keyword_set(width) then width=100


; Guessing max value

nn = where(lam ge lam_rest-width*0.5)
xtest = lam[nn]
ytest = flux[nn]

 cont = mean(ytest)
sigma = stddev(ytest)

mm = where(ytest-cont gt sigma)
xmax=xtest[mm]
ymax=ytest[mm]-cont 

ok = where(ymax gt sigma*3)
xok=xmax[ok]
yok=ymax[ok]

eline = min(xok)
eline = eline[0]



; Fitting eline

nn = where(lam ge eline-width and lam le eline+width)
xtest = lam[nn]
ytest = flux[nn]


yfit=gaussfit(xtest,ytest,a,nterms=4)
inten=a[0]
 clam=a[1]
  sig=a[2]
 cont=a[3]


; Output

    z = (clam/lam_rest)-1
lam_z = clam 

    c = 3e5
  vel = z * c
 vel1 = (vel-300.)/c
 vel2 = (vel+300.)/c

   sz = sctrim(z)
sclam = sctrim(clam)
srlam = sctrim(lam_rest)

svel1 = sctrim(vel1)
svel2 = sctrim(vel2)


if not keyword_set(silent) then begin

   window,0
   loadct,39,/silent
   plot, lam, flux, xr=[min(xtest),max(xtest)], $
         xtitle='Wavelength', ytitle='Flux', title='Integrated spectrum'
   oplot, xtest, yfit, color=50, thick=2
   oplot, xtest, inten*exp(-(xtest-clam)^2/(2*sig^2))+cont, color=250, thick=1
   
   print, ''
   print, ' PINGSoft: redshift_3D'
   print, ' ======================'
   print, ''
   print, '                     z:  '+sz
   print, ''
   print, '  Reference wavelength:  '+srlam
   print, ''
   print, ' Redshifted wavelength:  '+sclam
   print, ''
   print, '          +/- 300 km/h:  '+svel1+'  '+svel2
   print, ''
   print, ''
endif

return, z
fin:
end
