
pro copy_hdr, infile, outfile, RSS2CUBE=rss2cube


;+
; NAME:
;      COPY_HDR
;
; PURPOSE:
;
;      Copies the header of one FITS file to another, USE WITH CAUTION!.
;
;
; CALLING SEQUENCE:
;       
;      copy_hdr, 'origin.FITS', 'target.FITS'
;
; INPUTS:
;       
;      'source.FITS':  String of the FITS file with the header to be copied.
;                           
;      'target.FITS':  String of an existing FITS file to which the header will be copied.
;      
;
; EXAMPLES:
;
;       copy_hdr, 'origin.fits', 'target.fits'
;
;
; REVISION HISTORY:
;
;       Written by:     F. Fabian Rosales-Ortega
;
;       First version:  January, 2010
;
; NOTE:
;
;       This routine belongs to the PINGS IDL software available at:
;
;       http://www.ast.cam.ac.uk/research/pings
;
;       where additional documentation can be found. If you find this code
;       useful on your research, please acknowledge the use of PINGSoft by
;       including the original project reference in your publications.
;
; COPYRIGHT:
;
;       This file is part of PINGSoft.
;
;       Copyright (C) 2010  F. Fabian Rosales-Ortega
;
;       PINGSoft is free software: you can redistribute it and/or modify it
;       under the terms of the GNU General Public License as published by the
;       Free Software Foundation, version 3. See the GNU General Public License
;       for more details.
;
;       PINGSoft is distributed in the hope that it will be useful, but WITHOUT
;       ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;       FITNESS FOR A PARTICULAR PURPOSE. 
;
;       The GNU General Public License is found in: http://www.gnu.org/licenses
;
;-

compile_opt idl2, hidden
on_error, 2


if n_params() lt 2 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "    copy_hdr, 'source.FITS', 'target.FITS'"
    print, ''
    print, ''
    print, "        'source.FITS':  String of the FITS file with the header to be copied."
    print, ''
    print, "        'target.FITS':  String of an existing FITS file to which the header will be copied."     
    print, ''
    print, '        NOTE: USE WITH CAUTION!'
    print, ''
    return
endif



;--------------------------  Initial tests

 infile = sctrim(infile)
outfile = sctrim(outfile)

if file_test(infile) eq 0 then begin
    print, ''
    print, 'ERROR: The file: '+infile+' does not exist!'
    print, ''
    return
endif
if file_test(outfile) eq 0 then begin
    print, ''
    print, 'ERROR: The file: '+outfile+' does not exist!'
    print, ''
    return
endif


; Reads FITS source

dumb = readfits( infile, hdr1, /silent)

data = readfits(outfile, hdr2, /silent)


if keyword_set(rss2cube) then begin

   dimx = fxpar(hdr2,'NAXIS1')
   dimy = fxpar(hdr2,'NAXIS2')

   crpix1 = fxpar(hdr2,'CRPIX1')
   crpix2 = fxpar(hdr2,'CRPIX2')
   crpix3 = fxpar(hdr1,'CRPIX1')

   cdelt1 = fxpar(hdr2,'CDELT1')
   cdelt2 = fxpar(hdr2,'CDELT2')
   cdelt3 = fxpar(hdr1,'CDELT1')

   crval1 = fxpar(hdr1,'RA')
   crval2 = fxpar(hdr1,'DEC')
   crval3 = fxpar(hdr1,'CRVAL1')

   ; WCS reference
   pt = strsplit(infile,'.fits',/extract,/regex)+'.pt.txt'
   if file_test(pt) eq 1 then begin

      print, 'Coord Correction, PosTable used: '+pt 

      ;TEST
      ;pt='shift2.pt.txt'

      readcol, pt, f='a,f,f,i', mtype, xsize, ysize, numline=1, /silent
      xsize = xsize[0]  &  ysize = ysize[0]
      readcol, pt, f='l,f,f', id, x, y, /silent

      basex = (max(x) + abs(min(x)) + xsize*4)/cdelt1
      basey = (max(y) + abs(min(y)) + ysize*4)/cdelt2

      crpix1 = (dimx/basex) * ( (max(x) + xsize*4)/cdelt1 )
      crpix2 = (dimy/basey) * ( (max(y) + ysize*4)/cdelt2 )

   endif

   fxaddpar, hdr1, 'CRVAL1', crval1, ' updated by PINGSoft'
   fxaddpar, hdr1, 'CRVAL2', crval2, ' updated by PINGSoft'
   fxaddpar, hdr1, 'CRVAL3', crval3, ' updated by PINGSoft'

   fxaddpar, hdr1, 'CRPIX1', crpix1, ' updated by PINGSoft'
   fxaddpar, hdr1, 'CRPIX2', crpix2, ' updated by PINGSoft'
   fxaddpar, hdr1, 'CRPIX3', crpix3, ' updated by PINGSoft'

   fxaddpar, hdr1, 'CDELT1', cdelt1, ' updated by PINGSoft'
   fxaddpar, hdr1, 'CDELT2', cdelt2, ' updated by PINGSoft'
   fxaddpar, hdr1, 'CDELT3', cdelt3, ' updated by PINGSoft'

   ; WCS
   off1 = offset2radec(crval1,crval2,cdelt1,0)
   off2 = offset2radec(crval1,crval2,0,cdelt2)

   cd1_1 = crval1 - off1[0]
   cd2_2 = crval2 - off2[1]

   cd1_1 = cd2_2

   fxaddpar, hdr1, 'RADESYS',     'ICRS', ' updated by PINGSoft'
   fxaddpar, hdr1, 'CTYPE1',  'RA---TAN', ' updated by PINGSoft'
   fxaddpar, hdr1, 'CUNIT1',       'deg', ' updated by PINGSoft'
   fxaddpar, hdr1, 'CTYPE2',  'DEC--TAN', ' updated by PINGSoft'
   fxaddpar, hdr1, 'CUNIT2',       'deg', ' updated by PINGSoft'
   fxaddpar, hdr1, 'CD1_1',        cd1_1, ' updated by PINGSoft'
   fxaddpar, hdr1, 'CD2_2',        cd2_2, ' updated by PINGSoft'


endif

hdr = hdr1


; Writes FITS file
writefits, outfile, data, hdr


print, ''
print, " FITS header copied from  '"+infile+"'  to  '"+outfile+"'"
print, ''
print, ''
if keyword_set(rss2cube) then $
print, ' RSS2CUBE set!'
print, ''
print, ''

end
