
pro extract_rss, rss_file, output, row1, row2

;+
; NAME:
;      EXTRACT_RSS
;
; PURPOSE:
;
;       Extracts a section of consecutive rows in a RSS FITS file and creates a new one. 
;
;
; CALLING SEQUENCE:
;       
;       extract_rss, 'RSS_in.fits', 'RSS_out.fits', row1, row2
;
; INPUTS:
;       
;        'RSS_in.fits':  String of the RSS FITS file from which spectra will be extracted.
;
;       'RSS_out.fits':  String of the output RSS FITS file with extracted spectra
;
;               ROW1/2:  Range of physical rows in the RSS FITS file to be extracted, note
;                        that row2 >= row1, i.e. it can be used to extract a
;                        single spectrum.
;
; EXAMPLES:
;
;       extract_rss, 'object_rss.fits', 'extracted_rss.fits', 20, 340 
;
;
; REVISION HISTORY:
;
;       Written by:     F. Fabian Rosales-Ortega
;
;       First version:  January, 2010
;
; NOTE:
;
;       This routine belongs to the PINGS IDL software available at:
;
;       http://www.ast.cam.ac.uk/research/pings/
;
;       where additional documentation can be found. If you find this code
;       useful on your research, please acknowledge the use of PINGSoft by
;       including the original project reference in your publications.
;
;-


compile_opt idl2, hidden
on_error, 2


if n_params() lt 4 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, " extract_rss, 'RSS_in.fits', 'RSS_out.fits', row1, row2"
    print, ''
    print, ''
    print, "   'RSS_in.fits':  String of the RSS FITS file from which spectra will be extracted."
    print, ''
    print, "  'RSS_out.fits':  String of the output RSS FITS file with extracted spectra."
    print, '' 
    print, '          ROW1/2:  Range of physical rows in the RSS FITS file to be extracted.'
    print, ''
    print, '   NOTE: row2 >= row1'
    print, ''
    return
endif


;--------------------------  Initial tests

rss_file = sctrim(rss_file)

if file_test(rss_file) eq 0 then begin
    print, ''
    print, 'ERROR: The file:  "'+rss_file+'"  does not exist!'
    print, ''
    return
endif

if row2 lt row1 then begin
    print, ''
    print, 'Final row must be greater/equal than the initial one'
    print, ''
    return
endif



; Reading input RSS

read_rss, rss_file, data, hdr, size=size,  /silent

pixels = size[1]
  rows = size[2]


; Writing output RSS

num = row2 - row1

out_rss = fltarr(pixels,num+1)

k = 0

for i=row1, row2 do begin

    out_rss[*,k] = data[*,i]
    
    k = k + 1 

endfor


writefits, output, out_rss, hdr, /CHECKSUM

print, ''
print, 'DONE'
print, ''
print, ' File written: ', output
print, ''

fin:
end
