

pro merge_ptable, ptable_list, out_ptable, SILENT=silent


;+
; NAME:
;
;      MERGE_PTABLE
;
; PURPOSE:
;
;       Concatenates a list of position table files into a single one for
;       mosaicking purposes.
;
;
; CALLING SEQUENCE:
;       
;       merge_ptable, 'PosTable_list.txt', 'merged_PosTable.txt' [, /SILENT ]
;
; INPUTS:
;       
;       'PosTable_list.txt':  String with the name of an ASCII file containing a list
;                             of the position table files to be merged.
;
;     'merged_PosTable.txt':  String with the name of the output position table.
;
;
; OPTIONAL KEYWORDS:
;
;                   /SILENT:  No additional information is shown on screen.
;
; RESTRICTIONS:
;   
;        The spaxel type and size is defined by the first position table in the
;        input list. The position tables in the input file should be listed in
;        the same order as their corresponding RSS files are merged. See the
;        routines: merge_rss & shift_ptable for more information.
;
; EXAMPLES:
;
;       merge_ptable, 'ptable_list.txt', 'merged_pt.fits'
;
;
; REVISION HISTORY:
;
;       Written by:     F. Fabian Rosales-Ortega
;
;       First version:  January, 2010
;
; NOTE:
;
;       This routine belongs to the PINGS IDL software available at:
;
;       http://www.ast.cam.ac.uk/research/pings
;
;       where additional documentation can be found. If you find this code
;       useful on your research, please cite the following reference: 
;
; COPYRIGHT:
;
;       This file is part of PINGSoft.
;
;       Copyright (C) 2010  F. Fabian Rosales-Ortega
;
;       PINGSoft is free software: you can redistribute it and/or modify it
;       under the terms of the GNU General Public License as published by the
;       Free Software Foundation, version 3. See the GNU General Public License
;       for more details.
;
;       PINGSoft is distributed in the hope that it will be useful, but WITHOUT
;       ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;       FITNESS FOR A PARTICULAR PURPOSE. 
;
;       The GNU General Public License is found in: http://www.gnu.org/licenses
;
;-

compile_opt idl2, hidden
on_error, 2


if n_params() lt 2 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, ''
    print, "          merge_ptable, 'PosTable_list.txt', 'merged_PosTable.txt' [, /SILENT ]"
    print, ''
    print, ''
    print, "   'PosTable_list.txt':  String with the name of an ASCII file containing a list"
    print, '                         of the position table files to be merged.'
    print, ''
    print, " 'merged_PosTable.txt':  String with the name of the output position table."
    print, ''
    print, '               /SILENT:  No additional information is shown on screen.'
    print, ''
    print, ''
    return
endif


;--------------------------  Initial tests

if arg_present(ptable_list) then begin
    print, ''
    print, "ERROR: 'PosTable_list.txt' must be a string."
    print, ''
    return
endif

ptable_list = sctrim(ptable_list)

if file_test(ptable_list) eq 0 then begin
    print, ''
    print, 'ERROR: The file  "'+ptable_list+'"  does not exist!'
    print, ''
    return
endif

if arg_present(out_ptable) then begin
    print, ''
    print, "ERROR:  'merged_PosTable.txt' must be a string."
    print, ''
    return
endif

out_ptable = sctrim(out_ptable)


;--------------------------  Reading ASCII list

readcol, ptable_list, f='a', name, /silent


;--------------------------  Check files

for k=0, n_elements(name)-1 do begin
    if file_test(name[k]) eq 0 then begin
        print, ''
        print, 'ERROR: The file  "'+name[k]+'"  in  "'+ptable_list+'"  does not exist!'
        print, ''
        return
    endif
endfor

;--------------------------  Spaxel type (defined by the first position table)

readcol, name[0], f='a,f,f,i', mtype, xsize, ysize, numline=1, /silent

openw, lun, out_ptable, /get_lun

if mtype eq 'C' then printf, lun, 'C  '+sctrim(xsize)+'  '+sctrim(ysize)+'  0'
if mtype eq 'S' then printf, lun, 'S  '+sctrim(xsize)+'  '+sctrim(ysize)+'  0'


;--------------------------  Reading all tables

extra = 0

for k=0, n_elements(name)-1 do begin

    readcol, name[k], f='l,f,f,i', id, x, y, flag, /silent

    for i=0, n_elements(id)-1 do begin

        spaxel = id[i]+extra
        str = string(spaxel, x[i], y[i], flag[i])
        printf, lun, strtrim(str, 1)

    endfor

    extra = spaxel

endfor

close, lun
free_lun, lun


if not keyword_set(silent) then begin
    print, ''
    print, ' Total number of lines:  ', sctrim(spaxel)
    print, ''
    print, '          File written:  ', out_ptable
    print, ''
endif


end
