
pro merge_rss, rss_list, out_rss, SILENT=silent

;+
; NAME:
;      MERGE_RSS
;
; PURPOSE:
;
;       Merges a list of RSS files into a single RSS file.
;
;
; CALLING SEQUENCE:
;       
;       merge_rss, 'RSS_list.txt', 'RSS_out.fits'
;
; INPUTS:
;       
;       'RSS_list.txt':  String with the name of an ASCII file containing a list
;                        of the RSS FITS files to be merged.
;
;       'RSS_out.fits':  String with the name of the output RSS FITS file.
;
;
; OPTIONAL KEYWORDS:
;
;              /SILENT:  No additional information is shown on screen.
;
;
; RESTRICTIONS:
;   
;        All RSS files to be merged should contain the same number of pixels'
;        along the dispersion axis.'
;
; EXAMPLES:
;
;       merge_rss, 'rss_list.txt', 'merged_rss.fits'
;
;
; REVISION HISTORY:
;
;       Written by:     F. Fabian Rosales-Ortega
;
;       First version:  January, 2010
;
; NOTE:
;
;       This routine belongs to the PINGS IDL software available at:
;
;       http://www.ast.cam.ac.uk/research/pings
;
;       where additional documentation can be found. If you find this code
;       useful on your research, please acknowledge the use of PINGSoft by
;       including the original project reference in your publications.
;
; COPYRIGHT:
;
;       This file is part of PINGSoft.
;
;       Copyright (C) 2010  F. Fabian Rosales-Ortega
;
;       PINGSoft is free software: you can redistribute it and/or modify it
;       under the terms of the GNU General Public License as published by the
;       Free Software Foundation, version 3. See the GNU General Public License
;       for more details.
;
;       PINGSoft is distributed in the hope that it will be useful, but WITHOUT
;       ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;       FITNESS FOR A PARTICULAR PURPOSE. 
;
;       The GNU General Public License is found in: http://www.gnu.org/licenses
;
;-


compile_opt idl2, hidden
on_error, 2


if n_params() lt 2 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, " merge_rss, 'RSS_list.txt', 'RSS_out.fits'"
    print, ''
    print, ''
    print, "  'RSS_list.txt':  String with the name of an ASCII file containing a list"
    print, '                   of the RSS FITS files to be merged. '
    print, ''
    print, "  'RSS_out.fits':  String with the name of the output RSS FITS file."
    print, '' 
    print, ''
    print, '   NOTE: All RSS files to be merged should contain the same number of pixels'
    print, '         along the dispersion axis.'
    print, ''
    return
endif


;--------------------------  Initial tests

rss_list = sctrim(rss_list)

if file_test(rss_list) eq 0 then begin
    print, ''
    print, 'ERROR: The file:  "'+rss_list+'"  does not exist!'
    print, ''
    return
endif


;--------------------------  Reading ASCII list

readcol, rss_list, f='a', name, /silent

read_rss, name[0], data, hdr, ROWS=rows, PIXELS=pixels, /SILENT

all_rss = data


for i=1, n_elements(name)-1 do begin

    read_rss, name[i], data, /silent

    all_rss = [[all_rss],[data]]
    
endfor

writefits, out_rss, all_rss, hdr, /CHECKSUM 

read_rss, out_rss, dumb, ROWS=rows, /silent

trows = sctrim(rows)

if not keyword_set(silent) then begin
    print, ''
    print, ''
    print, 'DONE'
    print, ''
    print, ' Total number of rows:  '+trows
    print, ''
    print, '         File created:  '+out_rss
    print, ''
endif
  

end
