
pro read_rss, file, data, hdr, SIZE=size, ROWS=rows, PIXELS=pixels,  SILENT=silent


;+
; NAME:
;       READ_RSS
;
; PURPOSE:
;
;       Reads a RSS FITS file and stores the data into an IDL vector.
;
;
; CALLING SEQUENCE:
;       
;       read_rss, 'RSS.fits', data [, header, SIZE=size, ROWS=rows, PIXELS=pixels, /SILENT]
;
; INPUTS:
;       
;       'RSS.fits':  String of the wavelength calibrated RSS FITS file.
;
; OUTPUTS:
;             data:  Vector to store the spectral information.
;
;           header:  Name of the variable to store the FITS header.
;
;
; OPTIONAL KEYWORDS:
;       
;             SIZE:  Vector to store the spatial size of the RSS FITS image.
;
;             ROWS:  Variable containing the number of rows in the RSS file.
;
;           PIXELS:  Variable containing the number of pixels per row in the RSS file.
; 
;          /SILENT:  No additional information is shown on the terminal.
;
;
; EXAMPLES:
;
;       read_rss, 'object_rss.fits', data, header
;
;
; REVISION HISTORY:
;
;       Written by:     F. Fabian Rosales-Ortega
;
;       First version:  January, 2010
;
; NOTE:
;
;       This routine belongs to the PINGS IDL software available at:
;
;       http://www.ast.cam.ac.uk/research/pings
;
;       where additional documentation can be found. If you find this code
;       useful on your research, please acknowledge the use of PINGSoft by
;       including the original project reference in your publications.
;
; COPYRIGHT:
;
;       This file is part of PINGSoft.
;
;       Copyright (C) 2010  F. Fabian Rosales-Ortega
;
;       PINGSoft is free software: you can redistribute it and/or modify it
;       under the terms of the GNU General Public License as published by the
;       Free Software Foundation, version 3. See the GNU General Public License
;       for more details.
;
;       PINGSoft is distributed in the hope that it will be useful, but WITHOUT
;       ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;       FITNESS FOR A PARTICULAR PURPOSE. 
;
;       The GNU General Public License is found in: http://www.gnu.org/licenses
;
;-


compile_opt idl2, hidden
on_error, 2


if n_params() lt 2 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "   read_rss, 'RSS.fits', data, [, header, SIZE=size, ROWS=rows, PIXELS=pixels, /SILENT]"
    print, ''
    print, ''
    print, "    'RSS.fits':  String of the wavelength calibrated RSS FITS file."
    print, ''
    print, '          data:  Vector to store the spectral information.'
    print, ''
    print, '        header:  Name of the variable to store the FITS header.'
    print, ''
    print, '          SIZE:  Vector to store the spatial size of the RSS FITS image.'
    print, ''
    print, '          ROWS:  Variable containing the number of rows in the RSS file.'
    print, ''
    print, '        PIXELS:  Variable containing the number of pixels per row in the RSS file.'
    print, ''
    print, '       /SILENT:  No additional information is shown on the terminal.'
    print, ''
    return
endif

;--------------------------  Initial tests

file = sctrim(file)

if file_test(file) eq 0 then begin
    print, ''
    print, 'ERROR: The file:  "'+file+'"  does not exist!'
    print, ''
    return
endif


;--------------------------  Reading data

data = readfits(file, hdr,/silent)
size = size(data)

pixels = size[1]
  rows = size[2]

pixels_string = strcompress(string(pixels),/remove_all)
  rows_string = strcompress(string(rows),/remove_all)

if not keyword_set(silent) then begin

print, ''
print, 'INFO:'
print, ''
print, ' Number of rows in RSS: ', rows_string
print, '                Pixels: ', pixels_string
print, ''
print, 'USEFUL COMMANDS:'
print, ''
print, " crval = fxpar(hdr,'CRVAL1')"
print, " cdelt = fxpar(hdr,'CDELT1')"
print, " crpix = fxpar(hdr,'CRPIX1')"
print, "pixels = ",pixels_string
print, "  rows = ",rows_string
print, ''
print, 'lambda = dblarr(pixels)'
print, 'for i=0, pixels-1 do lambda[i] = (crval + i*cdelt)-(crpix-1)*cdelt'
print, ''
print, ''
print, 'flux = data[*,i]'
print, ''
print, 'xmin = min(lambda) & xmax = max(lambda)'
print, 'ymin = min(flux)   & ymax = max(flux)'
print, ''

endif


end
