

pro write_hdr, infile, field, content, comment, AFTER=after, BEFORE=before, FORMAT=format, STRING=string


;+
; NAME:
;      WRITE_HDR
;
; PURPOSE:
;
;      Adds or updates an entry in the header of a FITS file, using the 'fxaddpar.pro' utility.
;
;
; CALLING SEQUENCE:
;       
;      write_hdr, 'file.FITS', 'FIELD', CONTENT [, 'COMMENT', /STRING, AFTER='after', BEFORE='before', FORMAT='format']"
;
; INPUTS:
;       
;      'file.FITS':  String with the name of the FITS file.
;
;          'FIELD':  Name of the entry to be added or updated.
;
;          CONTENT:  Value of the entry to be added or updated, default none.
;
; OPTIONAL KEYWORDS:
;
;       'COMMENT':  Comment of the entry to be added or updated."
;
;         /STRING:  Declare this keyword if CONTENT is a string, otherwise there will be an error.
;
;           AFTER:  If declared, the new entry will be placed AFTER the name of the entry set in this keyword.
;
;          BEFORE:  If declared, the new entry will be placed BEFORE the name of the entry set in this keyword.
;          
;          FORMAT:  If CONTENT is a numerical value, this string defines the format of the entry.
;
;
; EXAMPLES:
;
;       1) Numerical entry:
;
;       write_hdr, 'file.fits', 'CRVAL3', 4500.45, 'This value is not OK', after='EQUINOX', format='f6.1'
;
;       2) String entry:
;
;       write_hdr, 'file.fits', 'TEXT1', 'This is a new entry', /string
;
;
; REVISION HISTORY:
;
;       Written by:     F. Fabian Rosales-Ortega
;
;       First version:  January, 2010
;
; NOTE:
;
;       This routine belongs to the PINGS IDL software available at:
;
;       http://www.ast.cam.ac.uk/research/pings/
;
;       where additional documentation can be found. If you find this code
;       useful on your research, please acknowledge the use of PINGSoft by
;       including the original project reference in your publications.
;
; COPYRIGHT:
;
;       This file is part of PINGSoft.
;
;       Copyright (C) 2010  F. Fabian Rosales-Ortega
;
;       PINGSoft is free software: you can redistribute it and/or modify it
;       under the terms of the GNU General Public License as published by the
;       Free Software Foundation, version 3. See the GNU General Public License
;       for more details.
;
;       PINGSoft is distributed in the hope that it will be useful, but WITHOUT
;       ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;       FITNESS FOR A PARTICULAR PURPOSE. 
;
;       The GNU General Public License is found in: http://www.gnu.org/licenses
;
;-


compile_opt idl2, hidden
on_error, 2


if n_params() lt 3 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "   write_hdr, 'file.FITS', 'FIELD', CONTENT [, 'COMMENT', /STRING, AFTER='after', BEFORE='before', FORMAT='format']"
    print, ''
    print, ''
    print, "    'file.FITS':  String with the name of the FITS file."
    print, ''
    print, "        'FIELD':  Name of the entry to be added or updated."
    print, ''
    print, '        CONTENT:  Value of the entry to be added or updated, default none.'
    print, ''
    print, "      'COMMENT':  Comment of the entry to be added or updated."
    print, ''
    print, "        /STRING:  Declare this keyword if CONTENT is a string, otherwise there will be an error."
    print, ''
    print, '          AFTER:  If declared, the new entry will be placed AFTER the name of the entry set in this keyword.'
    print, ''
    print, '         BEFORE:  If declared, the new entry will be placed BEFORE the name of the entry set in this keyword.'
    print, ''
    print, '         FORMAT:  If CONTENT is a numerical value, this string defines the format of the entry.'
    print, ''
    return
endif


;--------------------------  Initial tests



infile = sctrim(infile)

if file_test(infile) eq 0 then begin
    print, ''
    print, 'ERROR: The file: '+infile+' does not exist!'
    print, ''
    return
endif

;; if arg_present(field) then begin
;;     print, ''
;;     print, "ERROR:  'FIELD' must be a string"
;;     print, ''
;;     return
;; endif

;; if arg_present(comment) then begin
;;     print, ''
;;     print, "ERROR:  'COMMENT' must be a string"
;;     print, ''
;;     return
;; endif

;; if arg_present(after) then begin
;;     print, ''
;;     print, "ERROR:  'AFTER' must be a string"
;;     print, ''
;;     return
;; endif

;; if arg_present(before) then begin
;;     print, ''
;;     print, "ERROR:  'BEFORE' must be a string"
;;     print, ''
;;     return
;; endif

;; if arg_present(format) then begin
;;     print, ''
;;     print, "ERROR:  'FORMAT' must be a string, i.e. format='f5.2'"
;;     print, ''
;;     return
;; endif


data = readfits(infile,hdr,/silent)

if not keyword_set(comment) then comment=' '

if keyword_set(format) then fmt1 = ", format='"+format+"'" else fmt1 = ''

if keyword_set(before) then fmt2 = ", before='"+before+"'" else fmt2 = ''

if keyword_set(after)  then fmt3 = ", after='"+after+"'"   else fmt3 = ''


fmt =fmt1+fmt2+fmt3

if not keyword_set(string) then $
  com = "fxaddpar, hdr, '"+field+"', "+string(content)+", ' "+comment+"'"+fmt

if keyword_set(string) then $
  com = "fxaddpar, hdr, '"+field+"', '"+content+"', ' "+comment+"'"+fmt

s = execute(com)


; Writes RSS file

writefits, infile, data, hdr

print, ''
print, ' Value added to FITS header '
print, ''
print, ' The command was:  ', com
print, ''

end

