

pro write_wcs, file, ra, dec, name, EQUINOX=equinox, FORCE=force


;+
; NAME:
;
;      WRITE_WCS
;
; PURPOSE:
;
;      Adds or updates the WCS (World Coordinate Systems) entries in a FITS header.
;      
; EXPLANATION:
;
;      Programs such as 'view_rss' can display the spaxels' equatorial coordinates
;      information if the Right Ascension and Declination of the position table
;      reference point (0,0) is included in the FITS header. This prodecure
;      writes this information in the entries 'RA' and 'DEC' (in degree units).
;      Optionally, the name of the object and the equinox can be set (by default
;      the equinox is assumed to be J2000).
;
;
; CALLING SEQUENCE:
;       
;      write_wcs,'RSS.fits', RA, Dec [,'OBJECT', EQUINOX=equinox, /FORCE]
;
; INPUTS:
;
;      'RSS.fits':  String with the name of the FITS file.
;
;          RA/Dec:  Right Ascension/Declination of the position table reference point (0,0) in degrees.
;
;
; OPTIONAL KEYWORDS:
;
;        'OBJECT':  String with the name of the object, it set it will add/update the 'OBJECT' entry.
;
;         EQUINOX:  Equinox of the coordinates, default: J2000.
;
;          /FORCE:  Overwrites the fields in the FITS header.
;
;
; EXAMPLES:
;
;       write_wcs, 'ngc300.fits', 13.13232  -37.95525, 'NGC 300', EQUINOX=1950, /force
;
;
; REVISION HISTORY:
;
;       Written by:     F. Fabian Rosales-Ortega
;
;       First version:  January, 2010
;
; NOTE:
;
;       This routine belongs to the PINGS IDL software available at:
;
;       http://www.ast.cam.ac.uk/research/pings/
;
;       where additional documentation can be found. If you find this code
;       useful on your research, please acknowledge the use of PINGSoft by
;       including the original project reference in your publications.
;
; COPYRIGHT:
;
;       This file is part of PINGSoft.
;
;       Copyright (C) 2010  F. Fabian Rosales-Ortega
;
;       PINGSoft is free software: you can redistribute it and/or modify it
;       under the terms of the GNU General Public License as published by the
;       Free Software Foundation, version 3. See the GNU General Public License
;       for more details.
;
;       PINGSoft is distributed in the hope that it will be useful, but WITHOUT
;       ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;       FITNESS FOR A PARTICULAR PURPOSE. 
;
;       The GNU General Public License is found in: http://www.gnu.org/licenses
;
;-


compile_opt idl2, hidden
on_error, 2


if n_params() lt 3 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "    write_wcs, 'RSS.fits', RA, Dec [,'OBJECT', EQUINOX=equinox, /FORCE]"
    print, ''
    print, ''
    print, "    'RSS.fits':  String with the name of the FITS file."
    print, ''
    print, '        RA/Dec:  Right Ascension/Declination of the position table reference point (0,0) in degrees.'
    print, ''
    print, "      'OBJECT':  String with the name of the object, it set it will add/update the 'OBJECT' entry."
    print, ''
    print, '       EQUINOX:  Equinox of the coordinates, default: 2000.'
    print, ''
    print, '        /FORCE:  Overwrites the fields in the FITS header.'  
    print, ''
    return
endif


;--------------------------  Initial tests

file = sctrim(file)

if file_test(file) eq 0 then begin
    print, ''
    print, 'ERROR: The file: '+file+' does not exist!'
    print, ''
    return
endif



data = readfits(file,hdr,/silent)

if not keyword_set(equinox) then equinox = 2000


; Overwriting forced

if keyword_set(force) then begin

    if keyword_set(name) then $
    fxaddpar, hdr,  'OBJECT',    name, ' Observed object'
    fxaddpar, hdr,      'RA',      ra, ' Reference Right Ascension*', format='f10.5'
    fxaddpar, hdr,     'DEC',     dec, ' Reference Declination*', format='f10.5'
    fxaddpar, hdr, 'EQUINOX', equinox, ' Equinox of celestial coord. system'

    writefits, file, data, hdr
    print, ''
    print, ' WCS information UPDATED in the FITS header'
    print, ''
    goto, fin

endif


; Checks if info already in header

test1 = fxpar(hdr,'RA')
test2 = fxpar(hdr,'DEC')

if test1 ne 0 or test2 ne 0 then goto, next


; Writes WCS
if keyword_set(name) then $
fxaddpar, hdr,  'OBJECT',    name, ' Observed object', after='COMMENT'
fxaddpar, hdr,      'RA',      ra, ' Reference Right Ascension*', format='f10.5', after='OBJECT'
fxaddpar, hdr,     'DEC',     dec, ' Reference Declination*', format='f10.5', after='RA'
fxaddpar, hdr, 'EQUINOX', equinox, ' Equinox of celestial coord. system', after='DEC'
fxaddpar, hdr, 'COMMENT', '', after='EQUINOX'
fxaddpar, hdr, 'COMMENT', ' updated by PINGSoft'


; Writes RSS file

writefits, file, data, hdr
print, ''
print, ' WCS added to the RSS FITS header'
print, ''
goto, fin

next:
print, ''
print, ' WARNING!'
print, ''
print, ' WCS information already in FITS header, to overwrite use the /FORCE keyword.'
print, ''

fin:

end
