


;+
; NAME:
;       S2N_OPTIMIZE
;
; PURPOSE:
;
;       Extracts spectra interactively based on a S/N continuum and emission
;       line optimization method by Rosales-Ortega et al. (2012) A&A, 539, 73,
;       using cumulative integrated spectra.
;
;
; EXPLANATION:
;
;       The optimization method is performed by first calculating the 
;       (S/N)_continuum and (S/N)_emission for each of the spaxels
;       (preferably in a velocity-corrected 3D cube) following the same
;       prescriptions as in S2N_RATIO_3D. The spaxels are then sorted into two
;       groups (continuum and emission-line), one in decreasing order of the
;       (S/N)c value and the other in decreasing order of the (S/N)em. The
;       script creates an integrated spectrum for each of the spaxels subsample
;       by adding the spectra in the sorted order as described before. In each
;       step, the procedure calculates the S/N in the continuum and/or emission
;       line feature in the new generated spectrum. The rationale of this
;       criterion is the following: spectra of good quality (high S/N) should
;       help to enhance the S/N of the integrated spectrum in each of the two
;       groups. However, if the inclusion of a new spectrum (spectra)
;       decreases the S/N of the integrated spectrum with respect to the values
;       found in previous steps, then the individual S/N of the spectrum
;       (spectra) for which this turn-off is found marks the tentative
;       threshold value (or range) sought for high quality spectra within an IFS
;       cube.
;
;
;       When launched, the program displays the integrated spectrum of the 
;       input IFS data, and shows the continuum and emission-line spectral regions
;       defined by the input parameters; in the emission-line case, the graph
;       also displays the pseudo-continuum adjacent bands used to calculate
;       (S/N)_eline. If the default values does not satisfy the user, the band
;       widths and central wavelenghts of the pseudo-continuum adjacent bands
;       can be set with the WCONT, WIDTH and PSEUDO keywords. The command
;       prompts for confirmation, if the current ranges are accepted, the
;       program calculates the S/N for each case and displays a new window with
;       six panels: The left-column shows the "statistical" S/N of the continuum
;       (top) and emission (bottom) features sorted in decreasing order as a
;       function of spaxel position. The blue vertical-line shows the position
;       at which S/N=1. The middle-column panels show the cumulative S/N of the
;       integrated spectra in the same sorted order as the left-panels. The red
;       vertical-line shows the position of the maximum S/N. Spaxels to the left
;       of this threshold are displayed in the FoV panels on the right-column. 
;       Statistical information is printed on the terminal. 
;       At this point, the user can interactively change the
;       spaxel threshold and/or the continuum S/N  calculation (as explained in
;       in S2N_RATIO_3D), the program prompts for new threshold input values and
;       updates the corresponding window. If the user accepts and proceeds with the
;       extraction, a whole series of files are created with the same format as
;       in S2N_RATIO_3D. The final selected spaxels and S/N maps are shown on
;       screen.
;
;
; For CALLING SEQUENCE, INPUTS and OPTIONAL KEYWORDS enter:
;
;        IDL> s2n_optimize
;
;
; OUTPUTS:
;
;
;        If PREFIX is set:
;
;     CONTINUUM sample
;     -------------------
;
;          Extracted RSS:   PREFIX.cont_rss.fits
;         Position table:   PREFIX.cont_rss.pt.txt
;         FoV Postscript:   PREFIX.cont_FoV.eps
;        Integrated FITS:   PREFIX.cont_integ.fits
;                  ASCII:   PREFIX.cont_integ.txt
;             Postscript:   PREFIX.cont_integ.eps
;        S/N table ASCII:   PREFIX.cont_s2n.txt
;           S/N map FITS:   PREFIX.cont_s2n.fits
;        Extraction mask:   PREFIX.cont_mask.fits
;
;     EMISSION sample
;     -------------------
;
;          Extracted RSS:   PREFIX.eline_rss.fits
;         Position table:   PREFIX.eline_rss.pt.txt
;         FoV Postscript:   PREFIX.eline_FoV.eps
;        Integrated FITS:   PREFIX.eline_integ.fits
;                  ASCII:   PREFIX.eline_integ.txt
;             Postscript:   PREFIX.eline_integ.eps
;        S/N table ASCII:   PREFIX.eline_s2n.txt
;           S/N map FITS:   PREFIX.eline_s2n.fits
;        Extraction mask:   PREFIX.eline_mask.fits
;
;       Output structure:   PREFIX.str.sav
;        Plots PNG image:   PREFIX.plots.png
;
;
;    If not PREFIX set, then PREFIX='s2n_optim'
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-


pro s2n_optimize, infile, cband, eline, out_str,  SQRTN=sqrtn, PROXY=proxy, SKIP=skip,$
                  S2N_MIN=s2n_min, WIDTH=width, WCONT=wcont, PSEUDO=pseudo, $
                  PT=pt, PREFIX=prefix, EXTENSION=extension, $
                  FILTER=filter, BAND=band, CT=ct, FONT=font, DRAW=draw, $
                  CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
                  SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
                  _EXTRA=extra

compile_opt idl2, hidden
on_error, 2

if n_params() lt 3 then begin
    print, ''
    print, "    s2n_optimize, 'OBJECT.fits', lam_cont, lam_eline  [, OUT.str, WIDTH=width, WCONT=wcont,  "
    print, '                   PSEUDO=[lam_pseudo1,lam_pseudo2], /SQRTN, /PROXY, /SKIP,'
    print, "                   PREFIX=prefix, PT='Postable.txt', EXTENSION=extension,"
    print, '                  (+ all graphic options as VIEW_3D) ]'

    print, ''
    print, ''
    print, "   'OBJECT.fits':  String of the wavelength calibrated RSS or 3Dcube FITS file"
    print, ''
    print, '        lam_cont:  Central (observed) wavelength of the (featureless) continuum band '
    print, '                   to calculate the (S/N)_cont'
    print, ''
    print, '       lam_eline:  Central (observed) wavelength of the emission line feature band '
    print, '                   to calculate the (S/N)_eline'
    print, ''
    print, '  OPTIONAL'
    print, ''
    print, '         OUT.str:  Name of the optional output structure'
    print, ''
    print, '           WIDTH:  Width (in Angstroms) of the spectral regions (elines and pseudo-continuum bands)'
    print, '                   from where the (S/N)_eline is obtained. Default: 50'
    print, ''
    print, '           WCONT:  Width (in Angstroms) of the continuum band from where the (S/N)_cont is obtained'
    print, '                   Default: 50' 
    print, ''
    print, '          PSEUDO:  A two-entries vector [lam_pseudo1,lam_pseudo2] containing the central wavelenghts  '
    print, '                   of the pseudo-continuum adjacent bands used to calculate (S/N)_eline.'
    print, '                   Default: [lam_eline-100,lam_eline+100]'
    print, ''
    print, '          /SQRTN:  When this keyword is set, the continuum noise is equal to sqrt(sigma). This may be useful'
    print, '                   when the noise is strongly non Poissonian, if the structure of the signal in the spaxels'
    print, '                   is not optimally weighted and/or if there are strong gradients in the S/N.'
    print, ''
    print, '          /PROXY:  When this keyword is set, the signal is calculated as the sum of the flux'
    print, '                   (instead of the mean) and the noise is equal to sqrt(signal). This definition'
    print, '                   represents a "density" S/N, and follows the surface brightness of the object at the'
    print, '                   chosen spectral band.'
    print, ''
    print, '           /SKIP:  Skips wavelength range confirmation and prints minimum output information on-screen'
    print, ''
    print, '              PT:  String of the position table of the RSS file in ASCII format'
    print, '                   (compulsory if the RSS+PTable names are not in the standard format and/or '
    print, '                    they are not included in the default instruments/setups)'
    print, ''
    print, '          PREFIX:  String with the prefix of the output files.'
    print, '                   (if not set, the program constructs a string based on the input information)'
    print, ''
    print, ''
    print, '     + all visual options of VIEW_3D'
    print, ''
    print, ''
    return
endif



;--------------------------  Initial tests

infile = sctrim(infile)

if file_test(infile) eq 0 then begin
   print, ''
   print, 'ERROR: The file:  "'+infile+'"  does not exist!'
   print, ''
   return
endif


;--------------------------  Routine properties

if not keyword_set(prefix) then prefix='s2n_optim'

tmp1 = strmid(infile,strpos(infile,'/',/reverse_search)+1)
tmp2 = strsplit(tmp1,'.fits',/extract,/regex)
object = tmp2[0]



;--------------------------  Graphic device settings
pingsoft_param, pings, ''
pingsoft_display, pings, /silent

; Color tables 
spawn, 'echo $PINGSOFT_PATH', path
ctfile = path+'/pingsoft_ct.tbl'


;--------------------------  PINGSoft core


pingsoft_core, infile, pings, core, ps_file, $
               PT=pt, PREFIX=prefix, $
               MIN_FLUX=min_flux, MAX_FLUX=max_flux, LMIN=lmin, LMAX=lmax, $
               FILTER=filter, BAND=band, CT=ct, DRAW=draw, PS=0, FONT=font, PLOT=0, $
               CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
               SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
               LARGE=large, NOEXTRACT=noextract,  _EXTRA=extra


; Variables from structure
tag = get_tags(core)
for k=0, n_elements(tag)-1 do begin
   dumb = strmid(tag[k],1)
   com = execute(dumb+' = core.'+dumb)
endfor


;-------------------------- Loading Poisson Noise from Monte Carlo

spawn, 'echo $PINGSOFT_PATH/', path_noise
restore, path_noise+'Poisson_MC.sav'

if not keyword_set(sqrtn) then sqrtn = 0
if not keyword_set(proxy) then proxy = 0


;-------------------------- Spectral range

if not keyword_set(width) then width=50
if not keyword_set(wcont) then wcont=50

if not keyword_set(pseudo) then begin
   cont1 = eline-100
   cont2 = eline+100
endif else begin
   ss = size(pseudo)
   if n_elements(ss) lt 4 or ss[1] ne 2 then begin
      print, ''
      print, ' ERROR: PSEUDO parameter must be a two-entry vector PSEUDO=[lam_pseudo1,lam_pseudo2]'
      print, ''
      return
   endif
   cont1 = pseudo[0]
   cont2 = pseudo[1]
endelse

; lambda ranges
 nn = where(lambda ge eline-width*0.5 and lambda le eline+width*0.5)
mm1 = where(lambda ge cont1-width*0.5 and lambda le cont1+width*0.5)
mm2 = where(lambda ge cont2-width*0.5 and lambda le cont2+width*0.5)
 mm = where(lambda ge cband-wcont*0.5 and lambda le cband+wcont*0.5)


; same baseline
 nv = n_elements(nn)
nn1 = n_elements(mm1)
nn2 = n_elements(mm2)
 nc = min([nn1,nn2])
nn1 = mm1[0:nc-1]
nn2 = mm2[0:nc-1]

elam = lambda[nn]
clam = lambda[mm]


;-------------------------- Testing ranges

loadct, 39, /silent
set_font

msize=get_screen_size()

if not keyword_set(skip) then begin

   !p.multi=[0,1,2]

   x0 = 0.10
   x1 = 0.98

   y0 = 0.06
   y1 = 0.46
   y2 = 0.55
   y3 = 0.95

   pos1 = [x0,y0,x1,y1]
   pos2 = [x0,y2,x1,y3]

   xs=msize[0]*0.4
   ys=msize[1]*0.7
   window, 0, xs=xs, ys=ys, title='PINGSoft:  S/N optimization'

   integ = fltarr(pixels)
   for l=0l, rows-1 do integ = integ + data[*,l]

   ; emission
   blim = cont1-width*3
   rlim = cont2+width*3
   if blim ge eline+width then blim = eline-width*2
   if rlim le eline-width then rlim = eline+width*2
   kk = where(lambda ge blim and lambda le rlim)
   lplot = lambda[kk]
   fplot = integ[kk]
   plot, lplot, fplot, xs=1, ys=1, yr=[0, max(fplot)*1.2], xr=[min(lplot),max(lplot)], $
          title='Emission line spectral range', position=pos2, charsize=0.8
   oplot, elam, integ[nn], color=50, thick=2
   oplot, lambda[nn1], integ[nn1], color=140, thick=2
   oplot, lambda[nn2], integ[nn2], color=140, thick=2

   ; continuum
   kk = where(lambda ge cband-wcont*5 and lambda le cband+wcont*5)
   lplot = lambda[kk]
   fplot = integ[kk]
   plot, lplot, fplot, xs=1, ys=1, yr=[0, max(fplot)*1.2], xr=[min(lplot),max(lplot)], $
        xtitle='Wavelength', title='Continuum band', position=pos1, charsize=0.8
   oplot, clam, integ[mm], color=250, thick=2

   xyouts, 0.03, 0.5, 'Flux (integrated spectrum)', align=0.5, orient=90, charsize=0.9, /norm

   val = ''
   print, ''
   print, ''
   print, ' PINGSoft:  S/N extraction'
   print, ' ============================================================='
   print, ''
   print, ' Emission line '
   print, ''
   print, '   Blue:  spectral range to calculate S/N'
   print, '   Green: pseudo-continuum bands'
   print, ''
   print, ''
   print, ' Continuum'
   print, ''
   print, '    Red: spectral range to calculate S/N'
   print, ''
   print, ''
   read, val, prompt=' Are these limits OK ?  (Y/n) '
   if val eq 'n' or val eq 'N' then begin
      print, ''
      print, ' Returning...'
      print, ''
      return
   endif
   
endif


;-------------------------- Sorting spectra by S/N

    s2n_cont = fltarr(rows)
   s2n_eline = fltarr(rows)
 signal_cont = fltarr(rows)
signal_eline = fltarr(rows)
  noise_cont = fltarr(rows)
 noise_eline = fltarr(rows) 

   Pnoise = noise[nn]

iter_noise = 0
change_n = 0

new_noise:

print, ''
print, ' Calculating S/N of all spaxels...'
print, '' 

for i=0l, rows-1 do begin

   ; Spaxel continuum flux
   fluxc = data[mm,i]

   ; Spaxel eline flux 
   fluxe  = data[nn,i]
   fcont1 = data[nn1,i]
   fcont2 = data[nn2,i]

   ; S/N ratio
   s2n = s2n_rat_func(clam,fluxc,fluxe,fcont1,fcont2,Pnoise,SQRTN=sqrtn,PROXY=proxy)

   ; Vectors
   s2n_cont[i] = s2n[0]
   s2n_eline[i] = s2n[1]

   signal_cont[i] = s2n[2]
   signal_eline[i] = s2n[3]
   noise_cont[i] = s2n[4]
   noise_eline[i] = s2n[5]

   bar, i, rows-1

endfor


;-------------------------- Sorting by S/N

; Sorting  (continuum)
new_id1 = reverse(sort(s2n_cont))
s2n_ok1 = s2n_cont[new_id1]

; Sorting  (eline)
new_id2 = reverse(sort(s2n_eline))
s2n_ok2 = s2n_eline[new_id2]


;-------------------------- Spatial ID/maxima

id_c = new_id1[0]
id_e = new_id2[0]

x_cont = x[id_c]
y_cont = y[id_c]

x_eline = x[id_e]
y_eline = y[id_e]


;-------------------------- Sample definitions
thres:

sample1 = s2n_ok1
index1 = new_id1

sample2 = s2n_ok2
index2 = new_id2

val1 = 0
val2 = 1


;-------------------------- Continuum case

print, ''
print, ' Calculating cumulative S/N (continuum) ...'
print, '' 

 in_fib1 = where(sample1 gt val1)
out_fib1 = where(sample1 le val1)

good_fib1 = index1[in_fib1]
 bad_fib1 = index1[out_fib1]

good_s2n1 = sample1[in_fib1]

int1 = fltarr(pixels)
all1 = fltarr(pixels,rows)


; Tests for integrated spectrum

ngood1 = n_elements(good_fib1)

cum1 = fltarr(ngood1)

for i=0, ngood1-1 do begin

   ; integrated spectrum
   id_ok = good_fib1[i]
   int1 = int1 + data[*,id_ok] ; /(i+1) not OK for cont
   all1[*,i] = int1

   ; cumulative S/N
   data_c = int1[mm]
   data_e = int1[nn]
   data_1 = int1[nn1]
   data_2 = int1[nn2]

   thisval =  s2n_rat_func(clam,data_c,data_e,data_1,data_2,Pnoise,SQRTN=sqrtn,PROXY=proxy)
   cum1[i] = thisval[0]

   bar, i, ngood1-1

endfor


;-------------------------- Eline extraction

print, ''
print, ' Calculating cumulative S/N (emission) ...'
print, '' 

 in_fib2 = where(sample2 gt val2)
out_fib2 = where(sample2 le val2)

good_fib2 = index2[in_fib2]
 bad_fib2 = index2[out_fib2]

good_s2n2 = sample2[in_fib2]

int2 = fltarr(pixels)
all2 = fltarr(pixels,rows)


; Tests for integrated spectrum

ngood2 = n_elements(good_fib2)

cum2 = fltarr(ngood2)

for i=0, ngood2-1 do begin

   ; integrated spectrum
   id_ok = good_fib2[i]
   int2 = int2 + data[*,id_ok] / (i+1) ; OK for elines
   all2[*,i] = int2

   ; cumulative S/N
   data_c = int2[mm]
   data_e = int2[nn]
   data_1 = int2[nn1]
   data_2 = int2[nn2]

   thisval =  s2n_rat_func(clam,data_c,data_e,data_1,data_2,Pnoise,SQRTN=sqrtn,PROXY=proxy)
   cum2[i] = thisval[1]

   bar, i, ngood2-1

endfor

xx1 = indgen(ngood1)+1
xx2 = indgen(ngood2)+1

again = 0

again:

if again eq 0 then begin
   vv1 = where(cum1 eq max(cum1)) ; continuum
   vv2 = where(cum2 eq max(cum2)) ; eline
endif



;--------------------------------------------------   Plots

xs=msize[0]*0.7
ys=msize[1]*0.77

!p.multi=[0,3,2]
window, 0, xs=xs, ys=ys, title='PINGSoft:  S/N optimization'

x0 = 0.05
x1 = 0.32
x2 = 0.37
x3 = 0.65
x4 = 0.70
x5 = 0.99

y0 = 0.05
y1 = 0.47
y2 = 0.54
y3 = 0.97

pos1 = [x0,y2,x1,y3]
pos2 = [x0,y0,x1,y1]

pos3 = [x2,y2,x3,y3]
pos4 = [x2,y0,x3,y1]

pos5 = [x4,y2,x5,y3]
pos6 = [x4,y0,x5,y1]

csize = 1.3


;---------------------------------  S/N > 0 samples

; Continuum
var = sample1
xl = indgen(n_elements(var))
maxy = max(var)*1.05
minx = min(xl)-rows*0.1

plot, xl, var, psym=10, xs=1, ys=1, xr=[minx,max(xl)], yr=[0,maxy], $
  ytitle='Statistical S/N (continuum)', xtitle='Spaxels', thick=2, charsize=csize, $
  title='Continuum band centered at: '+sctrim(cband), position=pos1

vv = where(var le 0)
if n_elements(vv) gt 1 then xval = min(xl[vv]) else xval = max(xl)
plots, [xval,xval], [0,maxy], thick=2, color=50


; Emission
var = sample2
xl = indgen(n_elements(var))
maxy = max(var)*1.05

plot, xl, var, psym=10, xs=1, ys=1, xr=[minx,max(xl)], yr=[0,maxy], $
  ytitle='Statistical S/N (emission)', xtitle='Spaxels', thick=2, charsize=csize, $
  title='Reference emission line at: '+sctrim(eline), position=pos2

vv = where(var le 1)
if n_elements(vv) gt 1 then xval = min(xl[vv]) else xval = max(xl)
plots, [xval,xval], [0,maxy], thick=2, color=50



;---------------------------------  Integrated spectra S/N

miny = min(cum1)
maxy = max(cum1)*1.05
plot, xx1, cum1, psym=10, xs=1, ys=1, xr=[1,max(xx1)], yr=[miny,maxy], $
  ytitle='Cumulative S/N', xtitle='Spaxels', thick=1, charsize=csize, $
  title='Integrated spectrum (continuum)', position=pos3, /xlog
plots, [xx1[vv1],xx1[vv1]], [miny,maxy], thick=2, color=250

miny = min(cum2)
maxy = max(cum2)*1.05
plot, xx2, cum2, psym=10, xs=1, ys=1, xr=[1,max(xx2)], yr=[miny,maxy], $
  ytitle='Cumulative S/N', xtitle='Spaxels', thick=1, charsize=csize, $
  title='Integrated spectrum (emission)', position=pos4, /xlog
plots, [xx2[vv2],xx2[vv2]], [miny,maxy], thick=2, color=250



;---------------------------  Spaxels on-screen

etype = 0

loadct, 1, /silent

; Spaxels window limits
box_limits = pingsoft_boxlim(id,x,y,radius)
xmax_mosaic = box_limits[0]
xmin_mosaic = box_limits[1]
ymax_mosaic = box_limits[2]
ymin_mosaic = box_limits[3]

etype:

if etype eq 0 then thispos = pos5 else thispos = pos6
if etype eq 0 then ptitle = object else ptitle=''

plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
      xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, charsize=csize, title=ptitle, $
      xtitle=pings.xtitle0, ytitle=pings.ytitle0, /noerase, position=thispos


; Plot spaxels

loadct, 1, file=ctfile, /silent

for i=0l, n_elements(x)-1 do begin
   x_fib = xarr + x[i]
   y_fib = yarr + y[i]
   polyfill, x_fib, y_fib, color=scale[i]
endfor

loadct, 39, /silent
vv1 = vv1[0]
vv2 = vv2[0]
if vv1 eq 0 then vv1 = 1
if vv2 eq 0 then vv2 = 1
if etype eq 0 then extract = good_fib1[lindgen(vv1)] else extract = good_fib2[lindgen(vv2)] 

for i=0l, n_elements(extract)-1 do begin
   id_ex = extract[i]
   x_fib = xarr + x[id_ex]
   y_fib = yarr + y[id_ex]
   oplot, x_fib, y_fib, color=250, thick=2
endfor

loadct, 1, /silent

if etype eq 0 then begin
   etype = 1
   goto, etype
endif



;---------------------------  Output info



print, ''
print, ''
print, ' PINGSoft:  S/N optimization by cumulative integrated spectra'
print, ' ============================================================='
print, ''
print, ' Total number of spaxels:   '+sctrim(rows)
print, ''
print, ''
print, ' CONTINUUM sample'
print, ' ------------------------'
print, '                    Mean:   '+sctrim(mean(sample1))
print, '                   Sigma:   '+sctrim(stddev(sample1))
print, '    Spaxels with S/N > 1:   '+sctrim(ngood1)
print, ''
print, '     Threshold at spaxel:   '+sctrim(xx1[vv1]-1)
print, '           S/N threshold:   '+sctrim(round(cum1[vv1]))
print, ''
print, ''
print, ' EMISSION sample'
print, ' ------------------------'
print, '                    Mean:   '+sctrim(mean(sample2))
print, '                   Sigma:   '+sctrim(stddev(sample2))
print, '    Spaxels with S/N > 1:   '+sctrim(ngood2)
print, ''
print, '     Threshold at spaxel:   '+sctrim(xx2[vv2]-1)
print, '           S/N threshold:   '+sctrim(round(cum2[vv2]))
print, ''
print, ''


;---------------------------  Select extraction values

ok_rss=''
print, ''
print,               ' Change NOISE calculation and/or spaxel THRESHOLD ?'
print, ''
print,               '                       (Y)es'
print,               '                       (n)o, proceed to extraction'
read, ok_rss, prompt='                       (a)bort                      '

if ok_rss eq 'a' or ok_rss eq 'A' then begin
   print, ''
   print, ' Returning...'
   print, ''
   return
endif

print, ''
loadct, 39,  /silent
if ok_rss eq 'n' or ok_rss eq 'N' then goto, final


if change_n eq 1 then goto, set_thres

nstr = 'STANDARD (1-SIGMA)'
if sqrtn eq 1 then nstr = '/SQRTN'
if proxy eq 1 then nstr = 'S/N PROXY'
print, ''
print,                  ' Current S/N continuum type:  '+nstr
print, ''
print,                  '      Change method?        No  [Enter]'
print,                  '                      STANDARD  [1]'
print,                  '                        /SQRTN  [2]'
new_noise = ''
read, new_noise, prompt='                        /PROXY  [3] '
print, ''
if new_noise eq 'n' or new_noise eq 'N' or new_noise eq '' then goto, set_thres
iter_noise = 1
change_n = 1
f = 0
com = execute('f='+new_noise)
if f eq 1 then begin
   sqrtn = 0
   proxy = 0
   print, ' NEW S/N continuum type:  STANDARD'
endif
if f eq 2 then begin
   sqrtn = 1
   proxy = 0
   print, ' NEW S/N continuum type:  /SQRTN'
endif
if f eq 3 then begin
   sqrtn = 0
   proxy = 1
   print, ' NEW S/N continuum type:  /PROXY'
endif
goto, new_noise

set_thres:

print, ''
print, ' New threshold values (sorted spaxel)'
print, ''

read, vv1, prompt='      Continuum sample:  '
print, ''
if vv1 gt ngood1 then begin
   print, ''
   print, ' !!!!! WARNING: input spaxel out of sample range, using: '+sctrim(ngood1)
   vv1 = ngood1-1
endif

read, vv2, prompt='       Emission sample:  '
print, ''
if vv2 gt ngood2 then begin
   print, ''
   print, ' !!!!! WARNING: input spaxel out of sample range, using: '+sctrim(ngood2)
   vv2 = ngood2-1
endif

change_n = 0
   again = 1

goto, again

final:
extract1 = good_fib1[indgen(vv1)]
extract2 = good_fib2[indgen(vv2)]

s2n_min_set:

;-------------------------  Final extraction

print, ''
print, ' Extracting spectra...'
print, ''


;------------- PNG image
out_png = prefix+'.plots'
imagen = tvread(filename=out_png, /PNG, /NODIALOG, QUALITY=100)


;------------- Masks and S/N maps

if type eq 2 then begin

   ; Resolution
   ll = n_elements(x)
   testx = fltarr(ll)
   for i=0, ll-2 do testx[i] = abs(x[i+1]-x[i])
   testx = testx[where(testx gt 0.)]
   res = min(testx)

   xx = ( max(x) - min(x) + res ) / res
   yy = ( max(y) - min(y) + res ) / res

   ; Geometry
   testx = x[0:xx-1]
   if min(testx) eq max(testx) then geometry = 2 else geometry = 1

endif

; Masks
mask1 = fltarr(xx,yy)
mask2 = fltarr(xx,yy)
mask1[extract1] = 1
mask2[extract2] = 1

out_cont_mask  = prefix+'.cont_mask.fits'
   
writefits, out_cont_mask, mask1, hdr2
   
out_eline_mask  = prefix+'.eline_mask.fits'
writefits, out_eline_mask, mask2, hdr2

; S/N Maps
map1 = fltarr(xx,yy)
map2 = fltarr(xx,yy)

index_m1 = where(s2n_cont eq s2n_cont)
index_m2 = where(s2n_eline eq s2n_eline)
   
map1[index_m1] = round(s2n_cont)
map2[index_m2] = round(s2n_eline)
   
out_cont_map = prefix+'.cont_s2n.fits'
writefits, out_cont_map, map1, hdr2
   
out_eline_map = prefix+'.eline_s2n.fits'
writefits, out_eline_map, map2, hdr2


; S/N plots
dband = map1 * mask1
scale1 = AsinhScl(dband, beta=10, min=min(dband), max=max(dband), /negative)
dband = map2 * mask2
scale2 = AsinhScl(dband, beta=10, min=min(dband), max=max(dband), /negative)



;------------- Continuum
extra = {title:'Continuum S/N extraction'}
extract_index, infile, extract1, cc, prefix='tmp_cont', band=cband, /ps, /silent, $
               FONT=font, DRAW=draw, FILTER=filter, CT=ct, CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
               SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian, _EXTRA=extra


out_cont_rss  = prefix+'.cont_rss.fits'
out_cont_pt   = prefix+'.cont_rss.pt.txt'
out_cont_fits = prefix+'.cont_integ.fits'
out_cont_asci = prefix+'.cont_integ.txt'
out_cont_ps   = prefix+'.cont_integ.eps'
out_cont_fov   = prefix+'.cont_FoV.ps'

spawn, 'mv tmp_cont_rss.fits   '+out_cont_rss
spawn, 'mv tmp_cont_rss.pt.txt '+out_cont_pt
spawn, 'mv tmp_cont_integ.fits '+out_cont_fits
spawn, 'mv tmp_cont_integ.txt  '+out_cont_asci
spawn, 'mv tmp_cont_integ.eps  '+out_cont_ps
spawn, 'mv tmp_cont_FoV.eps    '+out_cont_fov



;------------- Emission
extra = {title:'Emission line S/N extraction'}
extract_index, infile, extract2, ee, prefix='tmp_eline', band=eline, /ps, /silent, $
               FONT=font, DRAW=draw, FILTER=filter, CT=ct, CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
               SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian, _EXTRA=extra

out_eline_rss  = prefix+'.eline_rss.fits'
out_eline_pt   = prefix+'.eline_rss.pt.txt'
out_eline_fits = prefix+'.eline_integ.fits'
out_eline_asci = prefix+'.eline_integ.txt'
out_eline_ps   = prefix+'.eline_integ.eps'
out_eline_fov  = prefix+'.eline_FoV.eps'

spawn, 'mv tmp_eline_rss.fits   '+out_eline_rss
spawn, 'mv tmp_eline_rss.pt.txt '+out_eline_pt
spawn, 'mv tmp_eline_integ.fits '+out_eline_fits
spawn, 'mv tmp_eline_integ.txt  '+out_eline_asci
spawn, 'mv tmp_eline_integ.eps  '+out_eline_ps
spawn, 'mv tmp_eline_FoV.eps    '+out_eline_fov



;------------- S/N tables

out_cont_table = prefix+'.cont_s2n.txt'

openw, 1, out_cont_table
printf,1, '#'
printf,1, '# PINGSoft: Continuum based S/N tables'
printf,1, '#'
printf,1, '# Object: '+object
printf,1, '#'
printf,1, '# Columns:   X,  Y,  Signal,  Noise,  S/N ratio,  Index'
printf,1, '# -----------------------------------------------------------------------------'

for i=0, n_elements(extract1)-1 do begin
   id = extract1[i]
   printf, 1, x[id], y[id], signal_cont[id], noise_cont[id], s2n_cont[id], id
endfor
close, 1

out_eline_table = prefix+'.eline_s2n.txt'

openw, 1, out_eline_table
printf,1, '#'
printf,1, '# PINGSoft: Emission line based S/N tables'
printf,1, '#'
printf,1, '# Object: '+object
printf,1, '#'
printf,1, '# Columns:   X,  Y,  Signal,  Noise,  S/N ratio,  Index'
printf,1, '# -----------------------------------------------------------------------------'

for i=0, n_elements(extract2)-1 do begin
   id = extract2[i]
   printf, 1, x[id], y[id], signal_eline[id], noise_eline[id], s2n_eline[id], id
endfor
close, 1


;-------------------------- Variables into structure

out_sav = prefix+'.str.sav'

s = { $

    cont_s2n:         s2n_cont      ,$
    cont_signal:      signal_cont   ,$
    cont_noise:       noise_cont    ,$
    cont_finteg:      cc.finteg     ,$
    cont_index:       extract1      ,$
    cont_s2n_map:     map1          ,$
    cont_s2n_mask:    mask1         ,$
    cont_rss:         out_cont_rss  ,$

    eline_s2n:        s2n_eline     ,$
    eline_signal:     signal_eline  ,$
    eline_noise:      noise_eline   ,$
    eline_finteg:     ee.finteg     ,$
    eline_index:      extract2      ,$
    eline_s2n_map:    map2          ,$
    eline_s2n_mask:   mask2         ,$
    eline_rss:        out_eline_rss ,$

    x:               x              ,$
    y:               y              ,$
    xdim:            xx             ,$
    ydim:            yy             ,$
    lambda:          cc.lambda      ,$
    eof:'' $
    }

save, s, filename=out_sav
out_str = s


;-------------------------- Final plots S/N

;  Graphic device settings
pingsoft_param, pings, ''
pingsoft_display, pings, /silent


xs=msize[0]*0.48
ys=msize[1]*0.75

!p.multi=[0,2,2]
window, 0, xs=xs, ys=ys, title='PINGSoft: S/N optimized spectra'

x0 = 0.10
x1 = 0.47
x2 = 0.57
x3 = 0.95

y0 = 0.06
y1 = 0.45
y2 = 0.55
y3 = 0.95

pos1 = [x0,y2,x1,y3]
pos2 = [x0,y0,x1,y1]

pos3 = [x2,y2,x3,y3]
pos4 = [x2,y0,x3,y1]
   
csize = 0.8
ssize = 0.5

spax1 = indgen(n_elements(s2n_cont))
spax2 = indgen(n_elements(s2n_eline))

maxy1 = max(s2n_cont)*1.1
maxy2 = max(s2n_eline)*1.1

loadct, 39, /silent

; Continuum
plot, spax1, s2n_cont, psym=1, xs=1, ys=1, yr=[0,maxy1], symsize=ssize,  $
      xthick=pings.wthick, ythick=pings.wthick, charsize=csize, $
      ytitle='S/N (continuum)', xtitle='Spaxels', $
      title='Continuum band at: '+sctrim(cband), position=pos1
oplot, spax1[extract1], s2n_cont[extract1], psym=1, symsize=ssize, color=250

; Emission
plot, spax2, s2n_eline, psym=1, xs=1, ys=1, yr=[0,maxy2], symsize=ssize, $
      xthick=pings.wthick, ythick=pings.wthick, charsize=csize, $
      ytitle='S/N (emission)', xtitle='Spaxels', $
      title='Reference emission line at: '+sctrim(eline), position=pos2
oplot, spax2[extract2], s2n_eline[extract2], psym=1, symsize=ssize, color=250

ptype = 0
loadct, 39, /silent

ptype:

if ptype eq 0 then mscale = scale1 else mscale = scale2

if ptype eq 0 then thispos = pos3 else thispos = pos4
if ptype eq 0 then ptitle = 'S/N continuum map' else ptitle='S/N emission map'

plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
      xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, charsize=csize, title=ptitle, $
      xtitle=pings.xtitle0, ytitle=pings.ytitle0, /noerase, position=thispos

; Spaxels
for i=0l, n_elements(x)-1 do begin
   x_fib = xarr + x[i]
   y_fib = yarr + y[i]
   polyfill, x_fib, y_fib, color=mscale[i]
endfor

if ptype eq 0 then begin
   ptype = 1
   goto, ptype
endif

loadct, 1, /silent



;-------------------------- Final screen output

print, ''
print, ''
print, ' PINGSoft:  S/N optimization'
print, ' ======================================================='
print, ''
print, ' Files written:'
print, ''
print, ' CONTINUUM sample'
print, ' -------------------'
print, ''
print, '      Extracted RSS:   '+out_cont_rss
print, '     Position table:   '+out_cont_pt
print, '     FoV Postscript:   '+out_cont_ps
print, '    Integrated FITS:   '+out_cont_fits
print, '              ASCII:   '+out_cont_asci
print, '         Postscript:   '+out_cont_ps
print, '    S/N table ASCII:   '+out_cont_table
print, '       S/N map FITS:   '+out_cont_map
print, '    Extraction mask:   '+out_cont_mask
print, ''

print, ''
print, ' EMISSION sample'
print, ' -------------------'
print, ''
print, '      Extracted RSS:   '+out_eline_rss
print, '     Position table:   '+out_eline_pt
print, '     FoV Postscript:   '+out_eline_ps
print, '    Integrated FITS:   '+out_eline_fits
print, '              ASCII:   '+out_eline_asci
print, '         Postscript:   '+out_eline_ps
print, '    S/N table ASCII:   '+out_eline_table
print, '       S/N map FITS:   '+out_eline_map
print, '    Extraction mask:   '+out_eline_mask
print, ''

print, ''
print, '    Output structure:   '+out_sav
print, '     Plots PNG image:   '+out_png
print, ''
print, ''


fin:
!p.multi=[0,1,1]
loadct, 39, /silent

end


