

;+
; NAME:
;       SET_VALUE2D
;
; PURPOSE:
;
;       Replaces the values in a 2D array of an index vector found with the IDL
;       WHERE()function.
;       
;
; For CALLING SEQUENCE, INPUTS and OPTIONAL KEYWORDS enter:
;
;        IDL> result = set_value2d()
;
;
; EXAMPLE:
;
;       IDL> image = congrid(bindgen(3,3),9,9)
;
;       IDL> print, image
;           0   0   0   1   1   1   2   2   2
;           0   0   0   1   1   1   2   2   2
;           0   0   0   1   1   1   2   2   2
;           3   3   3   4   4   4   5   5   5
;           3   3   3   4   4   4   5   5   5
;           3   3   3   4   4   4   5   5   5
;           6   6   6   7   7   7   8   8   8
;           6   6   6   7   7   7   8   8   8
;           6   6   6   7   7   7   8   8   8
;    
;       IDL> index = where(image eq 4)
;
;       IDL> new = set_value2d(image,index,9)
;
;       IDL> print, new
;           0   0   0   1   1   1   2   2   2
;           0   0   0   1   1   1   2   2   2
;           0   0   0   1   1   1   2   2   2
;           3   3   3   9   9   9   5   5   5
;           3   3   3   9   9   9   5   5   5
;           3   3   3   9   9   9   5   5   5
;           6   6   6   7   7   7   8   8   8
;           6   6   6   7   7   7   8   8   8
;           6   6   6   7   7   7   8   8   8
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-


function set_value2d, image, index, value

compile_opt idl2, hidden
on_error, 2

if n_params() lt 1 then begin
   print, ''
   print, " result = set_value2D( image, index, value )"
   print, ''
   print, '    image:  Variable name of the 2D array where values will be replaced.'
   print, ''
   print, '    index:  Index vector produced by the WHERE() function.'
   print, ''
   print, '    value:  Value to be replaced within the 2D array at the index locations.'
   print, ''
   print, '   result:  New 2D array with the values substituted.'
   print, ''
   goto, fin
endif

s = size(image)
ncol = s[1]
nrow = s[2]

col = index MOD ncol
row = index / ncol

nimage = image

for k=0, nrow-1 do begin
   mm = where(row eq k)
   if n_elements(mm) eq 1 and mm[0] lt 0 then goto, skip
   new = col[mm]
   nimage[new,k] = value
skip:
endfor

return, nimage
fin:
end
