


;+
; NAME:
;       SET_VALUE3D
;
; PURPOSE:
;
;       Replaces vectors in a 3D array of an index vector found with the IDL
;       WHERE()function.
;       
;
; For CALLING SEQUENCE, INPUTS and OPTIONAL KEYWORDS enter:
;
;        IDL> result = set_value3d()
;
;
; EXAMPLE:
;
;       IDL> cube = bindgen(3,3,3)
;
;       IDL> print, cube
;         0   1   2
;         3   4   5
;         6   7   8
;
;         9  10  11
;        12  13  14
;        15  16  17
;
;        18  19  20
;        21  22  23
;        24  25  26
;
;       Index vector produced by the WHERE() function  
;       based on a 2D slice of the cube:
;
;       IDL> index = where(cube[*,*,0] gt 5)
;
;       IDL> vector = [0,0,0]
;
;       IDL> new = set_value3d(cube,index,vector)
;
;       IDL> print, new
;         0   1   2
;         3   4   5
;         0   0   0
;
;         9  10  11
;        12  13  14
;         0   0   0
;
;        18  19  20
;        21  22  23
;         0   0   0
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-



function set_value3d, cube, index, vector

compile_opt idl2, hidden
on_error, 2

if n_params() lt 1 then begin
   print, ''
   print, " result = set_value3D( cube, index, vector )"
   print, ''
   print, '     cube:  Variable name of the 3D array where vectors will be replaced'
   print, ''
   print, '    index:  Index vector produced by the WHERE() function '
   print, '            (based on a 2D slice of the cube)'
   print, ''
   print, '   vector:  Vector to be replaced within the 3D array at the index locations.'
   print, ''
   print, '   result:  New 3D array with the vectors substituted.'
   print, ''
   goto, fin
endif

s = size(cube)
ncol = s[1]
nrow = s[2]

col = index MOD ncol
row = (index / ncol) MOD nrow
;frame = index / (nrow*ncol)

ncube = cube

for k=0, nrow-1 do begin
   mm = where(row eq k)
   if n_elements(mm) eq 1 and mm[0] lt 0 then goto, skip
   new_col = col[mm]
   for j=0, n_elements(new_col)-1 do begin
      new = new_col[j]
      ncube[new,k,*] = vector
   endfor
skip:
endfor


return, ncube
fin:
end
