
pro shift_ptable, in_file, out_file, x_off, y_off, MOSAIC=mosaic, SILENT=silent


;+
; NAME:
;       SHIFT_PTABLE
;
; PURPOSE:
;
;       Shifts the reference point or applies an offset to a given position table.
;
;
; EXPLANATION:
;
;       The program generates a new position table in which the reference point
;       of the original table is shifted by an offset (in arcsec) provided by
;       the user. Additionally, by selecting the /MOSAIC option, the output
;       position table is shifted with respect to the input table for the
;       purpose of building an IFS mosaic.
;
; CALLING SEQUENCE:
;       
;       shift_ptable, 'in_PosTable.txt', 'out_PosTable.txt', x_offset, y_offset [, /MOSAIC]
;
; INPUTS:
;       
;  'in_PosTable.txt':  String of the original position table.
;
; 'out_PosTable.txt':  Name of the shifted position table.
;
;         x/y_offset:  RA/Dec offsets in arcsec of the new reference point.
;
;
; OPTIONAL KEYWORDS:
;
;            /MOSAIC:  Shifts the position table to a NE orientation offset for
;                      mosaicking purposes.
;
; EXAMPLES:
;       
;       shift_ptable, 'original_pt.txt', 'shift_pt.txt', 4, -6
;
;
; REVISION HISTORY:
;
;       Written by:     F. Fabian Rosales-Ortega
;
;       First version:  January, 2010
;
; NOTE:
;
;       This routine belongs to the PINGS IDL software available at:
;
;       http://www.ast.cam.ac.uk/research/pings
;
;       where additional documentation can be found. If you find this code
;       useful on your research, please acknowledge the use of PINGSoft by
;       including the original project reference in your publications.
;
; COPYRIGHT:
;
;       This file is part of PINGSoft.
;
;       Copyright (C) 2010  F. Fabian Rosales-Ortega
;
;       PINGSoft is free software: you can redistribute it and/or modify it
;       under the terms of the GNU General Public License as published by the
;       Free Software Foundation, version 3. See the GNU General Public License
;       for more details.
;
;       PINGSoft is distributed in the hope that it will be useful, but WITHOUT
;       ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;       FITNESS FOR A PARTICULAR PURPOSE. 
;
;       The GNU General Public License is found in: http://www.gnu.org/licenses
;
;-

compile_opt idl2, hidden
on_error, 2


if n_params() lt 3 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "          shift_ptable, 'in_PosTable.txt', 'out_PosTable.txt', x_offset, y_offset [, /MOSAIC, /SILENT ]"
    print, ''
    print, ''
    print, "     'in_PosTable.txt':  String of the original position table."
    print, ''
    print, "    'out_PosTable.txt':  Name of the shifted position table."
    print, ''
    print, '            x/y_offset:  RA/Dec offsets in arcsec of the new reference point.'
    print, ''
    print, '               /MOSAIC:  Shifts the position table to a NE orientation offset for'
    print, '                         mosaicking purposes.'
    print, ''
    return
endif


;--------------------------  Initial tests


 in_file = sctrim(in_file)
out_file = sctrim(out_file)

if file_test(in_file) eq 0 then begin
    print, ''
    print, 'ERROR: The file:  "'+in_file+'"  does not exist!'
    print, ''
    return
endif


; Spaxel type (reading first line of the position table)

readcol, in_file, f='a,f,f,i', mtype, xsize, ysize, numline=1, /silent


; Reading the original table

readcol, in_file, f='l,f,f,i', id, x, y, flag, /silent

if keyword_set(mosaic) then begin
    xx = x + x_off
    yy = y + y_off
endif else begin
    xx = x - x_off
    yy = y - y_off
endelse


; Writing new position table

openw, lun, out_file, /get_lun

if mtype eq 'C' then printf, lun, 'C  '+sctrim(xsize)+'  '+sctrim(ysize)+'  0'
if mtype eq 'S' then printf, lun, 'S  '+sctrim(xsize)+'  '+sctrim(ysize)+'  0'

for i=0l, n_elements(id)-1 do begin

    str = string(id[i], xx[i], yy[i], flag[i])
    printf, lun, strtrim(str, 1)

endfor

close, lun
free_lun, lun


if not keyword_set(silent) then begin
    print, ''
    print, 'File created: ', out_file
    print, ''
endif


end
