
;+
; NAME:
;       SPLIT_CALIFA
;
; PURPOSE:
;
;       Extracts the FITS extensions of the CALIFA survey data, writing invididual
;       files with the appropriate headers for 3D visualisation.
;
;
; For CALLING SEQUENCE, INPUTS and OPTIONAL KEYWORDS enter:
;
;        IDL> split_califa
;
; OUTPUTS:
;
;        For 3D cubes, if PREFIX is set:
;
;                    Data:  PREFIX.DATA.fits
;                  Errors:  PREFIX.ERROR.fits
;           Error scaling:  PREFIX.ERRWEIGHT.fits
;              Bad pixels:  PREFIX.BADPIX.fits
;
;        For RSS files:
;
;                    Data:  PREFIX.DATA.fits
;                  Errors:  PREFIX.ERROR.fits
;              Bad pixels:  PREFIX.BADPIX.fits
;          Position table:  PREFIX.POSTABLE.fits (binary table)
;
;        If PREFIX is not set, then PREFIX=OBJECT (e.g. 'NGC1234.V500')
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-


pro split_califa, infile, PREFIX=prefix

compile_opt idl2, hidden
on_error, 2

if n_params() lt 1 then begin
    print, ''
    print, " split_califa, 'OBJECT.fits'  [, PREFIX='prefix']"
    print, ''
    print, ''
    print, "   'OBJECT.fits':  String of the CALIFA 3D cube or RSS FITS file"
    print, ''
    print, ''
    print, '    Extracts the FITS extensions for the CALIFA data: '
    print, ''
    print, '      3D cubes:  DATA, ERROR, ERRWEIGHT, BADPIX '
    print, ''
    print, '     RSS files:  DATA, ERROR, BADPIX, POSTABLE'
    print, ''
    return
endif


;--------------------------  Initial tests

infile = strcompress(string(infile))

if file_test(infile) eq 0 then begin
   print, ''
   print, 'ERROR: The file:  "'+infile+'"  does not exist!'
   print, ''
   return
endif


;--------------------------  Strings

tmp0 = strmid(infile,strpos(infile,'/',/reverse_search)+1)
tmp1 = strsplit(tmp0,'.rscube.fits.gz',/extract,/regex)
tmp2 = strsplit(tmp1,'.rscube.fits',/extract,/regex)
tmp3 = strsplit(tmp2,'.rss.fits.gz',/extract,/regex)
tmp4 = strsplit(tmp3,'.rss.fits',/extract,/regex)
tmp5 = strsplit(tmp4,'.fits.gz',/extract,/regex)
tmp6 = strsplit(tmp5,'.fits',/extract,/regex)
object = tmp6

if not keyword_set(prefix) then prefix=object


;--------------------------  Reading data

fits = readfits(infile,hdr,/silent)

size = size(fits)
type = size[0]

author = fxpar(hdr,'AUTHOR')
author = string(author)


if author ne 'CALIFA Collaboration' then begin
   print, ''
   print, ' ERROR: input file '+tmp0+' does not seem to be a CALIFA 3D cube'
   print, ''
   return
endif


print, ''
print, ' Extracting extensions...'
print, ''

if author eq 'CALIFA Collaboration' then begin

   ; Reading extensions

   if type eq 2 then begin

      data0 = fits
      data1 = readfits(infile,       ext=1, /silent) ; ERROR
      data2 = readfits(infile,       ext=2, /silent) ; BADPIX
      data3 = readfits(infile, hdr3, ext=3, /silent) ; POSTABLE

      hdr0=hdr
      sxdelpar, hdr0, 'EXTEND'
      fxaddpar, hdr0, 'EXTNAME', 'DATA', ' FITS extension extracted by PINGSoft', after='OBJECT'

      hdr1=hdr
      sxdelpar, hdr1, 'EXTEND'
      fxaddpar, hdr1, 'EXTNAME', 'ERROR', ' FITS extension extracted by PINGSoft', after='OBJECT'

      hdr2=hdr
      sxdelpar, hdr2, 'EXTEND'
      fxaddpar, hdr2, 'BITPIX', 8
      fxaddpar, hdr2, 'EXTNAME', 'BADPIX', ' FITS extension extracted by PINGSoft', after='OBJECT'

   endif

   if type eq 3 then begin

      data0 = fits
      data1 = readfits(infile, ext=1, /silent) ; ERROR
      data2 = readfits(infile, ext=2, /silent) ; ERRWEIGHT
      data3 = readfits(infile, ext=3, /silent) ; BADPIX

      hdr0=hdr
      sxdelpar, hdr0, 'EXTEND'
      fxaddpar, hdr0, 'EXTNAME', 'DATA', ' FITS extension extracted by PINGSoft', after='OBJECT'

      hdr1=hdr
      sxdelpar, hdr1, 'EXTEND'
      fxaddpar, hdr1, 'EXTNAME', 'ERROR', ' FITS extension extracted by PINGSoft', after='OBJECT'

      hdr2=hdr
      sxdelpar, hdr2, 'EXTEND'
      fxaddpar, hdr2, 'EXTNAME', 'ERRWEIGHT', ' FITS extension extracted by PINGSoft', after='OBJECT'

      hdr3=hdr
      sxdelpar, hdr3, 'EXTEND'
      fxaddpar, hdr3, 'BITPIX', 8
      fxaddpar, hdr3, 'EXTNAME', 'BADPIX', ' FITS extension extracted by PINGSoft', after='OBJECT'

   endif

endif


;---------------  Output

if type eq 2 then begin

   out0 = prefix+'.DATA.fits'
   out1 = prefix+'.ERROR.fits'
   out2 = prefix+'.BADPIX.fits'
   out3 = prefix+'.POSTABLE.fits'

   writefits, out0, data0, hdr0
   writefits, out1, data1, hdr1
   writefits, out2, data2, hdr2
   writefits, out3, data3, hdr3

   print, ''
   print, ' PINGSoft'
   print, ' ================'
   print, ''
   print, ' Files written '
   print, ''
   print, '       Data cube:  '+out0
   print, '          Errors:  '+out1
   print, '      Bad pixels:  '+out2
   print, '  Position table:  '+out3
   print, ''

endif


if type eq 3 then begin

   out0 = prefix+'.DATA.fits'
   out1 = prefix+'.ERROR.fits'
   out2 = prefix+'.ERRWEIGHT.fits'
   out3 = prefix+'.BADPIX.fits'

   writefits, out0, data0, hdr0
   writefits, out1, data1, hdr1
   writefits, out2, data2, hdr2
   writefits, out3, data3, hdr3

   print, ''
   print, ' PINGSoft'
   print, ' ================'
   print, ''
   print, ' Files written '
   print, ''
   print, '       Data cube:  '+out0
   print, '          Errors:  '+out1
   print, '   Error scaling:  '+out2
   print, '      Bad pixels:  '+out3
   print, ''

endif   


end
