

;+
; NAME:
;       VFIELD_3D
;
; PURPOSE:
;
;       Calculates the intrinsic velocity field in 3D data using a wavelength
;       cross-correlation.
;
; EXPLANATION:
;  
;        Taking the user's given reference spaxel and a spectral window containing
;        an emission line within that spectrum, the command applies a
;        cross-correlation in wavelength with respect to the reference template
;        and corrects the IFS data by the calculated shift in lambda. The spaxel
;        reference entry can be either an integer number with the ID value of
;        the reference spectrum (in IDL format) or a two element vector [x,y], 
;        with the pixel coordinates (offsets) of the reference spectrum (only
;        for 3D cubes). The procedure writes a new data file with the common
;        velocity field, a 2D image velocity map (only if the input file is a 3D
;        cube), and shows on screen histogram of the wavelenght shifts with
;        respect to the reference spectrum (in lambda units). Additional
;        information is printed on the terminal. If the /PS keyword is set,
;        the program writes an encapsulated Postscript file of the wavelength
;        shift histrogram.
;
;
; For CALLING SEQUENCE, INPUTS and OPTIONAL KEYWORDS enter:
;
;        IDL> vfield_3d
;
;
; OUTPUTS:
;
;        If PREFIX is set:
;
;         Velocity field corrected FITS:  PREFIX_vfield.fits
;                     Velocity map FITS:  PREFIX_vel.fits      (if input file is a 3D cube)
;                     Binary file saved:  PREFIX_vfield.sav
;                     Histogram PS file:  PREFIX_vfield.eps    (if /PS is set)
;
;        If PREFIX is not set: PREFIX=OBJECT
;
;
; COMMENTS
;
;        The program has been tested using only strong emission lines as
;        spectral templates, expect strange behaviour with absorption or weak
;        features. Note that for AGN/LINER kind of spectra, the program can be
;        confused if [NII] emission is of the same intensity (or greater) than
;        Halpha.
;
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-

pro vfield_3d, infile, id_ref, eline, out_str, PT=pt, PREFIX=prefix, EXTENSION=extension, $
               VMAX=vmax, LMIN=lmin, LMAX=lmax, PS=ps, CONT1=cont1, CONT2=cont2, FORCE_FIT=force_fit 


compile_opt idl2, hidden
on_error, 2


if n_params() lt 3 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "        vfield_3D, 'OBJECT.fits', spaxel_ref, eline_ref [, OUT.str, VMAX=vmax, /PS, "
    print, '                    LMIN=lmin, LMAX=lmax, CONT1=cont1, CONT2=cont2, /FORCE_FIT'
    print, '                    PREFIX=prefix, PT=pt, EXTENSION=extension ]'
    print, ''
    print, ''
    print, "    'OBJECT.fits':  String of the wavelength calibrated RSS or 3Dcube FITS file"
    print, ''
    print, '       spaxel_ref:  Either, an integer number with the ID value of the reference spectrum '
    print, '                    (as seen by VIEW_3D, in IDL format) or a two element vector [x,y], '
    print, '                    with the pixel coordinates of the reference spectrum (only for 3D cubes).'
    print, ''
    print, '        eline_ref:  Wavelength guess value of the emission line to use in order to perform '
    print, '                    the cross-correlation, e.g. Halpha 6563*(1+z)'
    print, ''
    print, '  OPTIONAL KEYWORDS'
    print, ''
    print, '          OUT.str:  Name of the optional output structure (including OUT.vfield, '
    print, '                    i.e. the shift applied to the spectra with respect to the reference spectrum).'
    print, ''
    print, '             VMAX:  Maximum velocity (km/s) with respect to the reference spectrum for which '
    print, '                    a wavelength shift is applied. Spectra with larger values are not shifted.'
    print, '                    Default: 500 km/s'
    print, ''
    print, '         LMIN/MAX:  min/max wavelength for the cross-correlation, default: eline +/- 100 Ang'
    print, ''
    print, '          CONT1/2:  Continuum bands used to normalize the spectra, default: eline +/- 80 Ang'
    print, ''
    print, '              /PS:  Writes an encapsulated Postscript file of the wavelength shift histrogram'
    print, ''
    print, '           PREFIX:  String with the prefix of the output files. Default: OBJECT'
    print, ''
    print, '               PT:  String of the position table of the RSS file in ASCII format'
    print, '                    (compulsory if the RSS+PTable names are not in the standard format and/or '
    print, '                    they are not included in the default instruments/setups)'
    print, ''
    print, '       /FORCE_FIT:  Forces a Gaussian fit to each spectra before calculating the cross-correlation'
    print, '                    (use with care, slower and somewhat inefficient)'
    print, ''
    return
endif


;--------------------------  Initial tests

infile = sctrim(infile)

if file_test(infile) eq 0 then begin
    print, ''
    print, 'ERROR: The file:  "'+infile+'"  does not exist!'
    print, ''
    return
endif



;--------------------------  Graphic device settings
pingsoft_param, pings, ''
pingsoft_display, pings, /silent


;--------------------------  PINGSoft core

if keyword_set(ps) then tmp_ps = 1 else tmp_ps = 0
this_ps = 0

pingsoft_core, infile, pings, core, ps_file, $
               PT=pt, PREFIX=prefix, $
               MIN_FLUX=min_flux, MAX_FLUX=max_flux, $
               FILTER=filter, BAND=band, CT=ct, DRAW=draw, PS=this_ps, FONT=font, PLOT=0, $
               LINEAR=linear, GAMMA=gamma, LOG=log, ASINH=asinh, $
               LARGE=large, NOEXTRACT=noextract, CALIFA=califa, _EXTRA=extra

; Variables from structure
tag = get_tags(core)
for k=0, n_elements(tag)-1 do begin
   dumb = strmid(tag[k],1)
   com = execute(dumb+' = core.'+dumb)
endfor

lam = lambda
ps = tmp_ps

if not keyword_set(prefix) then prefix=object
ps_file = prefix+'_vfield.eps'


;--------------------------  Reference spectrum

n_elem = size(id_ref, /n_elements)

if n_elem eq 1 then goto, idref_ok

if n_elem eq 2 and type eq 2 then begin
   print, ''
   print, 'ERROR:  spaxel_ref in pixels ([a,b]) only works with 3D cubes'
   print, ''
   return
endif

if n_elem eq 2 and type eq 3 then begin  ; find reference fibre based on pixels coord
   
   xid = id_ref[0]
   yid = id_ref[1]
   
   ncol = xx
   k=0
   diff=1e6
   while diff ne 0 do begin
      fibre = id[k]-1
      col = (fibre MOD ncol)+1
      row = (fibre / ncol)+1
      xval = abs(col - xid)
      yval = abs(row - yid)
      diff = xval + yval
      k = k+1
   endwhile
   id_ref = fibre
endif

idref_ok:

if not keyword_set(cont1) then cont1 = eline-80
if not keyword_set(cont2) then cont2 = eline+80

if not keyword_set(lmin) then lmin = eline-100
if not keyword_set(lmax) then lmax = eline+100

nn = where(lambda ge lmin and lambda le lmax)

lambda = lambda[nn]

fref = data[nn,id_ref]


; Data units (in case of units 10^-N)

meanf = min(fref)
smean = sctrim(meanf,format='(e30.15)')
full = strsplit(smean,'e',/extract,/regex)
expo = full[1]
com = 'etest = '+expo
res = execute(com)
if etest lt 0 then efactor = 10d^(etest*(-1)) else efactor = 1.

; Flux converted factor
fdata = data * efactor

fref = fdata[nn,id_ref]

normalize_cont, lambda, fref, cont1, cont2, fnorm, poly=2
fn_ref = fnorm


;------ Gaussian fit to emission line (old version)
;; yfit = gaussfit(lambda,fnorm,a,nterms=4)
;; inten=a[0] & clam=a[1] &  sig=a[2] & cont=a[3]
;; fn_ref = inten*exp(-(lambda-clam)^2/(2*sig^2))+cont



;--------------------------  Main loop

if not keyword_set(vmax) then vmax=500

vfield = fltarr(rows)
fluxOK = fltarr(pixels,rows)


print, ''
print, ' Performing cross-correlation...'
print, ''

for i=0, rows-1 do begin

    ; test spectrum
    ftest = fdata[nn,i]

    ; normalise
    normalize_cont, lambda, ftest, cont1, cont2, fn_test, poly=2

    ; cross correlation
    if not keyword_set(force_fit) then $
    shift = cross_correlate(lambda, fn_ref, lambda, fn_test, lmin, lmax) else $
    shift = cross_correlate(lambda, fn_ref, lambda, fn_test, lmin, lmax, /gauss)

    ; correct velocity field
    lam_shift = lam - shift
    fspaxel = data[*,i]
    fcross = interpol(fspaxel,lam,lam_shift)

    ; NaN and Infinites
    nan = where(fcross ne fcross)
    if n_elements(nan) gt 1 then fcross[nan] = 0.

    spax_vel = (abs(shift)/eline)*3e5

    ;if max(fcross) gt max(fspaxel*1.5) then goto, no_shift
    ;if min(fcross) lt min(fspaxel*0.5) then goto, no_shift
    if spax_vel gt vmax then goto, no_shift
    if shift ne shift then goto, no_shift

    vfield[i] = shift
    fluxOK[*,i] = fcross
    goto, shift_ok

    no_shift:
    vfield[i] = 0.
    fluxOK[*,i] = data[*,i]

    shift_ok:
    bar, i, rows-1

endfor

; NaN and Infinites
nan = where(vfield ne vfield)
if n_elements(nan) gt 1 then vfield[nan] = 0.


;--------------------------  Outputs

; FITS
out_fits = prefix+'_vfield.fits'


if type eq 2 then begin
   writefits, out_fits, fluxOK, hdr2
   out_pt = prefix+'_vfield.pt.txt'
   spawn, 'cp '+pos_table+' '+out_pt
endif


if type eq 3 then begin

   cube = fltarr(xx,yy,pixels)

   id = long(0)

   for j=0l, yy-1 do begin

      ran1 = j*xx
      ran2 = j*xx+(xx-1)

      array = fluxOK[*,ran1:ran2]

      array = transpose(array)

      cube[*,j,*] = array

   endfor

   writefits, out_fits, cube, hdr3

endif


;---------- Velocity map


; speed of light in km/s
c_vel = 299972

vel_km = vfield * (c_vel/eline)

index = where(vel_km eq vel_km)

mapv = fltarr(xx,yy)
mapv[index] = vel_km

out_map = prefix+'_vel.fits'
if type eq 3 then writefits, out_map, mapv, hdr2


; Plotting histogram

 vel = vel_km[where(vel_km ne 0)]
svel = stddev(vel)
mvel = mean(vel)
mm = where(vel ge mvel-svel*5 and vel le mvel+svel*5)
if n_elements(mm) eq 1 then mm = where(vel eq vel)
new_vel = vel[mm]

set_font
if keyword_set(ps) then pingsoft_pson, ps_file, pings.x_ps, pings.y_ps, font=psfont, axis_thick=pings.axis, encapsulate=1

pingsoft_hist, new_vel, /autobin, /fraction, /fill, /fline, forient=45, fcolor=100, $
               _extra={xtitle:'Velocity  (km/s)',ytitle:'Spectra fraction',title:'PINGSoft: velocity field'}


if keyword_set(ps) then pingsoft_psoff

if n_elem eq 1 then ref_id=id_ref else ref_id=[xid,yid]

if type eq 2 then mapv = 0

; Saving structure
out_str = { $
          newfits:   out_fits ,$
          ref_id:    ref_id   ,$
          ref_line:  eline    ,$
          vfield:    vfield   ,$
          mapv:      mapv     ,$
          eof:''$
          }

; Saving binary
out_sav = prefix+'_vfield.sav'
save, object, id_ref, eline, out_fits, vfield, filename=out_sav

if n_elem eq 1 then sref = sctrim(id_ref) else sref='['+sctrim(xid)+','+sctrim(yid)+']'
mvel = sctrim(mvel)
svel = sctrim(svel)

print, ''
print, ''
print, ' PINGSoft:  vfield_3D'
print, ' =============================='
print, ''
print, '              Reference spaxel:  '+sref
print, ''
print, '           Mean velocity shift:  '+mvel
print, ''
print, '          Sigma velocity shift:  '+svel
print, ''
print, ' Velocity field corrected FITS:  '+out_fits
print, ''
if type eq 3 then $
print, '             Velocity map FITS:  '+out_map
print, ''
print, '             Binary file saved:  '+out_sav
print, ''
if keyword_set(ps) then begin
print, '             Histogram PS file:  '+ps_file
print, ''
endif
if n_params() eq 4 then begin
print, ''
print, ' IDL structure saved:   '
print, ''
help, out_str, /struct
print, ''
endif

print, ''
if file_test('tmp.pt.txt') eq 1 then spawn, 'rm -f tmp.pt.txt'
if file_test('tmp_3D.pt.txt') eq 1 then spawn, 'rm -f tmp_3D.pt.txt'


end
