

pro view_3D, infile, out_str, PT=pt, EXTENSION=extension, $
             MIN_FLUX=min_flux, MAX_FLUX=max_flux, LMIN=lmin, LMAX=lmax, $
             FILTER=filter, BAND=band, CT=ct, DRAW=draw, PS=ps, FONT=font, $
             CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
             SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian, $
             VLINE=vline, NOBAND=noband, LARGE=large, _EXTRA=extra

;+
; NAME:
;       VIEW_3D
;
; PURPOSE:
;
;       Provides a 2D interactive visualisation of the spaxels and spectra of a
;       3D cube and/or an RSS file and its corresponding position table.
;
; EXPLANATION:
;       
;       The program displays two windows: on the right a visualisation of the
;       spatial distribution of spaxels or field-of-view (FoV). The
;       color-scaling corresponds to a narrow/broad-band image of a transmission
;       filter convolved with the data at a given wavelength. The spatial units
;       are assumed to be arcseconds in a standard North (up) East (left)
;       configuration. The spaxel shape can be square or circular (see below).
;       On the left, the window shows the spectrum of the spaxel corresponding
;       to the position of the mouse, the wavelength range is extracted from the
;       information on the FITS header, the spaxel ID shown on the top of the
;       window corresponds to the position of the spectrum in the case of a RSS
;       file (in the IDL format, i.e. starting at zero). The dotted-curve
;       corresponds to the narrow or broad-band used to draw the spaxels on the
;       right window, this band can be changed/shifted as explained below.
;
;       Additional information is shown on the IDL terminal: a left-click prints
;       the spaxel information ID, position, offset from the reference spaxel
;       (in arcsec), and if the WCS information is included in the FITS header
;       (see documentation), it shows the coordinates of the spaxel in
;       sexagesimal and degree units.
;
;       A middle-button click prompts for a PREFIX used to generate a new series
;       of files, all the subsequent left-clicks over the interactive window
;       will store the position and index of the selected spaxels which are
;       outlined in red. When the program is terminated (by a right-click) the
;       following files are created:
;
;          Extracted RSS:   PREFIX_rss.fits    (Extracted RSS of the selected spaxels)
;         Position table:   PREFIX_rss.pt.txt  (Position table of the new RSS file)
;       Integrated ASCII:   PREFIX_integ.txt   (Integrated spectrum in ASCII format)
;                   FITS:   PREFIX_integ.fits  (Integrated spectrum in FITS format)
;             Postscript:   PREFIX_integ.eps   (Postscript image of the integrated spectrum)
;            IDL indices:   PREFIX_index.txt   (Original indices of the selected spaxels)
;
;       and the left window shows the integrated spectrum of the selected
;       spaxels.
;
;       The script requires only the name of the 3D cube or the RSS file, in the
;       latter case, if the RSS file has the format OBJECT.fits, the name of the
;       position table should be in the format OBJECT.pt.txt for VIEW_3D to read
;       it automatically, otherwise the position table should be indicated using
;       the PT=pt option (see below).
;
;       Several options are available to better visualise the data, including
;       the central wavelength and type of narrow/broad band image, different
;       color tables and the display ranges in the spectral window. See the
;       documentation for more details.
;
;       
; CALLING SEQUENCE:
;       
;       view_3D, 'OBJECT.fits' [, OUT.str, PT='Ptable.txt', EXTENSION=extension, $
;                              MIN_FLUX=min_flux, MAX_FLUX=max_flux, LMIN=lam_min, LMAX=lam_max, $
;                              FILTER=filter, BAND=band, CT=ct, VLINE=vline, FONT=font, $
;                              /CLIP, /GAMMA, /LOG, /ASINH, /SQRT, /HISTOGRAM, /GAUSSIAN, $
;                              /PS, /DRAW, /LARGE, /NOBAND, _EXTRA={extra} ]
;                              
; INPUTS:
;       
;         'OBJECT.fits':  String of the wavelength calibrated 3D cube or RSS FITS file.
;
;
; OPTIONAL KEYWORDS:
;       
;               OUT.str:  Output IDL structure (when spaxels are manually selected).
;
;       PT='Ptable.txt':  Name of the position table in ASCII format for an input RSS file
;                         in (North-East configuration).
;                         NOTE: compulsory if not included in the default instruments/setups
;                         or when the name is not in the 'OBJECT.pt.txt' format.
;
;             EXTENSION:  Non-negative scalar integer specifying the FITS extension to read.
;                         For example, specify EXTENSION = 1 to read the first FITS extension.   
;
;          MIN/MAX_FLUX:  Minimum/maximum flux in the spectral window to be plot, if
;                         not set these are floating values.
;
;             LMIN/LMAX:  Defines the wavelength range on the spectral window, 
;                         if not set values are taken from the FITS header.
;                          
;                FILTER:  Internal number of the narrow or broad-band filter used to
;                         display the data. Available filters and corresponding
;                         numbers can be obtained by typing:  IDL> pingsoft_filters
;                         Default: 1 (Halpha KPNO-NOAO - CWL: 6547A  FWHM: 80)
;
;                  BAND:  Central wavelength of the narrow or broad-band used to
;                         display the data, i.e. shifts the band to the position
;                         defined by the user (if within the spectral range).
;                         Defaults: nominal central wavelength of the
;                         corresponding filter (mean wavelength if outside the range).
;
;                 VLINE:  Either a floating value or a vector of floating
;                         numbers containing the lambda value at which a single or
;                         several vertical lines will be drawn for reference
;                         purposes, equivalent to "Mark wavelength" in the VIEW_IFS widget,
;                         e.g. VLINE=6500 or VLINE=[4200,5400,6700].
;
;                    CT:  IDL Color Table used to display the data, (default ct=1, BLUE/WHITE).
;                     
;                   /PS:  Writes an encapsulated Postscript file of the spaxels visualisation.
;
;                  FONT:  Postscript IDL font to be used when /PS is set. Default: 12 (Helvetica)
;    
;                 /DRAW:  Draws the contours of the spaxels.
;
;                /LARGE:  Displays a larger window for the spatial distribution
;                         of spaxels (right window). 
;
;               /NOBAND:  The narrow/broad band is not drawn in the spectral window.
;
;                _EXTRA:  Structure with the _EXTRA tags for user's defined graphics output,
;                         e.g. _EXTRA={title:'IFS cube'}
;
;  Intensity Scalings:
;
;               Default:  LINEAR, displays the range of intensities using a linear min/max scaling.
;
;                 /CLIP:  A histogram stretch, with a 2% of pixels clipped at both the top and bottom.
;
;                /GAMMA:  Displays the range of intensities using a Power-law (gamma) scaling.
;
;                  /LOG:  Displays the range of intensities using a logarithmic scaling.
;
;                /ASINH:  Displays the range of intensities using an inverse hyperbolic sine function scaling.
;
;                 /SQRT:  Displays a linear stretch of the square root histogram of the image values.
;
;            /HISTOGRAM:  Displays a linear stretch of the histogram equalized image histogram.
;
;             /GAUSSIAN:  The scaling is performed by applying a Gaussian normal function to the image histogram.
;
;
; OUTPUTS:
;
;       Usually none, if the user applies the MIDDLE-button click option the
;       program generates a series of files as explained above.
;
;
; EXAMPLES:
;       
;       To visulise a wavelength calibrated 3D cube FITS file named 'OBJECT.fits'
;       (or a RSS FITS file with position table named 'OBJECT.pt.txt'):
;
;       view_3D, 'OBJECT.fits'
;
;
;       To visulise a wavelength calibrated RSS FITS file named 'OBJECT.fits'
;       with position table named 'PosTable.txt':
;
;       view_3D, 'OBJECT.fits', PT='PosTable.txt'
;
;
;       To draw the contours of the spaxels, set a rainbow colour table, draw two
;       vertical lines at lambda 4300 and 6500 and using a B-Johnson (1965) filter:
;
;       view_3D, 'OBJECT.fits', ct=39, vline=[4300,6500], filter=3, /draw
;
;
;       To limit the intensity and wavelength range on the spectral window and
;       use an inverse hyperbolic sine function scaling:
;
;       view_3D, 'OBJECT.fits',  min=-1, max=10, lmin=4500, lmax=6000, /asinh
;
;
; RESTRICTIONS:
;
;       3D cubes
;       The data should be wavelength calibrated and the spectral information in
;       the FITS header, namely the CRPIX3, CRVAL3 and CDELT3 values. 
;       
;       RSS files
;       The data should be wavelength calibrated and the spectral information 
;       included in the FITS header, namely the CRPIX1, CRVAL1 and CDELT1
;       values. The size and shape of the spaxels is determined by the first TWO
;       entries of the FIRST line of the position table, examples:
;
;       C  1.34    ('C'ircular spaxel, with a radius 1.34 arcsec, i.e. PPAK)
;       S  0.67    ('S'quare spaxel, with sides of length 0.67 arcsec, i.e. VIMOS)
;
;
; COMMENTS
;
;       The size, position, colours, etc. can be modified according to the
;       particular display by editing the files: 'PINGSOFT_PARAM.PRO' and
;       'PINGSOFT_WIN.PRO'
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful for your research, please acknowledge the
;       use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. (2011) NewAstron 16, 220
;
;
; COPYRIGHT:
;
;       This file is part of PINGSoft.
;
;       Copyright (C) 2010  F. Fabian Rosales-Ortega
;
;       PINGSoft is free software: you can redistribute it and/or modify it
;       under the terms of the GNU General Public License as published by the
;       Free Software Foundation, version 3. See the GNU General Public License
;       for more details.
;
;       PINGSoft is distributed in the hope that it will be useful, but WITHOUT
;       ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;       FITNESS FOR A PARTICULAR PURPOSE. 
;
;       The GNU General Public License is found in: http://www.gnu.org/licenses
;
;-


compile_opt idl2, hidden
on_error, 2

if n_params() lt 1 then begin
    print, ''
    print, ' CALLING SEQUENCE: '
    print, ''
    print, "   view_3D, 'OBJECT.fits' [, OUT.str, PT='PosTable.txt', EXTENSION=extension, "
    print, '                           MIN_FLUX=min_flux, MAX_FLUX=max_flux, LMIN=lam_min, LMAX=lam_max, '
    print, '                           FILTER=filter, BAND=band, VLINE=vline, CT=ct, FONT=font, '
    print, '                           /DRAW, /LARGE, /LINEAR, /GAMMA, /LOG, /ASINH, /PS, '
    print, '                           /NOBAND, _EXTRA={extra} ]'
    print, ''
    print, ''
    print, "    'OBJECT.fits':  String of the wavelength calibrated 3D cube or RSS FITS file."
    print, ''
    print, ''
    print, ' OPTIONAL KEYWORDS'
    print, ''
    print, "          OUT.str:  Name of the output structure when spaxels are manually selected."
    print, ''
    print, '        EXTENSION:  Non-negative scalar integer specifying the FITS extension to read.'
    print, '                    For example, specify EXTENSION = 1 to read the first FITS extension.'
    print, ''
    print, "  PT='Ptable.txt':  Name of the position table in the case of an RSS file"
    print, '                    in ASCII format (North-East configuration).'
    print, '                    NOTE: compulsory if not included in the default instruments/setups'
    print, "                    or when the name is not in the 'OBJECT.pt.txt' format."
    print, ''
    print, '     MIN/MAX_FLUX:  Minimum/maximum flux in the spectral window to be plot,'
    print, '                    if not set these are floating value.'
    print, ''
    print, '        LMIN/LMAX:  Defines wavelength range on the spectral window,'
    print, '                    if not set values are taken from the RSS header.'
    print, ''
    print, '           FILTER:  Internal number of narrow/broad band used to display the data.'
    print, '                    Available filters can be obtained by typing:  IDL> PINGSOFT_FILTERS'
    print, '                    Default: 1 (Halpha KPNO-NOAO / 6547A 80-FWHM).'
    print, ''
    print, '             BAND:  Central wavelength of the broad/narrow band used to display the data, '
    print, '                    defaults: BAND=6563 (i.e. Halpha) if within the spectral range, '
    print, '                    else BAND=mean(lambda).'
    print, ''
    print, '            VLINE:  Either a floating value or a vector of floating'
    print, '                    numbers containing the lambda value at which a single or'
    print, '                    several vertical lines will be drawn for reference purposes,'
    print, '                    e.g. VLINE=6500 or VLINE=[4200,5400,6700].'
    print, ''
    print, '               CT:  IDL Color Table used to display the data (default ct=1, BLUE/WHITE).'
    print, ''
    print, '             FONT:  Postscript IDL font to be used, default: 12 (Helvetica)'
    print, ''
    print, '            /DRAW:  Draws the contour of the spaxels.'
    print, ''
    print, '           /LARGE:  Displays a larger window for the spatial distribution of spaxels (right window). '
    print, ''
    print, '          /LINEAR:  Displays the range of intensities using a linear min/max scaling.'
    print ,''
    print, '           /GAMMA:  Displays the range of intensities using a power-law (gamma) scaling.'
    print, ''
    print, '             /LOG:  Displays the range of intensities using a logarithmic scaling.'
    print, ''
    print, '           /ASINH:  Displays the range of intensities using an inverse hyperbolic sine function scaling.'
    print, ''
    print, '              /PS:  Writes an encapsulated Postscript file of the spaxels visualisation.'
    print, ''
    print, '          /NOBAND:  The narrow/broad band is not drawn in the spectral window.'
    print, ''
    print, "           _EXTRA:  Structure with the _EXTRA tags for user's defined graphics output"
    print, ''
    return
endif


;--------------------------  Initial tests

infile = sctrim(infile)

if file_test(infile) eq 0 then begin
   print, ''
   print, 'ERROR: The file:  "'+infile+'"  does not exist!'
   print, ''
   return
endif

;  Strings
tmp0 = strmid(infile,strpos(infile,'/',/reverse_search)+1)
tmp1 = strsplit(tmp0,'.rscube.fits.gz',/extract,/regex)
tmp2 = strsplit(tmp1,'.rscube.fits',/extract,/regex)
tmp3 = strsplit(tmp2,'.rss.fits.gz',/extract,/regex)
tmp4 = strsplit(tmp3,'.rss.fits',/extract,/regex)
tmp5 = strsplit(tmp4,'.fits.gz',/extract,/regex)
tmp6 = strsplit(tmp5,'.fits',/extract,/regex)
object = tmp6



;--------------------------  Reading data

fits = readfits(infile,hdr,/silent)

author = fxpar(hdr,'AUTHOR')
author = string(author)

if keyword_set(extension) then begin
   if author eq 'CALIFA Collaboration' then $
      fits = readfits(infile,ext=extension,/silent) else $
      fits = readfits(infile,hdr,ext=extension,/silent)
endif

size = size(fits)
type = size[0]

;-------------------------- Testing for CALIFA data

califa = 0
ext_bpix = 0
if keyword_set(extension) then goto, ext_set
extend = fxpar(hdr,'EXTEND')

if author eq 'CALIFA Collaboration' then begin

   califa = 1

   ; CALIFA intensity map
   if not keyword_set(ct) then ct = 42

   ; CALIFA header without extensions
   if extend eq 0 then begin
      ext_bpix = -1
      goto, zero_ext
   endif

   ; Reading extensions
   if extend eq 1 then begin
      if type eq 2 then begin
         ext_bpix = readfits(infile,ext=2,/silent)
         ext_pt = readfits(infile,hdr_pt,ext=3,/silent)
      endif
      if type eq 3 then ext_bpix = readfits(infile,ext=3,/silent)
   endif

   zero_ext:
   ; Object type
   object = sctrim(fxpar(hdr,'OBJECT'))
   califaid = sctrim(fxpar(hdr,'CALIFAID'),format='(i4)')
   
endif

ext_set:


;--------------------------  Graphic device settings

ps_file = object+'_FoV.eps'
if califa eq 1 then title_str = 'CALIFA: '+object else title_str = object

pingsoft_param, pings, title_str
pingsoft_display, pings, /silent



;--------------------------  Data type

; Assuming data in arcsec
mas = 0

; RSS type
if type eq 2 then begin

   pixels = size[1]
   rows   = size[2]

   crval = fxpar(hdr,'CRVAL1')
   cdelt = fxpar(hdr,'CDELT1')
   crpix = fxpar(hdr,'CRPIX1')

   ref_ra  = fxpar(hdr,'RA')
   ref_dec = fxpar(hdr,'DEC')

   if crpix le 0 then begin
      print, ''
      print, 'WARNING: value of CRPIX1 in FITS header is 0 or negative, is this OK?'
      print, ''
   endif

   data = fits

   if keyword_set(califa) and ext_bpix[0] ne -1 then begin
      ; Badpix map
      bpix = ext_bpix
      ; Position table
      xpt = tbget(hdr_pt, ext_pt, 1)
      ypt = tbget(hdr_pt, ext_pt, 2)
      shape = fxpar(hdr_pt,'FIBSHAPE')
      pt_sizex = fxpar(hdr_pt,'FIBSIZEX')
      pt_sizey = fxpar(hdr_pt,'FIBSIZEy')
      header = shape+string(pt_sizex)+string(pt_sizey)+'  0'
      xpt = -xpt
      openw,1,'tmp_3D.pt.txt'
      printf,1,header
      for i=0,n_elements(xpt)-1 do printf,1, string(i+1), xpt[i], ypt[i], 0
      close,1
      pt = 'tmp_3D.pt.txt'
   endif


   ; FITS header for RSS data and 1D FITS

   hdr1 = hdr
   hdr3 = hdr
   
   sxdelpar, hdr1, 'CD1_1'
   sxdelpar, hdr1, 'CD2_2'
   sxdelpar, hdr1, 'CTYPE1'

   sxdelpar, hdr1, 'EXTEND'
   sxdelpar, hdr3, 'EXTEND'

   hdr2 = hdr

endif


; CUBE type
if type eq 3 then begin

   xx = size[1]
   yy = size[2]

   rows = xx*yy

   crval  = fxpar(hdr,'CRVAL3')
   cdelt  = fxpar(hdr,'CDELT3')
   crpix  = fxpar(hdr,'CRPIX3')
   pixels = fxpar(hdr,'NAXIS3')

   if crpix le 0 then begin
      print, ''
      print, 'WARNING: value of CRPIX3 in FITS header is 0 or negative, is this OK?'
     print, ''
   endif

   data = fltarr(pixels,rows)
   if keyword_set(califa) then bpix = fltarr(pixels,rows)

   ; New vector
   id = long(0)
   for j=0l, yy-1 do begin

      for i=0l, xx-1 do begin

         array = fits[i,j,*]
         array = transpose(array)

         ; NaN and Inf step 1
         mask = finite(array)
         array = array*mask
         nan = where(array ne array)
         if n_elements(nan) gt 1 then array[nan] = 0.
         if n_elements(nan) eq 1 and nan[0] ge 0 then array[nan] = 0.
         ; NaN and Inf step 2
         inf = where(array lt -1e10 or array gt 1e10)
         if n_elements(inf) gt 1 then array[inf] = 0.
         if n_elements(inf) eq 1 and inf[0] ge 0 then array[inf] = 0.
       
         data[*,id] = array
       
         if keyword_set(califa) and ext_bpix[0] ne -1 then begin
            array_bp = ext_bpix[i,j,*]
            array_bp = transpose(array_bp)
            bpix[*,id] = array_bp
         endif

         id = id+1

      endfor

      
   endfor

   cdelt1 = abs(fxpar(hdr,'CDELT1'))
   cdelt2 = abs(fxpar(hdr,'CDELT2'))
   crpix1 = fxpar(hdr,'CRPIX1')
   crpix2 = fxpar(hdr,'CRPIX2')
   naxis1 = fxpar(hdr,'NAXIS1')
   naxis2 = fxpar(hdr,'NAXIS2')

    RA = fxpar(hdr,'CRVAL1')
   Dec = fxpar(hdr,'CRVAL2')

   ref_ra  = RA
   ref_Dec = Dec

   ; Milli-arcseconds?
   if cdelt1 lt 1 or cdelt2 lt 1 then begin
      num = sctrim(cdelt1, format='(e)')
      exp = strsplit(num[0],'e',/extract,/regex)
      if exp[1] eq '-06' then begin
         cdelt1 = cdelt1 * 1e6
         cdelt2 = cdelt2 * 1e6
         mas = 1
      endif
   endif

   if cdelt1 eq 0 or cdelt2 eq 0 then resolution = 1 else resolution = cdelt1
   
   if crpix1 ne 0 then  RA_off = double(crpix1*resolution) else  RA_off = 0
   if crpix2 ne 0 then Dec_off = double(crpix2*resolution) else Dec_off = 0
   
   if  RA_off eq 0 then  RA_off = xx*resolution*0.5
   if Dec_off eq 0 then Dec_off = yy*resolution*0.5

   rectangular_pt, xx, yy, resolution, 1, xpt, ypt, off, $
                   X_OFF=RA_off, Y_OFF=Dec_off, OUT_PT='tmp_3D.pt.txt' ;, /PPAK

   pt = 'tmp_3D.pt.txt'

   ; FITS header for RSS data and 1D FITS

   hdr1 = hdr
   hdr3 = hdr

   ; hdr1
   fxaddpar, hdr1, 'CRVAL1', crval, ' updated by PINGSoft'
   fxaddpar, hdr1, 'CDELT1', cdelt, ' updated by PINGSoft'
   fxaddpar, hdr1, 'CRPIX1', crpix, ' updated by PINGSoft'
   fxaddpar, hdr1,     'RA',    RA, ' updated by PINGSoft'
   fxaddpar, hdr1,    'DEC',   Dec, ' updated by PINGSoft'
   sxdelpar, hdr1, 'CRVAL2'
   sxdelpar, hdr1, 'CRVAL3'
   sxdelpar, hdr1, 'CRPIX2'
   sxdelpar, hdr1, 'CRPIX3'
   sxdelpar, hdr1, 'CDELT2'
   sxdelpar, hdr1, 'CDELT3'
   sxdelpar, hdr1, 'DISPAXIS'
   sxdelpar, hdr1, 'WCSDIM'
   sxdelpar, hdr1, 'CTYPE1'
   sxdelpar, hdr1, 'CD1_1'
   sxdelpar, hdr1, 'CD2_2'
   sxdelpar, hdr1, 'CD1_2'
   sxdelpar, hdr1, 'CD2_1'

   sxdelpar, hdr1, 'EXTEND'
   sxdelpar, hdr3, 'EXTEND'

   hdr2 = hdr1

endif


if ref_ra le 0 then wcs = 0 else wcs = 1

read_wcs = offset2radec(ref_ra,ref_dec,0,0,/SEXAG) ; useless, just compiling functions

if cdelt eq 0 then begin
   print, ''
   print, ' WARNING: CDELT entry not found, using CDELT=1'
   print, ''
   cdelt = 1
endif

lambda = dblarr(pixels)
for i=0, pixels-1 do lambda[i] = (crval + i*cdelt)-(crpix-1)*cdelt

; Data in microns?
if mean(lambda) lt 100 then lambda = lambda*1e4

; Data in mas?
if mas eq 1 then wcs = 0

lam = lambda

if not keyword_set(lmin) then lmin = min(lambda)

if not keyword_set(lmax) then lmax = max(lambda)

indice = where(lambda ge lmin and lambda le lmax)

lambda = lambda[indice]

xmin = min(lambda) & xmax = max(lambda)


;--------------------------  Broad/Narrow band 

if crval ne 0. then begin

    ; Pick filter
    if not keyword_set(filter) then filter = pings.filter
    pingsoft_filters, filter, sband, lband, fband, cwl

    bad_cwl:

    ; Shift
    if not keyword_set(band) then band = cwl
    shift = cwl/band
    shift = shift[0]
    newl = lband/shift
    
    ; Interpolation with data
    res = interpol(fband,newl,lam)
    nn = where(lam lt min(newl) or lam gt max(newl))
    if n_elements(nn) gt 1 then res[nn] = 0.

    mm = where(lam ge min(newl) and lam le max(newl))
    if n_elements(mm) eq 1 and mm[0] lt 0 then begin
       print, ''
       print, 'WARNING: Using a V-band filter within the data wavelength limits: '+sctrim(lmin)+' - '+sctrim(lmax)
       print, ''
       filter = 4
       pingsoft_filters, filter, sband, lband, fband, cwl
       band = mean(lam)
       goto, bad_cwl
    endif

    ; convolution
    convolve = res[mm]
    dband = fltarr(rows)
    for k=0, rows-1 do dband[k] = total(convolve*data[mm,k])

    ; Negative values
    nband = where(dband lt 0)
    if nband[0] gt 0 then dband[nband]=0
    mband = min(dband)


    ;------ Default scaling (linear, clipping=1%)
    scale = pingsoft_scl( dband, /scale, stretch=1, clip=1 )

    ;------ Other scalings

    if keyword_set(clip) then $
       scale = pingsoft_scl( dband, /scale, stretch=2, minvalue=min(dband), maxvalue=max(dband) )
    if keyword_set(gamma) then $
       scale = pingsoft_scl( dband, /scale, stretch=3, minvalue=min(dband), maxvalue=max(dband) )
    if keyword_set(log) then $
       scale = pingsoft_scl( dband, /scale, stretch=4, minvalue=min(dband), maxvalue=max(dband) )
    if keyword_set(asinh) then $
       scale = pingsoft_scl( dband, /scale, stretch=5, minvalue=min(dband), maxvalue=max(dband), beta=30 )
     if keyword_set(sqrt) then $
        scale = pingsoft_scl( dband, /scale, stretch=6, minvalue=min(dband), maxvalue=max(dband) )
    if keyword_set(histogram) then $
       scale = pingsoft_scl( dband, /scale, stretch=7, minvalue=min(dband), maxvalue=max(dband) )
    if keyword_set(gaussian) then $
       scale = pingsoft_scl( dband, /scale, stretch=8, minvalue=min(dband), maxvalue=max(dband) )
    if keyword_set(stddev) then $
       scale = pingsoft_scl( dband, /scale, stretch=10, minvalue=min(dband), maxvalue=max(dband) )


    ; Borders
    nband = where(dband gt mband and scale le 5)
    if nband[0] gt 0 then scale[nband] = 5

 endif else begin
    print, ''
    print, 'ERROR: CRVAL1, CDELT1 and/or CRPIX1 not found in RSS header!'
    print, ''
    return
 endelse


;--------------------------  Position table

if keyword_set(pt) then begin

   pos_table = sctrim(pt)

   if file_test(pos_table) eq 0 then begin
      print, ''
      print, "ERROR: The file:  '"+pos_table+"'  does not exist!"
      print, ''
      return
   endif

endif


if not keyword_set(pt) then begin

   tmp0 = strsplit(infile,'.fits.gz',/extract,/regex)
   test_pt = strsplit(tmp0,'.fits',/extract,/regex)+'.pt.txt'

   if file_test(test_pt) eq 1 then pos_table = test_pt

   if file_test(test_pt) eq 0 then begin

      pos_table = identify_rss(rows,hdr) 

      if pos_table eq 'none' then begin
         print, ''
         print, "ERROR: PINGSoft cannot find/determine the associated position table"
         print, ''
         print, "       (is the instrument's resolution present in the FITS header?)"
         print, ''
         print, "       try setting the PT='pos_table.pt.txt' keyword"
         print, ''
         return
      endif
      
   endif

endif

pos_table = pos_table[0]


;-------------------------- Position table tests

; Mosaic type (reading first line of the position table)

readcol, pos_table, f='a,f,f,i', mtype, xsize, ysize, nflag, numline=1, /silent

if nflag eq 9 then draw=1

xsize = xsize[0]
ysize = ysize[0]

if mtype eq 'C' then begin
    points = 50
    fiber_size = xsize*2
    radius = fiber_size/2
    ang = 2*!PI*findgen(points+1.)/points
    xarr = radius*cos(ang)
    yarr = radius*sin(ang)
endif

if mtype eq 'S' then begin
    radius = xsize*0.5
    xarr = [-1,1,1,-1,-1]*radius
    yarr = [-1,-1,1,1,-1]*radius
endif


; Read position table

readcol, pos_table, f='l,f,f,f', id, x, y, flag, /silent

if n_elements(id) ne rows then begin
   print, ''
   print, 'ERROR: Number of rows in RSS file not equal to number of entries in the position table'
   print, ''
   print, '       Are you using the right position table?'
   print, ''
   print, '       PT file: '+pos_table
   print, ''
   return
endif



;--------------------------  Fibers in mosaic

!p.multi=[0,1,1]

loadct, 1, /silent

; Spaxels window limits
box_limits = pingsoft_boxlim(id,x,y,radius)
xmax_mosaic = box_limits[0]
xmin_mosaic = box_limits[1]
ymax_mosaic = box_limits[2]
ymin_mosaic = box_limits[3]

if keyword_set(ps) then begin
   
   if not keyword_set(font) then psfont=pings.font else psfont=font
   
   pingsoft_pson, ps_file, pings.x_ps, pings.y_ps, font=psfont, axis_thick=pings.axis, encapsulate=1
   
   if mas eq 0 then units = 'arcsec' else units = 'mas'

   plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
         xs=1, ys=1, charsize=pings.csize_ps, position=pings.pos0, $
         xtitle='!9D!X!e !nRA ('+units+')', ytitle='!9D!X!e !nDec ('+units+')', $
         title=title_str, _extra=extra
   goto, ps_on

endif


; Windows position and dimensions

  win_dim = pingsoft_win()

factor1=1.0 & factor2=1.0
if keyword_set(large) then begin
   factor1=1.5
   factor2=0.8
endif
xwin0_size = win_dim[0]*factor1
ywin0_size = win_dim[1]*factor1
xwin2_size = win_dim[2]*factor2
ywin2_size = win_dim[3]*factor2

set_font
window, 2, xs=xwin2_size, ys=ywin2_size
plot, [0], /nodata, xr=[xmin-80,xmax+80], yr=[0, 10], xs=1, ys=1, $
      xthick=pings.wthick, ythick=pings.wthick, position=pings.pos2, charsize=pings.csize, $
      xtitle=pings.xtitle2, ytitle=pings.ytitle2

if mas eq 0 then begin
   xtitle_w0 = pings.xtitle0
   ytitle_w0 = pings.ytitle0
endif
if mas eq 1 then begin
   xtitle_w0 = pings.xtitle3
   ytitle_w0 = pings.ytitle3
endif

window, 0, xs=xwin0_size, ys=ywin0_size
plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
      xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, position=pings.pos0, charsize=pings.csize, $
      xtitle=xtitle_w0, ytitle=ytitle_w0, $
      title=pings.title0, /noerase, _extra=extra

ps_on:


;--------- Color tables 

if not keyword_set(ct) then ct = 1

spawn, 'echo $PINGSOFT_PATH', path
ctfile = path+'/pingsoft_ct.tbl'

ctables = [4,5,25,27,33,39,41,42,43,44]
ntest = where(ctables eq ct)
if ntest[0] ge 0 then full_ct = 1 else full_ct = 0


;--------- Plot spaxels

for i=0l, n_elements(x)-1 do begin

    xcoor =  x[i]
    ycoor =  y[i]

    x_fib = xarr + xcoor
    y_fib = yarr + ycoor

    loadct, ct, file=ctfile, /silent
    polyfill, x_fib, y_fib, color=scale[i]

    loadct, 1, /silent

    if keyword_set(draw) then oplot, x_fib, y_fib, color=pings.dcolor1

    if keyword_set(ps) then begin
       loadct, ct, file=ctfile, /silent
       polyfill, x_fib, y_fib, color=scale[i]
       loadct, 1, /silent
       if keyword_set(draw) then oplot, x_fib, y_fib, color=pings.dcolor2
    endif

endfor

if keyword_set(ps) then begin
    pingsoft_psoff
    goto,fin
endif

print, ''
print, ''
print, ' PINGSoft: RSS and 3D cube spectra viewer'
print, ' =============================================='
print, ''
print, ' Filter:  '+sband
print, ''
print, ' Move mouse over the mosaic to plot the spectra'
print, ''
print, ' Options:'
print, ''
print, '   LEFT-click: spaxel information'
print, ''
print, ' MIDDLE-click: selects and stores spaxels by subsequent left-clicks'
print, ''
print, '  RIGHT-click: QUIT'
print, ''
print, ''

if type eq 2 then begin
   if wcs eq 0 then $
      print, '  Spaxel ID     [RA,Dec] offset' else $
      print, '  Spaxel ID     [RA,Dec] offset        RA              Dec           RA deg      Dec deg'
   print, ''
endif
if type eq 3 then begin
   if wcs eq 0 then $
      print, '    Spaxel       ID      [RA,Dec] offset' else $
      print, '    Spaxel       ID      [RA,Dec] offset         RA              Dec           RA deg      Dec deg'
   print, ''
endif

!MOUSE.button=0
write = 0
extract = long(0)

while (!MOUSE.button ne 4) do begin

    cursor, xm, ym, /change

    x_min = xm-radius
    x_max = xm+radius
    y_min = ym-radius
    y_max = ym+radius

    index = where(x ge x_min and x le x_max and y ge y_min and y le y_max)

    if n_elements(index) eq 1 then begin
        test = index[0]
        if test lt 0 then goto,next
    endif

    id_ok  = id[index]
    x_test = x[index]
    y_test = y[index]
    d = sqrt((x_test-xm)^2. + (y_test-ym)^2.)
    ind = where(d eq min(d))
    fibre = fix(id_ok[ind]-1,type=3)
    fibre = fibre[0]

    xcoor =  x[fibre]
    ycoor =  y[fibre]

    flux = data[*,fibre]
    if keyword_set(califa) and ext_bpix[0] ne -1 then fpix = bpix[*,fibre]

    if not keyword_set(min_flux) then a_flux = min(flux[indice]) else a_flux = min_flux

    if not keyword_set(max_flux) then y_flux = max(flux[indice])*1.2 else y_flux = max_flux

    wset, 2

    sfibre = sctrim(fibre)
    if type eq 2 then vtitle='!5Spaxel ID: '+sfibre+'!X'

    if type eq 3 then begin
       ncol = xx
       col = (fibre MOD ncol)+1
       row = (fibre / ncol)+1
       scol = sctrim(col)
       srow = sctrim(row)
       vtitle='!5Spaxel: ['+scol+','+srow+']    ID: '+sfibre+'!X'
    endif

    plot, [0], /nodata, xr=[xmin-80,xmax+80], yr=[a_flux,y_flux], xs=1, ys=1, $
          xthick=pings.wthick, ythick=pings.wthick, pos=pings.pos2, charsize=pings.csize, $
          title=vtitle, $
          xtitle=pings.xtitle2, ytitle=pings.ytitle2

    if keyword_set(noband) or pings.showband eq 0 then goto, noband
    loadct, 0, /silent
    oplot, lam, res*y_flux, linestyle=pings.bline, color=pings.bcolor
    loadct, 1, /silent
    noband:

    if keyword_set(vline) then begin
       if size(vline,/tname) eq 'STRING' then com=execute(vline)
       loadct, pings.ctline, /silent
       if n_elements(vline) gt 1 then for k=0, n_elements(vline)-1 do $
          plots, [vline[k],vline[k]],[a_flux,y_flux], color=pings.cline, thick=pings.tline $
       else $
          plots, [vline,vline],[a_flux,y_flux], color=pings.cline, thick=pings.tline
       loadct, 1, /silent
    endif

    oplot, lambda, flux[indice], color=pings.fcolor
    if keyword_set(califa) and ext_bpix[0] ne -1 then oplot, lambda, flux[indice]*fpix[indice], color=pings.bpcolor, min_value= 1e-6
    next:

    wset, 0
    plot, [0], /nodata, xr=[xmax_mosaic,xmin_mosaic], yr=[ymin_mosaic,ymax_mosaic], $
          xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, pos=pings.pos0, charsize=pings.csize, $
          xtitle=xtitle_w0, ytitle=ytitle_w0, $
          title=pings.title0, /noerase, _extra=extra


    if !MOUSE.button eq 2 then begin

       if write eq 1 then goto, unit_open
       print, ''    
       ;if keyword_set(prefix) then print, " Using as PREFIX:  '"+prefix+"'"
       prefix = ''
       read, prefix, prompt=' Enter PREFIX file name: ' 
       print, ''
       print, ' Storing all subsequent positions with a left-click'
       print, ''
       write = 1

       goto, unit_ok
       unit_open:
       print, ''
       print, 'Error: position file already open'
       print, ''
       goto, fin
       unit_ok:
    endif


    if !MOUSE.button eq 1 then begin

        if n_elements(index) eq 1 then begin
            test = index[0]
            if test lt 0 then goto, out_range
        endif

        if wcs eq 0 then wcs_coord1 = ''
        if wcs eq 0 then wcs_coord2 = ''

        if wcs eq 1 then begin

            coord1 = offset2radec( ref_ra, ref_dec, xcoor, ycoor, /SEXAG)

            if coord1[3] lt 0 or coord1[4] lt 0 or coord1[5] lt 0 then sign = '-' else sign=' '
            str_0 = string(coord1[0],format='(i02)')
            str_1 = string(coord1[1],format='(i02)')
            str_2 = string(coord1[2],format='(f04.1)')
            str_3 = sign+string(abs(coord1[3]),format='(i02)')
            str_4 = string(abs(coord1[4]),format='(i02)')
            str_5 = string(abs(coord1[5]),format='(f04.1)')
            wcs_coord1 = str_0+'h '+str_1+'m '+str_2+'s  '+str_3+'d '+str_4+'m '+str_5+'s'

            coord2 = offset2radec( ref_ra, ref_dec, xcoor, ycoor )
             ra_deg = coord2[0]
            dec_deg = coord2[1]
            wcs_coord2 = sctrim(ra_deg)+'   '+sctrim(dec_deg)

        endif

        s_fibre = string(fibre,format='(i6)')
        s_xcoor = string(xcoor,format='(f7.1)')
        s_ycoor = string(ycoor,format='(f7.1)')

        if type eq 2 then $
           print, '    '+s_fibre+'    '+s_xcoor+'  '+s_ycoor+'    '+wcs_coord1+'    '+wcs_coord2
        if type eq 3 then begin
           s_col = string(col,format='(i3)')
           s_row = string(row,format='(i3)')
           print, '  ['+s_col+','+s_row+']   '+s_fibre+'    '+s_xcoor+'  '+s_ycoor+'    '+wcs_coord1+'    '+wcs_coord2
        endif

        if full_ct eq 1 then thisCol = 255 else thisCol = 250 
        if write eq 1 then begin
            extract = [extract, fibre] 
            loadct, 39, /silent
            oplot, xarr+xcoor, yarr+ycoor, thick=2, color=thisCol
            polyfill, xarr+xcoor, yarr+ycoor, /line_fill, color=thisCol, orient=45, spacing=0.1
            loadct, 1, /silent
        endif
        out_range:
    endif


endwhile


if write eq 1 then begin

    extract = extract[1:*]

    ; ignoring repeated fibers
    new = long(0)
    for k=0l, n_elements(extract)-1 do begin
        test = extract[k]
        for n=k+1,  n_elements(extract)-1 do begin
            if test eq extract[n] then goto, incl
        endfor
        new = [new,test]
        incl:
    endfor

    new = fix(new[1:*],type=3)
    extract = new[sort(new)]
 
    integrated = fltarr(pixels)
    rss_data = fltarr(pixels,n_elements(extract))

    for i=0l, n_elements(extract)-1 do begin
        index_integ = fix(extract[i],type=3)
        integrated = integrated + data[*,index_integ]
        rss_data[*,i] = data[*,index_integ]
    endfor

    prefix = sctrim(prefix)

    ; Integrated flux ASCII
    out_integ = prefix+'_integ.txt'
    openw,3,out_integ
    for k=0l, n_elements(integrated)-1 do printf,3, k+1, lam[k], integrated[k]
    close, 3

    ; Integrated flux FITS
    out_fits = prefix+'_integ.fits'
    writefits, out_fits, integrated, hdr1

    ; RSS FITS file
    out_rss = prefix+'_rss.fits'
    writefits, out_rss, rss_data, hdr2

    ; New position table & indices
    out_pt = prefix+'_rss.pt.txt'
    out_index = prefix+'_index.txt'

    ; Saving structure
    out_str = { $
              newrss:  out_rss ,$
              lambda:  lam  ,$
              finteg:  integrated ,$
              indices: extract $
              }

    openw, 1, out_pt
    openw, 2, out_index

    if mtype eq 'C' then printf, 1, 'C  '+sctrim(xsize)+'  '+sctrim(xsize)+'  0'
    if mtype eq 'S' then printf, 1, 'S  '+sctrim(xsize)+'  '+sctrim(xsize)+'  0'
    
    for i=0l, n_elements(extract)-1 do begin
        nn = extract[i]
        printf, 1, sctrim(i+1), x[nn], y[nn], 1
        printf, 2, extract[i]
    endfor

    wset, 2
    plot, [0], /nodata, xr=[min(lam)-80,max(lam)+80], yr=[min(integrated), max(integrated)*1.2], $
          xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, pos=pings.pos2, charsize=pings.csize, $
          xtitle='Wavelength', ytitle='Flux', title='Integrated spectrum'
    loadct, 39, /silent
    oplot, lam, integrated, color=250

    ; PS extracted spectrum
    out_ps =  prefix+'_integ.eps'
    pingsoft_pson, out_ps, 20, 16, font=psfont, axis_thick=pings.axis, encapsulate=1
    loadct, 1, /silent
    plot, [0], /nodata, xr=[min(lam)-80,max(lam)+80], yr=[min(integrated), max(integrated)*1.2], $
          xs=1, ys=1, xthick=pings.wthick, ythick=pings.wthick, pos=pings.pos2, charsize=pings.csize*1.2, $
          xtitle='Wavelength', ytitle='Flux', title='Integrated spectrum'
    oplot, lam, integrated, color=100
    pingsoft_psoff


    print, ''
    print, ' PINGSoft'
    print, ' ==================='
    print, ''
    print, ' Files written '
    print, ''
    print, '      Extracted RSS:   ', out_rss
    print, '     Position table:   ', out_pt
    print, '   Integrated ASCII:   ', out_integ
    print, '               FITS:   ', out_fits
    print, '         Postscript:   ', out_ps
    print, '        IDL indices:   ', out_index

    if n_params() eq 2 then begin
       print, ''
       print, ' IDL structure saved:   '
       print, ''
       help, out_str, /struct
       print, ''
    endif

 endif

fin:

if keyword_set(ps) then begin
    print, ''
    print, ' File written: ', ps_file
    print, ''
endif

close, /all
print, ''

if file_test('tmp_3D.pt.txt') eq 1 then spawn, 'rm -f tmp_3D.pt.txt'

end
