


;+
; NAME:
;       VORONOI_3D
;
; PURPOSE:
;
;       Applies the Voronoi tessallation method to bin the IFS data to a
;       target signal-to-noise ratio per bin.
;
;
; EXPLANATION:
;
;       This program is a wrapper of the VORONOI_2D_BINNING code by 
;       Cappellari & Copin (2003, MNRAS, 342, 345) which performs an adaptive
;       spatial binning of IFS data. The Voronoi Binning method optimally solves
;       the problem of preserving the maximum spatial resolution of general
;       two-dimensional data, given a constraint on the minimum S/N ratio.
;       In order to work properly, this routine needs the VORONOI_2D_BINNING code
;       to be in the $IDL_PATH. The source code can be found at:
;
;       http://www-astro.physics.ox.ac.uk/~mxc/idl/#binning
;
;       In addition, this program MUST BE used with the output of S2N_RATIO_3D
;       (or use a similar input format).
;
;       The VORONOI_2D_BINNING code requires as an input the spatial position
;       (x,y), the signal and noise (as separate values) in order to perform
;       the bining, note that the code does not care what is the proxy used to
;       calculate the "signal" or "noise". The signal and noise can be
;       calculated by S2N_RATIO_3D for all the spaxels within a IFS data file,
;       and the information can be passed as input for this Voronoi binning
;       script (either considering all the spaxels or only those above a certain
;       threshold, using any of the available S/N "proxies"). 
;       In the case of S2N_RATIO_3D, the outputs used by VORONOI_3D can be 
;       either the "S/N table ASCII" (passed through the ASCII_S2N keyword) or
;       the output IDL structure (passed through the STR_S2N keyword), either for the
;       continuum or the emission-line case (see the S2N_RATIO_3D documentation).
;
;       The most important input parameter for the Voronoi binning is the
;       "target S/N", i.e. the minimum required S/N that each bin must posses
;       after the tessellation. Therefore, the main VORONOI_3D inputs are:
;       1) the input IFS data file, 2) the S/N information, and 3) the target S/N. 
;       The program then invokes the VORONOI_2D_BINNING script, prints the
;       binning/iterations information on the terminal and displays a
;       window which shows on the top-panel the FoV of the spatial binning and
;       on the bottom the S/N-per-bin as a function of radius. At this point, the 
;       S/N-binning target can be changed interactively or the user can proceed
;       with the extraction. In the latter case, the script writes a set of
;       output files and displays on the screen the FoV of the input IFS data
;       overlaid with the binning tessellation (where all visual options of
;       VIEW_3D can be applied). The output files include: a) a RSS file with
;       the integrated spectra of each Voronoi bin (the coordinates of the
;       associated Pseudo-Position table correspond to the centre-of-mass of the
;       bin); b) a 2D FITS segmentation map, corresponding to the Voronoi bins;
;       and 3) a full segmented 3D cube. An optional Postscript file of the FoV
;       tessallation can also be included.
;
;
; For CALLING SEQUENCE, INPUTS and OPTIONAL KEYWORDS enter:
;
;       IDL> voronoi_3d
;
;
; OUTPUTS:
;
;       If PREFIX is set: 
;
;                 Segmentation map:  PREFIX_voronoi.seg.fits
;         Extracted spectra in RSS:  PREFIX_voronoi.rss.fits
;            Pseudo-position table:  PREFIX_voronoi.rss.pt.txt
;           Full segmented 3D cube:  PREFIX_voronoi.cube.fits
;         Postscript segmented FoV:  PREFIX_voronoi.FoV.eps      (only if /PS set)
;
;       If PREFIX is not set, PREFIX=OBJECT 
;
; COMMENTS
;
;       DO NOT TRUST the S/N-per-bin reported by the VORONOI_2D_BINNING code,
;       several tests show that the "real" S/N goes as the signal-to-ratio value
;       reported by the code per bin, divided by the squared-root of the 
;       number of pixels in the bin: S/N = (S/N_Capellari_bin) / sqrt(N_pixels_bin).
;       The best strategy is to recalculate the S/N of the output Voronoi-binned
;       spectra (e.g. using S2N_RATIO_3D), update the target S/N in VORONOI_3D and 
;       proceed iteratively until the "real" S/N-per-bin corresponds to
;       the user's target S/N (see the PINGSoft documentation).
;
;
; REVISION HISTORY:
;
;        Written by:   F. Fabian Rosales-Ortega
;
;        Version 1:  January, 2010
;                2:  November, 2012
;      
; NOTE:
;
;       This routine belongs to the PINGSoft IDL Integral Field Spectroscopy
;       software available at:
;
;       http://califa.caha.es/pingsoft
;
;       where additional documentation can be found. 
;
;       If you find this code useful on your research, 
;       please acknowledge the use of PINGSoft in your publications:
;
;       Rosales-Ortega, F. NewAstron (2011) 16, 220
;
;-


pro voronoi_3d, infile, s2n_bin, STR_S2N=str_s2n, ASCII_S2N=ascii_s2n, $
                    NO_CVT=no_cvt, PIXSIZE=pixelsize, WVT=wvt, $ ; voronoi_2d_binning.pro options
                    NORMALIZE=normalize, SEGMENT=segment, PT=pt, EXTENSION=extension, PS=ps, PREFIX=prefix, $
                    FILTER=filter, BAND=band, CT=ct, DRAW=draw, FONT=font, $
                    LINEAR=linear, GAMMA=gamma, LOG=log, ASINH=asinh, _EXTRA=extra



compile_opt idl2, hidden
on_error, 2

if n_params() lt 2 then begin
   print, ''
   print, "  voronoi_3D, 'OBJECT.fits', S2N_target, ( ASCII_S2N='ascii_s2n' -OR- STR_S2N=str_s2n), "
   print, "               [ /NORMALIZE, /SEGMENT, /PS, PREFIX=prefix, EXTENSION=extension, PT='PosTable.txt', "
   print, '               /NO_CVT, /WVT, PIXSIZE=pixelsize, (voronoi_2d_binning.pro options)'
   print, '               (+ all display options as in VIEW_3D) ]'
   print, ''
   print, ''
   print, "     'OBJECT.fits':  String of the wavelength calibrated 3D cube or RSS FITS file."
   print, ''
   print, '        S2N_target:  Minimum required S/N that each bin must posses after the tessellation'
   print, ''
   print, ' and EITHER:'
   print, ''
   print, '         ASCII_S2N:  String with the name of the "S/N table ASCII" output of S2N_RATIO_3D'
   print, '                     (or a table with a similar format) containing the spatial position (x,y),'
   print, '                     the signal and noise (as separate values) in order to perform the bining.'
   print, ' OR'
   print, ''
   print, '           STR_S2N:  Variable of the output IDL structure of S2N_RATIO_3D with the same input'
   print, '                     information as described above.'
   print, ''
   print, ''
   print, ' OPTIONAL KEYWORDS'
   print, ''
   print, '        /NORMALIZE:  If set, the integrated spectrum is normalized by the number'
   print, '                     of spaxels within the Voronoi bin.'
   print, ''
   print, '          /SEGMENT:  If set, the on-screen or Postscript visualisation of the'
   print, '                     segmented regions uses a special, nicer color-coding'
   print, ''
   print, '               /PS:  Writes an encapsulated Postscript file of the segmented regions'
   print, ''
   print, ''
   print, ' VORONOI_2D_BINNING options:'
   print, ''
   print, '           /NO_CVT:  Set this keyword to skip the Centroidal Voronoi Tessellation'
   print, '                     (CVT) step (vii) of the algorithm in Section 5.1 of Cappellari & Copin (2003).'
   print, ''
   print, '              /WVT:  When this keyword is set, the routine bin2d_cvt_equal_mass is'
   print, '                     modified as proposed by Diehl & Statler (2006, MNRAS, 368, 497).'
   print, ''
   print, '           PIXSIZE:  Optional pixel scale of the input data.'
   print, ''
   return
endif


;--------------------------  Initial tests

infile = sctrim(infile)

if file_test(infile) eq 0 then begin
   print, ''
   print, 'ERROR: The file:  "'+infile+'"  does not exist!'
   print, ''
   return
endif


if not keyword_set(str_s2n) and not keyword_set(ascii_s2n) then begin
   print, ''
   print, ' Missing output of S2N_RATIO_3D, please provide either:'
   print, ''
   print, '   1. An IDL structure: STR_S2N = structure_name'
   print, "   2. An ASCII table: ASCII_S2N = 'table_name.txt' "
   print, ''
   return
endif


;--------------------------  Graphic device settings
pingsoft_param, pings, ''
pingsoft_display, pings, /silent


;--------------------------  PINGSoft core

if keyword_set(ps) then tmp_ps = 1 else tmp_ps = 0
this_ps = 0

pingsoft_core, infile, pings, core, ps_file, x0, y0,  $
               PT=pt, PREFIX=prefix, EXTENSION=extension, $
               MIN_FLUX=min_flux, MAX_FLUX=max_flux, LMIN=lmin, LMAX=lmax, $
               FILTER=filter, BAND=band, CT=ct, DRAW=draw, PS=0, FONT=font, PLOT=plot, $
               CLIP=clip, GAMMA=gamma, LOG=log, ASINH=asinh,  $
               SQRT=sqrt, HISTOGRAM=histogram, GAUSSIAN=gaussian,  $
               EFF_RAD=eff_rad, NOEXTRACT=noextract, _EXTRA=extra

; Variables from structure
tag = get_tags(core)
for k=0, n_elements(tag)-1 do begin
   dumb = strmid(tag[k],1)
   com = execute(dumb+' = core.'+dumb)
endfor
ps = tmp_ps

if not keyword_set(prefix) then prefix=object[0]

;--------------------------  Data test

if type ne 3 then begin
     print, ''
     print, ' ERROR: The input file is NOT a 3D cube!! '
     print, ''
     return
endif

xcase = 0

; Inputs

if keyword_set(str_s2n) then begin
   str_s2n = s
   index = s.cont_index
   x = s.x[index]
   y = s.y[index]
   signal = s.cont_signal[index]
   noise = s.cont_noise[index]
   goto, vor_cal
endif

if keyword_set(ascii_s2n) then begin
   if file_test(ascii_s2n) eq 0 then begin
      print, ''
      print, 'ERROR: The file:  "'+ascii_s2n+'"  does not exist!'
      print, ''
      return
   endif
   readcol, ascii_s2n, f='f,f,f,f,f,l', x, y, signal, noise, s2n_rat, index, /silent
   goto, vor_cal
endif

vor_cal:


; window
;if keyword_set(ps) then pingsoft_psoff
dim = get_screen_size(resolution=resolution)
xdim = dim[0]
ydim = dim[1]

window, 0, xs=xdim*0.4, ys=ydim*0.8, title='Voronoi binning:  Cappellari & Copin (2003) MNRAS, 342, 345'



;-------------------------- Voronoi binning  (Cappellari & Copin, 2003, MNRAS, 342, 345)

; RA correction
if xcase eq 0 then x = -x

if keyword_set(NO_CVT) then no_cvt= 1
if keyword_set(WVT) then wvt = 1
if keyword_set(PIXSIZE) then pixelsize=pixelsize

voronoi_2d_binning, x, y, signal, noise, s2n_bin, bin, xbin, ybin, xbar, ybar, snbin, npixels, $
                    NO_CVT=no_cvt, PIXSIZE=pixelsize, WVT=wvt, /plot


string_s2n = sctrim(s2n_bin)

; Feedback
val = ''
print, ''
print,            ' Change S/N binning threshold ? (current: '+string_s2n+')'
print, ''
print,            '                        (N)o, proceed to extraction'
print,            '                        (y)es'
read, val, prompt='                        (a)bort                      '

if val eq 'a' or val eq 'A' then begin
   print, ''
   print, ' Returning...'
   print, ''
   return
endif

if val eq 'y' or val eq 'Y' then begin
   print, ''
   print, ''
   read, s2n_bin, prompt=' New S/N target: '
   xcase = 1
   goto, vor_cal
endif

sn_end = '_sn'+sctrim(s2n_bin)
sn_end = ''

;------------------------  Extraction

print, ''
print, ' Extracting...'


; Segmentation map

xdim = xx
ydim = yy
map = fltarr(xdim,ydim)

map[index] = bin + 1

nbin = sctrim(max(bin+1))

; output
out_mask = prefix+'_voronoi'+sn_end+'.seg.fits'
writefits, out_mask, map


; Create new cube
extract_mask, infile, out_mask, PT=pt, EXTENSION=extension, /SILENT, $
              NORMALIZE=normalize, SEGMENT=segment, PS=ps, PREFIX=prefix, $
              FILTER=filter, BAND=band, CT=ct, DRAW=draw, FONT=font, $
              LINEAR=linear, GAMMA=gamma, LOG=log, ASINH=asinh, _EXTRA=extra
   

; Outputs

 out_rss = prefix+'_voronoi'+sn_end+'.rss.fits'
out_cube = prefix+'_voronoi'+sn_end+'.cube.fits'
  out_ps = prefix+'_voronoi'+sn_end+'.FoV.eps'

spawn, 'mv '+prefix+'_mask.rss.fits  '+out_rss
spawn, 'mv '+prefix+'_mask.cube.fits '+out_cube
if keyword_set(ps) then spawn,  'mv '+prefix+'_mask.FoV.eps    '+out_ps

spawn, 'rm -f '+prefix+'_mask.rss.pt.txt'


; Pseudo Position Table

snbin = round(snbin)

out_pt = prefix+'_voronoi'+sn_end+'.rss.pt.txt'
openw, 1, out_pt

printf, 1, 'S  1.0  1.0  0'
printf, 1, ''
printf, 1, '# PINGSoft: Pseudo-position table for Voronoi extraction'
printf, 1, '# ------------------------------------------------------'
printf, 1, '#'
printf, 1, '# N-bin, X-bin, Y-bin, S/N bin, N-pixels per bin'
printf, 1, '#'
printf, 1, ''
for i=0l, n_elements(xbin)-1 do printf, 1, i+1, -xbin[i], ybin[i], snbin[i], npixels[i]
close, 1



; Screen output

print, ''
print, ''
print, ' PINGSoft:  Voronoi_3D'
print, ' ==========================='
print, ''
print, ' Number of bins: '+nbin
print, ''
print, '  Files written: '
print, ''
print, '           Segmentation map:  '+out_mask
print, '   Extracted spectra in RSS:  '+out_rss
print, '      Pseudo-position table:  '+out_pt
print, '     Full segmented 3D cube:  '+out_cube
if keyword_set(ps) then begin
print, '   Postscript segmented FoV:  '+prefix+'_voronoi'+sn_end+'.FoV.eps'
endif
print, ''
print, ' NOTE: This script makes use of the Voronoi binning method by: '
print, ''
print, '       Cappellari & Copin (2003) MNRAS, 342, 345'
print, ''

if file_test('tmp.pt.txt') eq 1 then spawn, 'rm -f tmp.pt.txt'
if file_test('tmp_3D.pt.txt') eq 1 then spawn, 'rm -f tmp_3D.pt.txt'

end
